/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef BOUNDARY_STRATEGY_H
#define BOUNDARY_STRATEGY_H

#include "Boundary.h"
#include "Algorithm.h"
// #include <CompuCell3D/Field3D/Field3DImpl.h>

#include <CompuCell3D/Field3D/Dim3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Field3D/Neighbor.h>

#include <vector>

using namespace std;


namespace CompuCell3D {

/*
 * Implements the singleton for Boundary strategies
 * Each axis is assigned its own boundary strategy
 */
   //need to include it to avoid problems with "inclomplete types"
/*   template <typename Y>
   class Field3DImpl;*/
   

    class BoundaryStrategy {

          private:

               static BoundaryStrategy *singleton;

               Dim3D dim;
               int currentStep;
               bool regular;
               Boundary *strategy_x;
               Boundary *strategy_y;
               Boundary *strategy_z;

               Algorithm* algorithm;
               std::vector<std::vector<std::vector<unsigned char> > > checkField;

               BoundaryStrategy();
               BoundaryStrategy(string boundary_x, string boundary_y,
                             string boundary_z, string alg, int index, int size, string inputfile) ;

               
               bool isValid(const int coordinate, const int max_value);
               
               std::vector<Point3D> offsetVec;
               std::vector<float> distanceVec;
               std::vector<unsigned int> neighborOrderIndexVec;
               bool checkIfOffsetAlreadyStacked(Point3D &, std::vector<Point3D> &);
               bool checkEuclidianDistance(Point3D &,Point3D & , float);
               void initializeQuickCheckField(Dim3D );
               float maxDistance;
               bool neighborListsInitializedFlag;


          public:
               
               int getNumPixels(int x, int y, int z) const;
               bool isValid(const Point3D &pt) const;
               bool isValidDirect(const Point3D &pt) const;
               void setIrregular();
               void setDim(const Dim3D theDim);
               void setCurrentStep(const int currentStep);
               Point3D getNeighbor(const Point3D& pt, unsigned int& token,
                                   double& distance, bool checkBounds = true);

               const std::vector<Point3D> & getOffsetVec() const {return offsetVec;}
               const std::vector<float> & getDistanceVec() const {return distanceVec;}
               const std::vector<unsigned int> & getNeighborOrderIndexVec() const {return neighborOrderIndexVec;}
               Neighbor getNeighborDirect(Point3D & pt,unsigned int idx ,bool checkBounds=true) ;
               float getMaxDistance(){return maxDistance;}
               unsigned int getMaxNeighborIndexFromDepth(float depth);

               ~BoundaryStrategy();
   
               /*
                * Instantiates a singleton, if not done already.
                *
                * @param boundary_x boundary condition for x axis
                * @param boundary_y boundary condition for y axis
                * @param boundary_z boundary condition for z axis
                *
                **/            
               static void instantiate(string boundary_x, string boundary_y,
                                       string boundary_z, string alg,
                                       int index, int size, string inputfile) {

                     
                         if(!singleton) {

                             singleton = new BoundaryStrategy(boundary_x, boundary_y,
                                                          boundary_z, alg, index, size, inputfile);

                             
                         }
                      
               }
               //prepares list of offsets for neighbors
               void prepareNeighborLists(float _maxDistance=4.0);
               
               /*
                * Return the instance of Boundary Strategy
                *
                * @return BoundaryStrategy singleton instance
                */
               static BoundaryStrategy *getInstance() {

                   if(!singleton) {
                    
                       singleton = new BoundaryStrategy();
                  
                   }
                  
                   return singleton;

               }
               
               /*
                * Destroy the singleton 
                *
                */
               static void destroy() {

                  if(singleton) delete singleton;

               }              
               

    }; 


};

#endif
