#include "EnergyFunctionCalculatorStatistics.h"
#include "EnergyFunction.h"
#include <iostream>
#include <iomanip>
#include <sstream>
#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <sstream>




using namespace CompuCell3D;
using namespace std;

EnergyFunctionCalculatorStatistics::EnergyFunctionCalculatorStatistics():EnergyFunctionCalculator(){

   NTot=0;
   NAcc=0;
   NRej=0;
   lastFlipAttempt=-1;
   out=0;
   outAccSpinFlip=0;
   outRejSpinFlip=0;
   outTotSpinFlip=0;

   wroteHeader=false;
   outputEverySpinFlip=false;
   gatherResultsSpinFlip=false;
   outputAcceptedSpinFlip=false;
   outputRejectedSpinFlip=false;
   outputTotalSpinFlip=false;

   gatherResultsFilesPrepared=false;

//    outputAcceptedSpinFlipHeader=false;
//    outputRejectedSpinFlipHeader=false;
//    outputTotalSpinFlipHeader=false;


   mcs=0;
   fieldWidth=30;
   analysisFrequency=1;
   singleSpinFrequency=1;
}

EnergyFunctionCalculatorStatistics::~EnergyFunctionCalculatorStatistics(){
   if(out){
      out->close();
      delete out;
      out=0;

   }

   if(outAccSpinFlip){
      outAccSpinFlip->close();
      delete outAccSpinFlip;
      outAccSpinFlip=0;
   }

   if(outRejSpinFlip){
      outRejSpinFlip->close();
      delete outRejSpinFlip;
      outRejSpinFlip=0;
   }

   if(outTotSpinFlip){
      outTotSpinFlip->close();
      delete outTotSpinFlip;
      outTotSpinFlip=0;
   }

}


double EnergyFunctionCalculatorStatistics::changeEnergy(Point3D &pt, const CellG *newCell,const CellG *oldCell,const unsigned int _flipAttempt){

  if(lastFlipAttempt<0){
      initialize();
   }

   if(_flipAttempt<lastFlipAttempt){
      ++mcs;
      if(! (mcs % analysisFrequency)){
         outputResults();
      }
      if(!gatherResultsSpinFlip && outputEverySpinFlip && ! (mcs % singleSpinFrequency) ){
         outputResultsSingleSpinFlip();
      }

      if(gatherResultsSpinFlip && outputEverySpinFlip && ! (mcs % singleSpinFrequency) ){
         if(!gatherResultsFilesPrepared){
            prepareGatherResultsFiles();
         }
         outputResultsSingleSpinFlipGatherResults();
      }

      prepareNextStep();
   }

  lastFlipAttempt=_flipAttempt;

  double change = 0;
  for (unsigned int i = 0; i < energyFunctions.size(); i++){
    lastEnergyVec[i]=energyFunctions[i]->changeEnergy(pt, newCell, oldCell);
    change+=lastEnergyVec[i];

  }
   totEnergyDataList.push_back(lastEnergyVec); //inserting lastEnergyVec into  totEnergyVecVec - for later stdDev calculations

  return change;

}

//this function will inccrement appropriate energy vectors based on whether flip was accepted or not
void EnergyFunctionCalculatorStatistics::setLastFlipAccepted(bool _accept){
  lastFlipAccepted=_accept;

   if (lastFlipAccepted){
      accNotAccList.push_back(true);
      

   }else{
      accNotAccList.push_back(false);      
      

   }


}
void EnergyFunctionCalculatorStatistics::writeHeader(){

   (*out)<<setw(fieldWidth)<<"1.STEP"<<setw(fieldWidth)<<"2.NAcc"<<setw(fieldWidth)<<"3.NRej"<<setw(fieldWidth)<<"4.NTot";
   int n=5;
   for (int i = 0 ; i < energyFunctions.size() ; ++i){
      ostringstream str1;
      ostringstream str2;
      str1<<n<<".Acc_"<<energyFunctionsNameVec[i]<<"_AVG";
      str2<<n+1<<". +/-";
      (*out)<<setw(fieldWidth)<<str1.str()<<setw(fieldWidth)<<str2.str();
      n+=2;
   }

   for (int i = 0 ; i < energyFunctions.size() ; ++i){
      ostringstream str1;
      ostringstream str2;
      str1<<n<<".Rej_"<<energyFunctionsNameVec[i]<<"_AVG";
      str2<<n+1<<". +/-";
      (*out)<<setw(fieldWidth)<<str1.str()<<setw(fieldWidth)<<str2.str();
      n+=2;
   }

   for (int i = 0 ; i < energyFunctions.size() ; ++i){
      ostringstream str1;
      ostringstream str2;
      str1<<n<<".Tot_"<<energyFunctionsNameVec[i]<<"_AVG";
      str2<<n+1<<". +/-";
      (*out)<<setw(fieldWidth)<<str1.str()<<setw(fieldWidth)<<str2.str();
      n+=2;
   }



   (*out)<<endl;
}



void EnergyFunctionCalculatorStatistics::initialize(){
   if(!wroteHeader && out){
      writeHeader();
      wroteHeader=true;
   }   
   NAcc=0;
   NRej=0;
   NTot=0;
   
   int energyFunctionsVecSize=energyFunctions.size();

   lastEnergyVec.assign(energyFunctionsVecSize,0.0);
   stdDevEnergyVectorTot.assign(energyFunctionsVecSize,0.0);
   stdDevEnergyVectorAcc.assign(energyFunctionsVecSize,0.0);
   stdDevEnergyVectorRej.assign(energyFunctionsVecSize,0.0);


   //allocating vectors needed for std_dev calculations
   

   totEnergyDataList.clear();
   accNotAccList.clear();
   

}

void EnergyFunctionCalculatorStatistics::prepareNextStep(){

   initialize();

}


void EnergyFunctionCalculatorStatistics::calculateStatData(){
   unsigned int energyFunctionCount=energyFunctions.size();

   avgEnergyVectorTot.assign(energyFunctionCount,0.0);
   stdDevEnergyVectorTot.assign(energyFunctionCount,0.0);

   avgEnergyVectorAcc.assign(energyFunctionCount,0.0);
   stdDevEnergyVectorAcc.assign(energyFunctionCount,0.0);

   avgEnergyVectorRej.assign(energyFunctionCount,0.0);
   stdDevEnergyVectorRej.assign(energyFunctionCount,0.0);



   
   //calculating averages
   std::list<std::vector<double> >::iterator lVecItr;
   std::list<bool>::iterator lItr;

   NTot=0;
   NAcc=0;
   NRej=0;

   //calculating averages
   lItr=accNotAccList.begin();
   for(lVecItr=totEnergyDataList.begin() ; lVecItr != totEnergyDataList.end() ; ++lVecItr){
      ++NTot;
      vector<double> & energyData=*lVecItr;
      for(int i = 0 ; i < energyFunctionCount ; ++i){
         avgEnergyVectorTot[i]+=energyData[i];
      }
      if(*lItr){//accepted flip
         ++NAcc;
         for(int i = 0 ; i < energyFunctionCount ; ++i){
            avgEnergyVectorAcc[i]+=energyData[i];
         }
      }else{//rejected flip
         ++NRej;
         for(int i = 0 ; i < energyFunctionCount ; ++i){
            avgEnergyVectorRej[i]+=energyData[i];
         }
      }
      ++lItr;
   }
      
   for(int i = 0 ; i < energyFunctions.size() ; ++i){
      if(NTot)
         avgEnergyVectorTot[i]/=NTot;
      if(NAcc)
         avgEnergyVectorAcc[i]/=NAcc;
      if(NRej)
         avgEnergyVectorRej[i]/=NRej;
   }

   //calculating stdDev
   stdDevEnergyVectorTot.assign(energyFunctionCount,0.);
   stdDevEnergyVectorAcc.assign(energyFunctionCount,0.);
   stdDevEnergyVectorRej.assign(energyFunctionCount,0.);

   lItr=accNotAccList.begin();
   for(lVecItr=totEnergyDataList.begin() ; lVecItr != totEnergyDataList.end() ; ++lVecItr){
      vector<double> & energyData=*lVecItr;
      for(int i = 0 ; i < energyFunctionCount ; ++i){
         stdDevEnergyVectorTot[i]+=(energyData[i]-avgEnergyVectorTot[i])*(energyData[i]-avgEnergyVectorTot[i]);
      }
      if(*lItr){//accepted flip
         for(int i = 0 ; i < energyFunctionCount ; ++i){
            stdDevEnergyVectorAcc[i]+=(energyData[i]-avgEnergyVectorAcc[i])*(energyData[i]-avgEnergyVectorAcc[i]);
         }
      }else{//rejected flip
         for(int i = 0 ; i < energyFunctionCount ; ++i){
            stdDevEnergyVectorRej[i]+=(energyData[i]-avgEnergyVectorRej[i])*(energyData[i]-avgEnergyVectorRej[i]);
         }
      }
      ++lItr;
   }
   


   // "Normalizing" stdDev
      for (int j = 0 ; j < energyFunctions.size() ; ++j){
         if(NTot)
            stdDevEnergyVectorTot[j]=sqrt(stdDevEnergyVectorTot[j])/NTot;
         if(NAcc)
            stdDevEnergyVectorAcc[j]=sqrt(stdDevEnergyVectorAcc[j])/NAcc;
         if(NRej)
            stdDevEnergyVectorRej[j]=sqrt(stdDevEnergyVectorRej[j])/NRej;
         
      }
   

}

void EnergyFunctionCalculatorStatistics::writeHeaderFlex(std::ofstream & _out){

   int n=1;

   for (int i = 0 ; i < energyFunctions.size() ; ++i){
      ostringstream str1;
      str1<<n<<". "<<energyFunctionsNameVec[i];
      _out<<setw(fieldWidth)<<str1.str();
      n+=1;
   }

  _out<<endl;

}


void EnergyFunctionCalculatorStatistics::writeDataLineFlex(std::ofstream & _out, std::vector<double> & _energies){

   for(int i = 0 ; i<_energies.size() ; ++i){
      _out<<setw(fieldWidth)<<_energies[i];
   }
   _out<<endl;
}


void EnergyFunctionCalculatorStatistics::prepareGatherResultsFiles(){

   if(outputAcceptedSpinFlip){
      ostringstream outSingleSpinFlipStreamNameAccepted;
      outSingleSpinFlipStreamNameAccepted<<outFileCoreNameSpinFlips<<"."<<"accepted"<<"."<<"txt";
      outAccSpinFlip =new ofstream(outSingleSpinFlipStreamNameAccepted.str().c_str());
      
      writeHeaderFlex(*outAccSpinFlip);
   }

   if(outputRejectedSpinFlip){
      ostringstream outSingleSpinFlipStreamNameRejected;
      outSingleSpinFlipStreamNameRejected<<outFileCoreNameSpinFlips<<"."<<"rejected"<<"."<<"txt";
      outRejSpinFlip =new ofstream(outSingleSpinFlipStreamNameRejected.str().c_str());

      writeHeaderFlex(*outRejSpinFlip);
   }

   if(outputTotalSpinFlip){
      ostringstream outSingleSpinFlipStreamNameTotal;
      outSingleSpinFlipStreamNameTotal<<outFileCoreNameSpinFlips<<"."<<"total"<<"."<<"txt";
      outTotSpinFlip =new ofstream(outSingleSpinFlipStreamNameTotal.str().c_str());

      writeHeaderFlex(*outTotSpinFlip);
   }

   gatherResultsFilesPrepared=true;

}

void EnergyFunctionCalculatorStatistics::outputResultsSingleSpinFlipGatherResults(){


   std::list<std::vector<double> >::iterator lVecItr;
   std::list<bool>::iterator lItr;

   lItr=accNotAccList.begin();
   for(lVecItr=totEnergyDataList.begin() ; lVecItr != totEnergyDataList.end() ; ++lVecItr){

      vector<double> & energyData=*lVecItr;
      if(outputTotalSpinFlip)
         writeDataLineFlex(*outTotSpinFlip,*lVecItr);

      if(*lItr){//accepted flip
         if(outputAcceptedSpinFlip)
            writeDataLineFlex(*outAccSpinFlip,*lVecItr);
      }else{//rejected flip
         if(outputRejectedSpinFlip)
            writeDataLineFlex(*outRejSpinFlip,*lVecItr);
      }
      ++lItr;
   }




}



void EnergyFunctionCalculatorStatistics::outputResultsSingleSpinFlip(){
   ostringstream outSingleSpinFlipStreamNameAccepted;
   outSingleSpinFlipStreamNameAccepted<<outFileCoreNameSpinFlips<<"."<<"accepted"<<"."<<mcs<<"."<<"txt";

   ostringstream outSingleSpinFlipStreamNameRejected;
   outSingleSpinFlipStreamNameRejected<<outFileCoreNameSpinFlips<<"."<<"rejected"<<"."<<mcs<<"."<<"txt";

   ostringstream outSingleSpinFlipStreamNameTotal;
   outSingleSpinFlipStreamNameTotal<<outFileCoreNameSpinFlips<<"."<<"total"<<"."<<mcs<<"."<<"txt";

   ofstream outSingleSpinFlipAccepted(outSingleSpinFlipStreamNameAccepted.str().c_str());
   ofstream outSingleSpinFlipRejected(outSingleSpinFlipStreamNameRejected.str().c_str());
   ofstream outSingleSpinFlipTotal(outSingleSpinFlipStreamNameTotal.str().c_str());

   writeHeaderFlex(outSingleSpinFlipAccepted);
   writeHeaderFlex(outSingleSpinFlipRejected);
   writeHeaderFlex(outSingleSpinFlipTotal);

   std::list<std::vector<double> >::iterator lVecItr;
   std::list<bool>::iterator lItr;

   lItr=accNotAccList.begin();
   for(lVecItr=totEnergyDataList.begin() ; lVecItr != totEnergyDataList.end() ; ++lVecItr){

      vector<double> & energyData=*lVecItr;
      writeDataLineFlex(outSingleSpinFlipTotal,*lVecItr);
      if(*lItr){//accepted flip
         writeDataLineFlex(outSingleSpinFlipAccepted,*lVecItr);
      }else{//rejected flip
         writeDataLineFlex(outSingleSpinFlipRejected,*lVecItr);
      }
      ++lItr;
   }



}


void EnergyFunctionCalculatorStatistics::outputResults(){
   cerr<<"-------------ENERGY CALCULATOR STATISTICS-------------"<<endl;
   cerr<<"Accepted Energy:"<<endl;
   double totAccEnergyChange=0;
   

   calculateStatData(); //this actually calculates stat data and allocates all necessary vectors

   for (int i = 0 ; i < energyFunctions.size() ; ++i){
      cerr<<"TOT "<<energyFunctionsNameVec[i]<<" "<<avgEnergyVectorTot[i]*NTot<<" avg: "<<avgEnergyVectorTot[i]<<" stdDev: "<<stdDevEnergyVectorTot[i]<<endl;
      cerr<<"ACC "<<energyFunctionsNameVec[i]<<" "<<avgEnergyVectorAcc[i]*NAcc<<" avg: "<<avgEnergyVectorAcc[i]<<" stdDev: "<<stdDevEnergyVectorAcc[i]<<endl;
      cerr<<"REJ "<<energyFunctionsNameVec[i]<<" "<<avgEnergyVectorRej[i]*NRej<<" avg: "<<avgEnergyVectorRej[i]<<" stdDev: "<<stdDevEnergyVectorRej[i]<<endl;

      totAccEnergyChange+=avgEnergyVectorAcc[i]*NAcc;
   }

   if(out){
      (*out)<<setw(fieldWidth)<<mcs<<setw(fieldWidth)<<NAcc<<setw(fieldWidth)<<NRej<<setw(fieldWidth)<<NTot;
      for (int i = 0 ; i < energyFunctions.size() ; ++i){
         (*out)<<setw(fieldWidth)<<avgEnergyVectorAcc[i]<<setw(fieldWidth)<<stdDevEnergyVectorAcc[i];
      }
      for (int i = 0 ; i < energyFunctions.size() ; ++i){
         (*out)<<setw(fieldWidth)<<avgEnergyVectorRej[i]<<setw(fieldWidth)<<stdDevEnergyVectorRej[i];
      }
      for (int i = 0 ; i < energyFunctions.size() ; ++i){
         (*out)<<setw(fieldWidth)<<avgEnergyVectorTot[i]<<setw(fieldWidth)<<stdDevEnergyVectorTot[i];
      }
      (*out)<<endl;

   }

   cerr<<"TOTAL ACC ENERGY CHANGE="<<totAccEnergyChange<<endl;
   cerr<<"-------------End of ENERGY CALCULATOR STATISTICS-------------"<<endl;
   cerr<<"Output File name = "<<outFileName<<endl;

}

void EnergyFunctionCalculatorStatistics::readXML(XMLPullParser &in){
  in.match(START_ELEMENT); 
  in.skip(TEXT);
  
  while (in.check(START_ELEMENT)) {
    if (in.getName() == "OutputFileName") {
      if(in.findAttribute("Frequency")>=0){
         analysisFrequency=BasicString::parseUInteger(in.getAttribute("Frequency").value);
      }
      
      outFileName = in.matchSimple();
  
    } else if (in.getName() == "OutputCoreFileNameSpinFlips") {
      
      if(in.findAttribute("Frequency")>=0){
         singleSpinFrequency=BasicString::parseUInteger(in.getAttribute("Frequency").value);
      }

      if(in.findAttribute("GatherResults")>=0){
         gatherResultsSpinFlip=true;
         
      }

      if(in.findAttribute("OutputAccepted")>=0){
         outputAcceptedSpinFlip=true;
         
      }

      if(in.findAttribute("OutputRejected")>=0){
          outputRejectedSpinFlip=true;
      }

      if(in.findAttribute("OutputTotal")>=0){
         outputTotalSpinFlip=true;
      }
      

      outFileCoreNameSpinFlips = in.matchSimple();
      outputEverySpinFlip=true;
  
    } else {
      throw BasicException(string("Unexpected element '") + in.getName() + "'!", in.getLocation());
    }

    in.skip(TEXT);
  }  
   in.match(END_ELEMENT,-TEXT);

   out=new ofstream(outFileName.c_str());
   
   ASSERT_OR_THROW("Could not open a file to store energy statistics",out && out->good());
   

}


void EnergyFunctionCalculatorStatistics::writeXML(XMLSerializer &out){}

