/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "CellOrientationEnergy.h"
#include "CellOrientationPlugin.h"

#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <PublicUtilities/NumericalUtils.h>
#include <CompuCell3D/Simulator.h>
#include <string>


#include <CompuCell3D/plugins/PolarizationVector/PolarizationVector.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <BasicUtils/BasicClassGroup.h>


using namespace std;


double CellOrientationEnergy::localEnergy(const Point3D &pt) {
  return 0;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CellOrientationEnergy::initialize(Simulator * _simulator){
   simulator=_simulator;
   fieldDim=simulator->getPotts()->getCellFieldG()->getDim();

}




double CellOrientationEnergy::changeEnergy(const Point3D &pt,
				  const CellG *newCell,
				  const CellG *oldCell) {

   float energy=0.0;
   PolarizationVector * polarizationVecPtr;
   Point3D spinCopyVector;

   //spin change takes place at pt  and spin from potts->getFlipNeighbor() is copied to pt. We define spinCopyVector as:
//    spinCopyVector=pt-potts->getFlipNeighbor();
      
   //this will return distance vector which will properly account for different boundary conditions
   spinCopyVector=distanceVectorInvariant(pt,potts->getFlipNeighbor(),fieldDim);
   double lambdaCellOrientationValue=0.0;

   if(oldCell){
      if(!lambdaFlexFlag){
         lambdaCellOrientationValue=lambdaCellOrientation;
      }else{
         lambdaCellOrientationValue=lambdaCellOrientationAccessorPtr->get(oldCell->extraAttribPtr)->lambdaVal;
      }
      
      polarizationVecPtr = polarizationVectorAccessorPtr->get(oldCell->extraAttribPtr);
      energy+=-lambdaCellOrientationValue*(polarizationVecPtr->x * spinCopyVector.x + polarizationVecPtr->y * spinCopyVector.y + polarizationVecPtr->z * spinCopyVector.z);

   }

   
   if(newCell){
      if(!lambdaFlexFlag){
         lambdaCellOrientationValue=lambdaCellOrientation;
      }else{
         lambdaCellOrientationValue=lambdaCellOrientationAccessorPtr->get(newCell->extraAttribPtr)->lambdaVal;
      }
      
      polarizationVecPtr = polarizationVectorAccessorPtr->get(newCell->extraAttribPtr);
      energy+=-lambdaCellOrientationValue*(polarizationVecPtr->x * spinCopyVector.x + polarizationVecPtr->y * spinCopyVector.y + polarizationVecPtr->z * spinCopyVector.z);

   }

//    cerr<<"energy="<<energy<<endl;
   return energy;
}


void CellOrientationEnergy::readXML(XMLPullParser &in) {
  in.skip(TEXT);

  while (in.check(START_ELEMENT)) {
   if (in.getName() == "LambdaCellOrientation") {
      lambdaCellOrientation = BasicString::parseDouble(in.matchSimple());

    }else if (in.getName() == "LambdaFlex") {
      lambdaFlexFlag=true;
      in.matchSimple();
    }else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
			   "'!", in.getLocation());
    }

   cerr<<"lambdaCellOrientation="<<lambdaCellOrientation<<endl;

    in.skip(TEXT);
  }  



}

void CellOrientationEnergy::writeXML(XMLSerializer &out) {
}

std::string CellOrientationEnergy::toString(){
   return string("CellOrientation");
}



