/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "CompartmentEnergy.h"

#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Automaton/Automaton.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <CompuCell3D/Potts3D/Cell.h>

#include <algorithm>


#include <string>
using namespace std;

double ContactCompartmentEnergy::localEnergy(const Point3D &pt) {
  return 0;
}


double ContactCompartmentEnergy::changeEnergy(const Point3D &pt,
                                  const CellG *newCell,
                                  const CellG *oldCell) {
   //cerr<<"ChangeEnergy"<<endl;
   
   
  double energy = 0;
  unsigned int token = 0;
  double distance = 0;
  Point3D n;
  Neighbor neighbor;
  
  CellG *nCell=0;
  Field3D<CellG *> *fieldG = potts->getCellFieldG();

   if(weightDistance){

//       while (true) {
//          n = fieldG->getNeighbor(pt, token, distance, false);
//          if (distance > depth) break;
// 
//          nCell = fieldG->get(n);
      for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
         neighbor=boundaryStrategy->getNeighborDirect(const_cast<Point3D&>(pt),nIdx);
         if(!neighbor.distance){
         //if distance is 0 then the neighbor returned is invalid
         continue;
         }
         nCell = fieldG->get(neighbor.pt);

         if(nCell!=oldCell){
            energy -= contactEnergy(oldCell, nCell) / distance;
         }
         if(nCell!=newCell){
            energy += contactEnergy(newCell, nCell) / distance;
         }
      }
  }else{
   //default behaviour  no energy weighting 
//       while (true) {
//          n = fieldG->getNeighbor(pt, token, distance, false);
//          if (distance > depth) break;
// 
//          nCell = fieldG->get(n);
      for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
         neighbor=boundaryStrategy->getNeighborDirect(const_cast<Point3D&>(pt),nIdx);
         if(!neighbor.distance){
         //if distance is 0 then the neighbor returned is invalid
         continue;
         }
         nCell = fieldG->get(neighbor.pt);

         if(nCell!=oldCell){
            if((nCell != 0) && (oldCell != 0)) {
               if((nCell->clusterId) == (oldCell->clusterId)) {
                  energy -= internalEnergy(oldCell,nCell);
               }
               else {
                  energy -= contactEnergy(oldCell, nCell);
               }
            }
            else{
               energy -= contactEnergy(oldCell, nCell);
           }
         }



         if(nCell!=newCell){
            if((newCell != 0) && (nCell != 0)) {
               if((newCell->clusterId) == (nCell->clusterId)) {
                  energy += internalEnergy(newCell,nCell);
               }
               else{
                  energy += contactEnergy(newCell, nCell);

               }
            }
            else{
               energy += contactEnergy(newCell, nCell);

            }
         }

   }
  }

  return energy;
}

double ContactCompartmentEnergy::contactEnergy(const CellG *cell1, const CellG *cell2) {
//          cerr << "Contact Energy is: " << contactEnergyArray[cell1 ? cell1->type : 0][cell2? cell2->type : 0] << endl;
   return contactEnergyArray[cell1 ? cell1->type : 0][cell2? cell2->type : 0];
}

double ContactCompartmentEnergy::internalEnergy(const CellG *cell1, const CellG *cell2) {
//       cerr << "Internal Energy is: " << internalEnergyArray[cell1 ? cell1->type : 0][cell2? cell2->type : 0] << endl;
   
   return internalEnergyArray[cell1 ? cell1->type : 0][cell2? cell2->type : 0];
}
void ContactCompartmentEnergy::setContactCompartmentEnergy(const string typeName1,
				     const string typeName2,
				     const double energy) {
                    
  char type1 = automaton->getTypeId(typeName1);
  char type2 = automaton->getTypeId(typeName2);
    
  int index = getIndex(type1, type2);

  contactEnergies_t::iterator it = contactEnergies.find(index); //return an iterator for the contact Energy
  ASSERT_OR_THROW(string("Contact energy for ") + typeName1 + " " + typeName2 +
		  " already set!", it == contactEnergies.end());

  contactEnergies[index] = energy;
}

void ContactCompartmentEnergy::setInternalEnergy(const string typeName1,
				     const string typeName2,
				     const double energy) {
                    cerr<< "PING\n";
  char type1 = automaton->getTypeId(typeName1);
  char type2 = automaton->getTypeId(typeName2);
    
  int index = getIndex(type1, type2);

  contactEnergies_t::iterator it = internalEnergies.find(index); //return an iterator for the contact Energy
  ASSERT_OR_THROW(string("Internalenergy for ") + typeName1 + " " + typeName2 +
		  " already set!", it == internalEnergies.end());

  internalEnergies[index] = energy;
}
int ContactCompartmentEnergy::getIndex(const int type1, const int type2) const {
  if (type1 < type2) return ((type1 + 1) | ((type2 + 1) << 16));
  else return ((type2 + 1) | ((type1 + 1) << 16));
}


void ContactCompartmentEnergy::readXML(XMLPullParser &in) {
  in.skip(TEXT);

   automaton = potts->getAutomaton();
   set<unsigned char> cellTypesSet;
   set<unsigned char> cellInternalTypesSet;

   
  while (in.check(START_ELEMENT)) {
    if (in.getName() == "Energy") {
      string type1 = in.getAttribute("Type1").value;
      string type2 = in.getAttribute("Type2").value;

      double energy = BasicString::parseDouble(in.matchSimple());
      setContactCompartmentEnergy(type1, type2, energy);
      cerr<<"Contact Energy Set\n";
      //inserting all the types to the set (duplicate are automatically eleminated) to figure out max value of type Id
      cellTypesSet.insert(automaton->getTypeId(type1));
      cellTypesSet.insert(automaton->getTypeId(type2));
     }

     else if (in.getName() == "InternalEnergy") {
      string type1 = in.getAttribute("Type1").value;
      string type2 = in.getAttribute("Type2").value;
      double energy = BasicString::parseDouble(in.matchSimple());
      setInternalEnergy(type1, type2, energy);
      //inserting all the types to the set (duplicate are automatically eleminated) to figure out max value of type Id
      cellInternalTypesSet.insert(automaton->getTypeId(type1));
      cellInternalTypesSet.insert(automaton->getTypeId(type2));
    } 
    else if (in.getName() == "Depth") {
      depth = BasicString::parseDouble(in.matchSimple());
      
    }else if (in.getName() == "Weight") {
      weightDistance=true;
      in.matchSimple();
    }
   else {
      throw BasicException(string("Unexpected element '") + in.getName() +
                           "'!", in.getLocation());
    }

    in.skip(TEXT);
  }

  //Now that we know all the types used in the simulation we will find size of the contactEnergyArray
  vector<unsigned char> cellTypesVector(cellTypesSet.begin(),cellTypesSet.end());//coping set to the vector

  int size= * max_element(cellTypesVector.begin(),cellTypesVector.end());
  size+=1;//if max element is e.g. 5 then size has to be 6 for an array to be properly allocated
  
  int index ;
  contactEnergyArray.assign(size,vector<double>(size,0.0));

//Begin Internal Vector allocation
vector<unsigned char> cellInternalTypesVector(cellInternalTypesSet.begin(),cellInternalTypesSet.end());//coping set to the vector

  int i_size= *  max_element(cellInternalTypesVector.begin(),cellInternalTypesVector.end());
   i_size+=1;
  int i_index ;
  internalEnergyArray.assign(size,vector<double>(size,0.0));
  cerr <<"i_size: " << i_size <<"  size: " << size << endl;
//End Internal vector allocation
  for(int i = 0 ; i < i_size ; ++i) {
   for(int j = 0 ; j < i_size ; ++j){
   
      i_index = getIndex(cellInternalTypesVector[i],cellInternalTypesVector[j]);
      
      internalEnergyArray[i][j] = internalEnergies[i_index];
      
      }
   }
   cerr<<"isize="<<size<<endl;
   
  for(int i = 0 ; i < size ; ++i){
   for(int j = 0 ; j < size ; ++j){
   
       cerr<<"internal_energy["<<i<<"]["<<j<<"]="<<internalEnergyArray[i][j]<<endl;
      
      }
   }
   

  for(int i = 0 ; i < size ; ++i) {
   for(int j = 0 ; j < size ; ++j){
   
      index = getIndex(cellTypesVector[i],cellTypesVector[j]);
      
      contactEnergyArray[i][j] = contactEnergies[index];
      
      }
   }
   cerr<<"size="<<size<<endl;
   
  for(int i = 0 ; i < size ; ++i){
   for(int j = 0 ; j < size ; ++j){
   
      cerr<<"contact["<<i<<"]["<<j<<"]="<<contactEnergyArray[i][j]<<endl;
      
      }
   }
   //setting up neighbors 
   boundaryStrategy=BoundaryStrategy::getInstance();
   maxNeighborIndex=0;


   maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(depth);

//   exit(0);
}

void ContactCompartmentEnergy::writeXML(XMLSerializer &out) {
}

std::string ContactCompartmentEnergy::toString(){
  return string("Contact");
}
