/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR1 PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef LENGTHCONSTRAINTENERGY_H
#define LENGTHCONSTRAINTENERGY_H

#include <CompuCell3D/Potts3D/EnergyFunction.h>

#include <XMLCereal/XMLSerializable.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <vector>
//#include <string>

namespace CompuCell3D {
   class Simulator;
  	
  /** 
   * Calculates length energy based on a target length and
   * lambda length.
   */
   class LengthEnergyParam{
      public:
         LengthEnergyParam():targetLength(0),lambdaLength(0.0){}
         LengthEnergyParam(unsigned int _targetLength,float _lambdaLength):
                  targetLength(_targetLength),lambdaLength(_lambdaLength)
         {}
         unsigned int targetLength;
         float lambdaLength;

   };
   
  class LengthConstraintEnergy: public EnergyFunction, public virtual XMLSerializable {
       

    std::vector<LengthEnergyParam> lengthEnergyParamVector;
    std::vector<std::string> typeNameVec;//temporary vector for storage type names
  public:
    LengthConstraintEnergy(){}

   
   
    virtual double localEnergy(const Point3D &pt);
    virtual double changeEnergy(const Point3D &pt, const CellG *newCell,
				const CellG *oldCell);

   const std::vector<LengthEnergyParam> & getLengthEnergyParamVector()const{return lengthEnergyParamVector;}
    
   // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    virtual std::string toString();
    
    // End XMLSerializable interface
    void initTypeId(Potts3D * potts);
  };
};
#endif
