/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/
 
#include "PDESolverCallerPlugin.h"
#include <CompuCell3D/ClassRegistry.h>

using namespace CompuCell3D;

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <iostream>
using namespace std;

PDESolverCallerPlugin::PDESolverCallerPlugin() {}

PDESolverCallerPlugin::~PDESolverCallerPlugin() {}

void PDESolverCallerPlugin::init(Simulator *simulator) {
   sim=simulator;
   potts = sim->getPotts();

   potts->registerFixedStepper(this);

}

void PDESolverCallerPlugin::extraInit(Simulator *simulator) { 

   ClassRegistry *classRegistry=simulator->getClassRegistry();
   Steppable * steppable;
   
      
   for(unsigned int i=0; i < PDESolverNames.size() ; ++i ){
      steppable=classRegistry->getStepper(PDESolverNames[i]);
      solverPtrVec.push_back(steppable);
   }

   
}

void PDESolverCallerPlugin::step(){

//    cerr<<"inside STEP"<<endl;
   unsigned int currentStep;
   unsigned int currentAttempt;
   unsigned int numberOfAttempts;
   

   currentStep=sim->getStep();
   currentAttempt=potts->getCurrentAttempt();
   numberOfAttempts=potts->getNumberOfAttempts();
   


   
   for(int i=0 ; i <timesPerMCVec.size() ; ++i){
      int reminder= (numberOfAttempts % (timesPerMCVec[i]+1));
      int ratio=(numberOfAttempts / (timesPerMCVec[i]+1));

      if( ! ((currentAttempt-reminder) % ratio ) && currentAttempt>reminder ){
//          cerr<<"before calling step"<<endl;
          solverPtrVec[i]->step(currentStep);
//          float a=reminder+ratio;
//       cerr<<"calling Solver"<<PDESolverNames[i]<<" currentAttempt="<<currentAttempt<<" numberOfAttempts="<<numberOfAttempts<<endl;

      }

   }
   
   


}

void PDESolverCallerPlugin::readXML(XMLPullParser &in) {
  in.skip(TEXT);

   cerr<<"inside readXML PDECaller"<<endl;
   
  while (in.check(START_ELEMENT)) {
    if (in.getName() == "CallPDE") {
      string solverName = in.getAttribute("PDESolverName").value;
      unsigned int timesPerMC = BasicString::parseUInteger(in.getAttribute("ExtraTimesPerMC").value);
      PDESolverNames.push_back(solverName);
      timesPerMCVec.push_back(timesPerMC);

      in.matchSimple();
    }
    else {
      throw BasicException(string("Unexpected element '") + in.getName() +
                           "'!", in.getLocation());
    }

    in.skip(TEXT);
  }

/*   for (int i = 0 ; i <PDESolverNames.size() ;++i){
      cerr<<"PDESolverNames["<<i<<"]="<<PDESolverNames[i]<<" times="<<timesPerMCVec[i]<<endl;
   }*/
//    exit(0);

  
}

void PDESolverCallerPlugin::writeXML(XMLSerializer &out) {
}
