#ifndef PLASTICITYTRACKERPLUGIN_H
#define PLASTICITYTRACKERPLUGIN_H

#include <CompuCell3D/Plugin.h>

#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Potts3D/CellGChangeWatcher.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <BasicUtils/BasicClassGroup.h> //had to include it to avoid problems with template instantiation
#include "PlasticityTracker.h"
#include <CompuCell3D/Field3D/AdjacentNeighbor.h>

namespace CompuCell3D {

  class Cell;
  class Field3DIndex;
  template <class T> class Field3D;

  class CellInventory;
  class BoundaryStrategy;
  
class PlasticityTrackerPlugin : public Plugin, public CellGChangeWatcher {

      Field3D<CellG *> *cellFieldG;
      Dim3D fieldDim;
      BasicClassAccessor<PlasticityTracker> plasticityTrackerAccessor;
      Simulator *simulator;
      CellInventory * cellInventoryPtr;
      bool initialized;

      unsigned int maxNeighborIndex;
      BoundaryStrategy *boundaryStrategy;
    
   public:
      PlasticityTrackerPlugin();
      virtual ~PlasticityTrackerPlugin();
      
      // SimObject interface
      virtual void init(Simulator *simulator);
      virtual void extraInit(Simulator *simulator);

      // BCGChangeWatcher interface
      virtual void field3DChange(const Point3D &pt, CellG *newCell,
                                 CellG *oldCell);
      
      // Begin XMLSerializable interface
      virtual void readXML(XMLPullParser &in);
      virtual void writeXML(XMLSerializer &out);
      BasicClassAccessor<PlasticityTracker> * getPlasticityTrackerAccessorPtr(){return & plasticityTrackerAccessor;}
      // End XMLSerializable interface
     void initializePlasticityNeighborList();

   protected:

      
      std::set<std::string> plasticityTypesNames;
      std::set<unsigned char> plasticityTypes;

  };
};
#endif
