/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR1 PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef REARRANGEMENTENERGY_H
#define REARRANGEMENTENERGY_H

#include <CompuCell3D/Potts3D/EnergyFunction.h>

#include <XMLCereal/XMLSerializable.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <NeighborTracker/NeighborTrackerPlugin.h>


namespace CompuCell3D {

  
  	class BoundaryStrategy;
   class Potts3D;
   class CellG;
   template <class T> class Field3D;
  /** 
   * Calculates cell rearrengement suppression energy
   * 
   */
  class RearrangementEnergy: public EnergyFunction, public virtual XMLSerializable {
       
    double fRearrangement;
    double lambdaRearrangement;
    unsigned int maxNeighborIndex;
    BoundaryStrategy *boundaryStrategy;
    Potts3D * potts;
    Field3D<CellG *> *cellFieldG;
    BasicClassAccessor<NeighborTracker> * neighborTrackerAccessorPtr;
    float percentageLossThreshold;
    float defaultPenalty;
   

  public:
    RearrangementEnergy() ;

    RearrangementEnergy(double fRearrangement, double lambdaRearrangement) ;
    void init(Potts3D *_potts);
    std::pair<CellG*,CellG*> preparePair(CellG* , CellG*);
    
    
    virtual double localEnergy(const Point3D &pt);

    virtual double changeEnergy(const Point3D &pt, const CellG *newCell,
				const CellG *oldCell);

   double getLambdaRearrangement()const{return lambdaRearrangement;}
   double getFRearrangement()const{return fRearrangement;}
   
    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    virtual std::string toString();
    // End XMLSerializable interface
  };


};
#endif
