/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "FoamDataOutput.h"

#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Automaton/Automaton.h>

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Field3D/Dim3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <BasicUtils/BasicRandomNumberGenerator.h>
#include <string>

#include <CompuCell3D/plugins/NeighborTracker/NeighborTrackerPlugin.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <fstream>
#include <sstream>

using namespace std;


FoamDataOutput::FoamDataOutput() :
  potts(0)
{
    surFlag=false;
    volFlag=false;
    numNeighborsFlag=false;
    cellIDFlag=false;
    neighborTrackerAccessorPtr=0;
}


void FoamDataOutput::init(Simulator *simulator) {
  potts = simulator->getPotts();
  cellInventoryPtr=& potts->getCellInventory();
}

void FoamDataOutput::extraInit(Simulator *simulator) {

   if(numNeighborsFlag){
      NeighborTrackerPlugin * neighborTrackerPluginPtr=(NeighborTrackerPlugin*)(Simulator::pluginManager.get("NeighborTracker"));
      ASSERT_OR_THROW("NeighborTracker plugin not initialized!", neighborTrackerPluginPtr);
      neighborTrackerAccessorPtr=neighborTrackerPluginPtr->getNeighborTrackerAccessorPtr();
      ASSERT_OR_THROW("neighborAccessorPtr  not initialized!", neighborTrackerAccessorPtr);
   }
   
}


void FoamDataOutput::start() {}

void FoamDataOutput::step(const unsigned int currentStep) {

   ostringstream str;
   str<<fileName<<"."<<currentStep;
   ofstream out(str.str().c_str());
   
//    cerr<<"cellIDFlag= " << cellIDFlag<<" numNeighborsFlag="<<numNeighborsFlag<<" volFlag="<<volFlag<<" surFlag="<<surFlag<<endl;

   CellInventory::cellInventoryIterator cInvItr;
   CellG * cell;

   std::set<NeighborSurfaceData > * neighborData;

   for(cInvItr=cellInventoryPtr->cellInventoryBegin() ; cInvItr !=cellInventoryPtr->cellInventoryEnd() ;++cInvItr ){

      cell=*cInvItr;
      if(cellIDFlag){
         out<<cell->id<<"\t";
      }
      if(volFlag){
         out<<cell->volume<<"\t";
      }

      if(surFlag){
         out<<cell->surface<<"\t";
      }

      if(numNeighborsFlag){
         neighborData = &(neighborTrackerAccessorPtr->get(cell->extraAttribPtr)->cellNeighbors);
         out<<neighborData->size()<<"\t";
      }

      out<<endl;

   }
   
}

void FoamDataOutput::readXML(XMLPullParser &in) {

   in.skip(TEXT);

  string type;
  
  while (in.check(START_ELEMENT)) {
  
    if (in.getName() == "Output") {
  
      if(in.findAttribute("Volume")>=0){
//          volumeData=in.getAttribute("Volume").value;
            volFlag=true;
      }

      if(in.findAttribute("Surface")>=0){
//          surfaceData=in.getAttribute("Surface").value;
            surFlag=true;
      }

      if(in.findAttribute("NumberOfNeighbors")>=0){
         numNeighborsFlag=true;
      }

      if(in.findAttribute("CellID")>=0){
         cellIDFlag=true;
      }

      if(in.findAttribute("FileName")>=0){
         fileName=in.getAttribute("FileName").value;;
      }else{
         fileName="Output";
      }
      

      in.matchSimple();

    }else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
			   "'!", in.getLocation());
    }

    in.skip(TEXT);
  }
}

void FoamDataOutput::writeXML(XMLSerializer &out) {
}







