#include "FastDiffusionSolver2DFE.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <CompuCell3D/Field3D/Field3DImpl.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/Field3DIO.h>
#include <BasicUtils/BasicClassGroup.h>



#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <BasicUtils/BasicRandomNumberGenerator.h>
#include <PublicUtilities/StringUtils.h>
#include <string>
#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>

#include <time.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// std::ostream & operator<<(std::ostream & out,CompuCell3D::DiffusionData & diffData){
//    
//    
// }


using namespace CompuCell3D;
using namespace std;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DSerializer::serialize(){

   for(int i = 0 ; i < solverPtr->diffDataVec.size() ; ++i){
      ostringstream outName;

      outName<<solverPtr->diffDataVec[i].fieldName<<"_"<<currentStep<<"."<<serializedFileExtension;
      ofstream outStream(outName.str().c_str());
      solverPtr->outputField( outStream,solverPtr->concentrationFieldVector[i]);
   }

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DSerializer::readFromFile(){
   try{
      for(int i = 0 ; i < solverPtr->diffDataVec.size() ; ++i){
         ostringstream inName;
         inName<<solverPtr->diffDataVec[i].fieldName<<"."<<serializedFileExtension;
   
            solverPtr->readConcentrationField(inName.str().c_str(),solverPtr->concentrationFieldVector[i]);;
      }

   } catch (BasicException &e) {
      cerr<<"COULD NOT FIND ONE OF THE FILES"<<endl;
      throw BasicException("Error in reading diffusion fields from file",e);
   }


}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
FastDiffusionSolver2DFE::FastDiffusionSolver2DFE()
 : DiffusableVector2D<float>(),deltaX(1.0),deltaT(1.0)
{
   serializerPtr=0;
   serializeFlag=false;
   readFromFileFlag=false;
   haveCouplingTerms=false;
   serializeFrequency=0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
FastDiffusionSolver2DFE::~FastDiffusionSolver2DFE()
{

   if(serializerPtr)
      delete serializerPtr ; serializerPtr=0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::init(Simulator *simulator) {
  simPtr=simulator;
  potts = simulator->getPotts();
  automaton=potts->getAutomaton();
   
  ///getting cell inventory
  cellInventoryPtr=& potts->getCellInventory(); 
  
  ///getting field ptr from Potts3D
  cellFieldG=potts->getCellFieldG();
  fieldDim=cellFieldG->getDim();
  
  

  
  ///setting member function pointers
  diffusePtr=&FastDiffusionSolver2DFE::diffuse;
  secretePtr=&FastDiffusionSolver2DFE::secrete;


}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::extraInit(Simulator *simulator){

  if((serializeFlag || readFromFileFlag) && !serializerPtr){
   serializerPtr=new FastDiffusionSolver2DSerializer();
   serializerPtr->solverPtr=this;
  }

   if(serializeFlag){
      simulator->registerSerializer(serializerPtr);
   }


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::start() {
//     if(diffConst> (1.0/6.0-0.05) ){ //hard coded condtion for stability of the solutions - assume dt=1 dx=dy=dz=1
// 
//       cerr<<"CANNOT SOLVE DIFFUSION EQUATION: STABILITY PROBLEM - DIFFUSION CONSTANT TOO LARGE. EXITING..."<<endl;
//       exit(0);
// 
//    }


   dt_dx2=deltaT/(deltaX*deltaX);
   if(readFromFileFlag){
      try{
      
//          serializerPtr->readFromFile();
      
      } catch (BasicException &e){
         cerr<<"Going to fail-safe initialization"<<endl;
         initializeConcentration(); //if there was error, initialize using failsafe defaults
      }

   }else{
      initializeConcentration();//Normal reading from User specified files
   }

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FastDiffusionSolver2DFE::initializeConcentration(){


   for(unsigned int i = 0 ; i <diffDataVec.size() ; ++i){
      if(diffDataVec[i].concentrationFileName.empty()) continue;
      cerr<<"fail-safe initialization "<<diffDataVec[i].concentrationFileName<<endl;
      readConcentrationField(diffDataVec[i].concentrationFileName,concentrationFieldVector[i]);
   }


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::step(const unsigned int _currentStep) {

    currentStep=_currentStep;

   (this->*secretePtr)();

   (this->*diffusePtr)();


      if(serializeFrequency>0 && serializeFlag && !(_currentStep % serializeFrequency)){
         serializerPtr->setCurrentStep(currentStep);
         serializerPtr->serialize();
      }
      
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FastDiffusionSolver2DFE::secreteOnContactSingleField(unsigned int idx){


SecretionData & secrData=secrDataVec[idx];


std::map<unsigned char,SecretionOnContactData>::iterator mitr;
std::map<unsigned char,SecretionOnContactData>::iterator end_mitr=secrData.typeIdSecrOnContactDataMap.end();

CellG *currentCellPtr;

ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
Array2D_t & concentrationArray = concentrationFieldPtr->getContainer();

float currentConcentration;
float secrConst;
float secrConstMedium=0.0;
std::map<unsigned char, float> * contactCellMapMediumPtr;
std::map<unsigned char, float> * contactCellMapPtr;
std::map<unsigned char, float>::iterator mitrTypeConst;

bool secreteInMedium=false;
//the assumption is that medium has type ID 0
mitr=secrData.typeIdSecrOnContactDataMap.find(automaton->getTypeId("Medium"));

if( mitr != end_mitr ){
   secreteInMedium=true;
   contactCellMapMediumPtr = &(mitr->second.contactCellMap);
}



Point3D pt;
Neighbor n;
CellG *nCell=0;
Field3D<CellG *> *fieldG = potts->getCellFieldG();
unsigned char type;



      for (int x = 1; x < workFieldDim.x-1; x++)
         for (int y = 1; y < workFieldDim.y-1; y++){
            pt=Point3D(x-1,y-1,0);
            currentCellPtr=cellFieldG->get(pt);
            currentConcentration = concentrationArray[x][y];

                        
            if(secreteInMedium && ! currentCellPtr){

                  for (int i = 0  ; i<offsetVec.size() ; ++i ){ 
                     n=boundaryStrategy->getNeighborDirect(pt,i);
                     if(!n.distance)//not a valid neighbor
                        continue;
                     nCell = fieldG->get(n.pt);

                     if(nCell)
                        type=nCell->type;
                     else
                        type=0;
                        

                     mitrTypeConst=contactCellMapMediumPtr->find(type);
                     
                     if(mitrTypeConst != contactCellMapMediumPtr->end()){//OK to secrete, contact detected
                        secrConstMedium = mitrTypeConst->second;

                        concentrationArray[x][y]=currentConcentration+secrConstMedium;
                     }                     
         
                     
                     
                  }                  

                  continue;
            }

             if(currentCellPtr){
                mitr=secrData.typeIdSecrOnContactDataMap.find(currentCellPtr->type);
                if(mitr!=end_mitr){
                  
                  contactCellMapPtr = &(mitr->second.contactCellMap);
                  
                  for (int i = 0  ; i<offsetVec.size() ; ++i ){ 

                     n=boundaryStrategy->getNeighborDirect(pt,i);
                     if(!n.distance)//not a valid neighbor
                        continue;

                     nCell = fieldG->get(n.pt);

                     if(nCell)
                        type=nCell->type;
                     else
                        type=0;
                        
                     mitrTypeConst=contactCellMapPtr->find(type);
                     if(mitrTypeConst != contactCellMapPtr->end()){//OK to secrete, contact detected
                        secrConst=mitrTypeConst->second;
//                         concentrationField->set(pt,currentConcentration+secrConst);
                        concentrationArray[x][y]=currentConcentration+secrConst;
                     }
                     
                  }

                  
                }
             }
         }

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::secreteSingleField(unsigned int idx){


SecretionData & secrData=secrDataVec[idx];


std::map<unsigned char,float>::iterator mitr;
std::map<unsigned char,float>::iterator end_mitr=secrData.typeIdSecrConstMap.end();

CellG *currentCellPtr;
Field3DImpl<float> * concentrationField=concentrationFieldVector[idx];
float currentConcentration;
float secrConst;
float secrConstMedium=0.0;

ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
Array2D_t & concentrationArray = concentrationFieldPtr->getContainer();


bool secreteInMedium=false;
//the assumption is that medium has type ID 0
mitr=secrData.typeIdSecrConstMap.find(automaton->getTypeId("Medium"));

if( mitr != end_mitr){
   secreteInMedium=true;
   secrConstMedium=mitr->second;
   
}

Point3D pt;
      for (int x = 0; x < workFieldDim.x-1; x++)
         for (int y = 0; y < workFieldDim.y-1; y++){
            pt=Point3D(x-1,y-1,0);
            currentCellPtr=cellFieldG->get(pt);
            currentConcentration = concentrationArray[x][y];
            
            if(secreteInMedium && ! currentCellPtr){
               concentrationArray[x][y]=currentConcentration+secrConstMedium;
            }

            if(currentCellPtr){
               mitr=secrData.typeIdSecrConstMap.find(currentCellPtr->type);
               if(mitr!=end_mitr){
                  secrConst=mitr->second;
                  concentrationArray[x][y]=currentConcentration+secrConst;
               }
            }
         }


}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::secrete() {

   for(unsigned int i = 0 ; i < secrDataVec.size() ; ++i ){

      for(unsigned int j = 0 ; j <secrDataVec[i].secretionFcnPtrVec.size() ; ++j){
         (this->*secrDataVec[i].secretionFcnPtrVec[j])(i);
      }
               

   }

      

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
float FastDiffusionSolver2DFE::couplingTerm(Point3D & _pt,std::vector<CouplingData> & _couplDataVec,float _currentConcentration){

   float couplingTerm=0.0;
   float coupledConcentration;
   for(int i =  0 ; i < _couplDataVec.size() ; ++i){
      coupledConcentration=concentrationFieldVector[_couplDataVec[i].fieldIdx]->get(_pt);
      couplingTerm+=_couplDataVec[i].couplingCoef*_currentConcentration*coupledConcentration;
   }

   return couplingTerm;

         

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::boundaryConditionInit(ConcentrationField_t *concentrationField){
   Array2D_t & concentrationArray=concentrationField->getContainer();


   if(periodicBoundaryCheckVector[0]){//periodic BC along X
      for(int y=0 ; y<workFieldDim.y ; ++y){
            concentrationArray[0][y]=concentrationArray[workFieldDim.x-2][y];
            concentrationArray[workFieldDim.x-1][y]=concentrationArray[1][y];
      }


   }else{//noFlux BC along X
      for(int y=0 ; y<workFieldDim.y ; ++y){
            concentrationArray[0][y]=concentrationArray[1][y];
            concentrationArray[workFieldDim.x-1][y]=concentrationArray[workFieldDim.x-2][y];
      }


   }

   if(periodicBoundaryCheckVector[1]){//periodic BC along Y
      for(int x=0 ; x<workFieldDim.x ; ++x){
            concentrationArray[x][0]=concentrationArray[x][workFieldDim.y-2];
            concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][1];
      }

   }else{//noFlux BC along Y
      for(int x=0 ; x<workFieldDim.x ; ++x){
            concentrationArray[x][0]=concentrationArray[x][1];
            concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][workFieldDim.y-2];
      }

   }
   
//    if(periodicBoundaryCheckVector[0] || periodicBoundaryCheckVector[1]){
//       for(int y=0 ; y<workFieldDim.y ; ++y){
//             concentrationArray[0][y]=concentrationArray[workFieldDim.x-2][y];
//             concentrationArray[workFieldDim.x-1][y]=concentrationArray[1][y];
//       }
// 
//       for(int x=0 ; x<workFieldDim.x ; ++x){
//             concentrationArray[x][0]=concentrationArray[x][workFieldDim.y-2];
//             concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][1];
//       }
// 
//       
//    }else{
//       for(int y=0 ; y<workFieldDim.y ; ++y){
//             concentrationArray[0][y]=concentrationArray[1][y];
//             concentrationArray[workFieldDim.x-1][y]=concentrationArray[workFieldDim.x-2][y];
//       }
// 
//       for(int x=0 ; x<workFieldDim.x ; ++x){
//             concentrationArray[x][0]=concentrationArray[x][1];
//             concentrationArray[x][workFieldDim.y-1]=concentrationArray[x][workFieldDim.y-2];
//       }
// 
// 
//    }


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FastDiffusionSolver2DFE::diffuseSingleField(unsigned int idx){

// cerr<<" DIFFUSE"<<endl;

// clock_t start, end;
// double time;
// start = clock();
/// 'n' denotes neighbor

///this is the diffusion equation 
///C_{xx}+C_{yy}+C_{zz}=(1/a)*C_{t}
///a - diffusivity - diffConst

///Finite difference method:
///T_{0,\delta \tau}=F*\sum_{i=1}^N T_{i}+(1-N*F)*T_{0}
///N - number of neighbors
///will have to double check this formula


// Point3D pt, n;
// unsigned int token = 0;
// double distance;
CellG * currentCellPtr=0,*nCell=0;

// short currentCellType=0;
float concentrationSum=0.0;
float updatedConcentration=0.0;


float currentConcentration=0.0;
// short neighborCounter=0;

DiffusionData & diffData = diffDataVec[idx];
float diffConst=diffData.diffConst;
float decayConst=diffData.decayConst;
float deltaT=diffData.deltaT;
float deltaX=diffData.deltaX;
float dt_dx2=deltaT/(deltaX*deltaX);



// std::set<unsigned char>::iterator sitr;
// std::set<unsigned char>::iterator end_sitr=diffDataVec[idx].avoidTypeIdSet.end();
// std::set<unsigned char>::iterator end_sitr_decay=diffDataVec[idx].avoidDecayInIdSet.end();

// Automaton *automaton=potts->getAutomaton();

ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
ConcentrationField_t * scratchFieldPtr;

//    scratchFieldPtr=concentrationFieldVector[diffDataVec.size()];
   if(!haveCouplingTerms)
      scratchFieldPtr=concentrationFieldVector[diffDataVec.size()];
   else
      scratchFieldPtr=concentrationFieldVector[diffDataVec.size()+idx];

   Array2D_t & concentrationArray = concentrationFieldPtr->getContainer();
   Array2D_t & scratchArray = scratchFieldPtr->getContainer();
    boundaryConditionInit(concentrationFieldPtr);//initializing boundary conditions

//    cerr<<" concentrationArray[1][1] = "<<concentrationArray[1][1]<<endl;
//    cerr<<"concentrationFieldPtr->get(Point3D(0,0,0))="<<concentrationFieldPtr->get(Point3D(0,0,0))<<endl;

std::set<unsigned char>::iterator end_sitr_decay=diffDataVec[idx].avoidDecayInIdSet.end();   
bool avoidDecayInMedium=false;
//the assumption is that medium has type ID 0

if(diffDataVec[idx].avoidDecayInIdSet.find(automaton->getTypeId("Medium")) != end_sitr_decay){
   avoidDecayInMedium=true;
}

   

   unsigned int offset=workFieldDim.x*workFieldDim.y;
    unsigned int counter=offset;
    float *temp;
    
      for (int x = 1; x < workFieldDim.x-1; ++x)
         for (int y = 1; y < workFieldDim.y-1; ++y){
         Point3D pt(x-1,y-1,0);
         currentCellPtr=cellFieldG->get(pt);
         currentConcentration = concentrationArray[x][y];
         temp=&scratchArray[x][y];
//          temp  = &scratchVec[x][y][1];
//          temp=&scratch[counter];
//          scratchArray[x][y][1]=0;
         
         updatedConcentration=0.0;
         concentrationSum=0.0;

         concentrationSum+=concentrationArray[x][y+1];
         concentrationSum+=concentrationArray[x+1][y];
         concentrationSum+=concentrationArray[x][y-1];
         concentrationSum+=concentrationArray[x-1][y];

         updatedConcentration =  dt_dx2*diffConst*(concentrationSum - 4*currentConcentration)+currentConcentration;
                                    
                                    

         //processing decay depandent on type of the current cell

//          updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed
         if(currentCellPtr){
            if(diffData.avoidDecayInIdSet.find(currentCellPtr->type)!=end_sitr_decay){
               ;//decay in this type is forbidden
            }else{
               updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed
            }
         }else{
            if(avoidDecayInMedium){
               ;//decay in Medium is forbidden
            }else{
               updatedConcentration-=deltaT*(decayConst*currentConcentration); //decay in Medium is allowed
            }
         }
//          if(!currentCellPtr){
//             updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed  
//          }
//          if (x<=3&&y<=3){
//             cerr<<"concentrationSum="<<concentrationSum<<" updatedConcentration="<<updatedConcentration<<endl;
//          }

            *temp=updatedConcentration;
// 	         scratch[counter]=updatedConcentration;
//             ++counter;
//          scratchArray[x][y][1]=updatedConcentration;//updating scratch

      }


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::diffuse() {
   
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i ){
      diffuseSingleField(i);
      if(!haveCouplingTerms){ //without coupling terms field swap takes place immediately aftera given field has been diffused
         ConcentrationField_t * concentrationField=concentrationFieldVector[i];
         ConcentrationField_t * scratchField=concentrationFieldVector[diffDataVec.size()];
         //copy updated values from scratch to concentration field
         scrarch2Concentration(scratchField,concentrationField);
      }

   }

   if(haveCouplingTerms){  //with coupling terms we swap scratch and concentration field after all fields have been diffused
      for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i ){
         ConcentrationField_t * concentrationField=concentrationFieldVector[i];
         ConcentrationField_t * scratchField=concentrationFieldVector[diffDataVec.size()+i];
         scrarch2Concentration(scratchField,concentrationField);

      }

   }
      

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::scrarch2Concentration( ConcentrationField_t *scratchField, ConcentrationField_t *concentrationField){
   scratchField->switchContainersQuick(*(concentrationField));

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::outputField( std::ostream & _out, ConcentrationField_t *_concentrationField){
Point3D pt;
float tempValue;

    
      for (pt.x = 0; pt.x < fieldDim.x; pt.x++)
         for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
            for (pt.z = 0; pt.z < fieldDim.z; pt.z++){
         tempValue=_concentrationField->get(pt);
         _out<<pt.x<<" "<<pt.y<<" "<<pt.z<<" "<<tempValue<<endl;
      }
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::readConcentrationField(std::string fileName,ConcentrationField_t *concentrationField){

   ifstream in(fileName.c_str());

   ASSERT_OR_THROW(string("Could not open chemical concentration file '") +
                      fileName + "'!", in.is_open());

   Point3D pt;
   float c;
   //Zero entire field
   for (pt.z = 0; pt.z < fieldDim.z; pt.z++)
    for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
      for (pt.x = 0; pt.x < fieldDim.x; pt.x++){
         concentrationField->set(pt,0);
      }  

   while(!in.eof()){
      in>>pt.x>>pt.y>>pt.z>>c;
      if(!in.fail())
         concentrationField->set(pt,c);
   }

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void FastDiffusionSolver2DFE::readXML(XMLPullParser &in) {
  in.skip(TEXT);
  string concentrationFieldName;
  unsigned int concentrationFieldCounter=0;

  float diffConst;
  float decayConst;
  float secrConst;
  float deltaX;
  float deltaT;

   while(in.check(START_ELEMENT)) {
      if (in.getName() == "DiffusionField") {
         cerr<<"Got Diffusion field tag"<<endl;
         diffDataVec.push_back(DiffusionData());
         secrDataVec.push_back(SecretionDataFast());
         DiffusionData & diffData=diffDataVec[diffDataVec.size()-1];
         SecretionData & secrData=secrDataVec[secrDataVec.size()-1];
         diffData.setAutomaton(potts->getAutomaton());
         secrData.setAutomaton(potts->getAutomaton());
         
         in.match(START_ELEMENT);
         in.skip(TEXT);
       
         while(in.check(START_ELEMENT)){
            cerr<<"AFTER INNER LOOP MATCH START_ELEMENT"<<endl;
            if(in.getName() == "DiffusionData"){
               cerr<<"Will read diffusionData"<<endl;
               
               in.match(START_ELEMENT);
               diffData.readXML(in);
               in.match(END_ELEMENT);
               //in.skip(TEXT);
               cerr<<"FINISHED reading diffusionData"<<endl;
            }
            else if (in.getName() == "SecretionData"){
               cerr<<"Will read secrData"<<endl;
               in.match(START_ELEMENT);
               secrData.readXML(in);
               in.match(END_ELEMENT);
               cerr<<"FINISHED read secrData"<<endl;
            }

            else if(in.getName() == "ReadFromFile"){
               readFromFileFlag=true;
               in.matchSimple();
            }


            else{
               throw BasicException(string("Unexpected element '") + in.getName() +
                                 "'!", in.getLocation());
            }
           in.skip(TEXT);
         }
         in.skip(TEXT);
         in.match(END_ELEMENT);
         cerr<<"AFTER INNER LOOP "<<endl;
         

      }
      else if(in.getName() == "Serialize"){
         serializeFlag=true;
         if(in.findAttribute("Frequency")>=0){
            serializeFrequency=BasicString::parseUInteger(in.getAttribute("Frequency").value);
         }
         in.matchSimple();
         in.skip(TEXT);
         cerr<<"serialize Flag="<<serializeFlag<<endl;
         
      }
      else if(in.getName() == "ReadFromFile"){
         readFromFileFlag=true;
         in.matchSimple();
         in.skip(TEXT);
         cerr<<"readFromFileFlag="<<readFromFileFlag<<endl;
         
      }

      else{
        throw BasicException(string("Unexpected element '") + in.getName() +
                                 "'!", in.getLocation());
      }

      in.skip(TEXT);
      cerr<<"AFTER OUTER LOOP "<<endl;
      //exit(0);
   }
   
   vector<string> concentrationFieldNameVectorTmp; //temporary vector for field names
   ///assign vector of field names
   concentrationFieldNameVectorTmp.assign(diffDataVec.size(),string(""));

   cerr<<"diffDataVec.size()="<<diffDataVec.size()<<endl;
   
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
//       cerr<<" concentrationFieldNameVector[i]="<<diffDataVec[i].fieldName<<endl;
//       concentrationFieldNameVector.push_back(diffDataVec[i].fieldName);
      concentrationFieldNameVectorTmp[i] = diffDataVec[i].fieldName;
      cerr<<" concentrationFieldNameVector[i]="<<concentrationFieldNameVectorTmp[i]<<endl;
   }

  //setting up couplingData - field-field interaction terms
   vector<CouplingData>::iterator pos;
   
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
      pos=diffDataVec[i].couplingDataVec.begin();
      for(int j = 0 ; j <diffDataVec[i].couplingDataVec.size() ; ++j){

         for(int idx=0; idx<concentrationFieldNameVectorTmp.size() ; ++idx){
            if( concentrationFieldNameVectorTmp[idx] == diffDataVec[i].couplingDataVec[j].intrFieldName ){
               diffDataVec[i].couplingDataVec[j].fieldIdx=idx;
               haveCouplingTerms=true; //if this is called at list once we have already coupling terms and need to proceed differently with scratch field initialization
               break;
            }
            //this means that required interacting field name has not been found
            if( idx == concentrationFieldNameVectorTmp.size()-1 ){
               //remove this interacting term
//                pos=&(diffDataVec[i].degradationDataVec[j]);
               diffDataVec[i].couplingDataVec.erase(pos);
            }
         }
         ++pos;
      }
   } 


 cerr<<"FIELDS THAT I HAVE"<<endl; 
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
      cerr<<"Field "<<i<<" name: "<<concentrationFieldNameVectorTmp[i]<<endl;
   }
   



   cerr<<"FastDiffusionSolver2DFE: extra Init in read XML"<<endl;
   if(fieldDim.z!=1){
      cerr<<"2D only and XY plane only, please"<<endl;
      exit(0);
   }

   workFieldDim=Dim3D(fieldDim.x+2,fieldDim.y+2,fieldDim.z+2);
//    scratch=new float [workFieldDim.x*workFieldDim.y*workFieldDim.z];
//    scratchVec.assign(workFieldDim.x,vector<vector<float> >(workFieldDim.y,vector<float>(workFieldDim.z,0.0)));

   ///allocate fields including scrartch field
   if(!haveCouplingTerms){
      allocateDiffusableFieldVector(diffDataVec.size()+1,workFieldDim); //+1 is for additional scratch field
   }else{
      allocateDiffusableFieldVector(2*diffDataVec.size(),workFieldDim); //with coupling terms every field need to have its own scratch field
   }
   
   //here I need to copy field names from concentrationFieldNameVectorTmp to concentrationFieldNameVector
   //because concentrationFieldNameVector is reallocated with default values once I call allocateDiffusableFieldVector
   
   
   for(unsigned int i=0 ; i < concentrationFieldNameVectorTmp.size() ; ++i){
      concentrationFieldNameVector[i]=concentrationFieldNameVectorTmp[i];
   }


   //register fields once they have been allocated
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
      simPtr->registerConcentrationField(concentrationFieldNameVector[i] , concentrationFieldVector[i]);
      cerr<<"registring field: "<<concentrationFieldNameVector[i]<<" field address="<<concentrationFieldVector[i]<<endl;
   }

   
   ///assigning member method ptrs to the vector
   
   for(unsigned int i = 0 ; i < secrDataVec.size() ; ++i){
   
      secrDataVec[i].secretionFcnPtrVec.assign(secrDataVec[i].secrTypesNameSet.size(),0);
      unsigned int j=0;
      for(set<string>::iterator sitr=secrDataVec[i].secrTypesNameSet.begin() ; sitr != secrDataVec[i].secrTypesNameSet.end()  ; ++sitr){
         if((*sitr)=="Secretion"){
            secrDataVec[i].secretionFcnPtrVec[j]=&FastDiffusionSolver2DFE::secreteSingleField;
            ++j;
         }
         else if((*sitr)=="SecretionOnContact"){
            secrDataVec[i].secretionFcnPtrVec[j]=&FastDiffusionSolver2DFE::secreteOnContactSingleField;
            ++j;
         }
      }
   }


   periodicBoundaryCheckVector.assign(3,false);
   string boundaryName;
   boundaryName=potts->getBoundaryXName();
   changeToLower(boundaryName);
   if(boundaryName=="periodic")  {
     periodicBoundaryCheckVector[0]=true; 
   }
   boundaryName=potts->getBoundaryYName();
   changeToLower(boundaryName);
   if(boundaryName=="periodic")  {
     periodicBoundaryCheckVector[1]=true; 
   }

   boundaryName=potts->getBoundaryZName();
   changeToLower(boundaryName);
   if(boundaryName=="periodic")  {
     periodicBoundaryCheckVector[2]=true; 
   }

//    cerr<<"ALLOCATED ALL FIELDS"<<endl;
//    cerr<<"FROM FLEX SOLVER 2"<<endl;
//    cerr<<"concentrationFieldNameVector.size()="<<concentrationFieldNameVector.size()<<endl;
//    for(unsigned int i=0 ; i < concentrationFieldNameVector.size() ; ++i){
//       cerr<<"THIS IS FIELD NAME "<<concentrationFieldNameVector[i]<<endl;
//    }
// 
//    exit(0);
}

