#include "ScreenshotData.h"
#include "GraphicsBase.h"
#include <iostream>
#include <sstream>

using namespace std;

ScreenshotData::ScreenshotData(){
   graphicsPtr=0;
   inventoryFilePtr=0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ScreenshotData::~ScreenshotData(){
   if(graphicsPtr)
      delete graphicsPtr;
   graphicsPtr=0;

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ScreenshotData::setCoreName(const std::string  & _name )
{
   coreName=_name;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ScreenshotData::setScreenshotIdName(const QString & scrName){
   screenshotIdName = scrName;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ScreenshotData::setVisualizationWidgetType(const QString & _type){
   visualizationWidgetType=_type;

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

const QString & ScreenshotData::getVisualizationWidgetType(){
   return visualizationWidgetType;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ScreenshotData::okToProceed(const QString & _simulationRootDir,const QString & _coreName){
   QString thisDirName;
   thisDirName=QString(_simulationRootDir+QDir::separator()+_coreName);


   bool dirExists=dir.exists(thisDirName);
   if(dirExists){
      cerr<<"Directory already exists and is in use. Ignoring request"<<endl;
   }
   
   return !dirExists;

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ScreenshotData::activate(const QString & _simulationRootDir){
   simulationRootDir=_simulationRootDir;
   thisDirectoryName=QString(simulationRootDir+QString(QDir::separator())+QString(coreName.c_str()));

   bool dirExists=dir.exists(thisDirectoryName);
   if(dirExists){
      cerr<<"Directory already exists and is in use. Ignoring request"<<endl;
      return;
   }

   bool dirOK=dir.mkdir(thisDirectoryName);
      if(!dirOK && !dirExists){
         cerr<<"Could not create directory: "<<thisDirectoryName.toStdString()<<endl;
         cerr<<"Make sure you have permissions to write in current directory and that you have enough disk space "<<endl;
         return;
      }
   fullInventoryFileName = QString(thisDirectoryName+QString(QDir::separator())+QString(coreName.c_str())+".grl");
   
   inventoryFilePtr=new ofstream();
   inventoryFilePtr->open(fullInventoryFileName.toStdString().c_str());

   graphicsPtr->setUnivGraphSetPtr(&univGraphSet);
   
   //cerr<<"&univGraphSet="<<&univGraphSet<<endl;
   //cerr<<"graphicsPtr="<<graphicsPtr<<endl;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ScreenshotData::outputScreenshot(unsigned int screenshotCounter,unsigned int numScreenNameDigits){
    QImage image;
    ofstream &out = *inventoryFilePtr;

    ostringstream numberStream;

    numberStream.width(numScreenNameDigits);
    numberStream.fill('0');
    numberStream<<screenshotCounter;


    QString screenshotName(QString(coreName.c_str())+"."+QString(numberStream.str().c_str())+".png");
    QString screenshotFullName(thisDirectoryName+QString(QDir::separator())+screenshotName);
    //draws scene (cell field concentration etc. depending on the initialization)
    //(graphicsPtr->*(graphicsPtr->getCurrentPainitgFcnPtr()))();
    
    graphicsPtr->drawCurrentScene();
    
    cerr<<"GOT HERE"<<endl;
    cerr<<"fcn address="<<graphicsPtr->getCurrentPainitgFcnPtr()<<endl;
    cerr<<"graphicsPtr="<<graphicsPtr<<endl;

    cerr<<"visualizationWidgetType="<<visualizationWidgetType.toStdString()<<endl;

    if(visualizationWidgetType==QString("3D")){
      cerr<<" \n\n\n CALLING PRODUCE IMAGE FOR 3D WIDGET \n\n\n"<<endl;
      graphicsPtr->produceImage(screenshotFullName.toStdString());

    }else{
      cerr<<" \n\n\n CALLING PRODUCE IMAGE FOR 2D WIDGET \n\n\n"<<endl;
      graphicsPtr->produceImage(image);
      image.save(QString(screenshotFullName),"PNG");
      //Here I will save informaiton about min and max quantities
      if(graphicsPtr->getSilentMode() && !graphicsPtr->getXServerFlag()){//we will output text files with max min quantities only in silent mode and when there is no connection to XServer (as specified by the user)
         ofstream outMinMax;
         string fileName=screenshotFullName.toStdString()+".minmax";
         outMinMax.open(fileName.c_str());
         outMinMax<<"minConcentration="<<graphicsPtr->getMinConcentration()<<endl;
         outMinMax<<"maxConcentration="<<graphicsPtr->getMaxConcentration()<<endl;
         outMinMax<<"minMagnitude="<<graphicsPtr->getMinMagnitude()<<endl;
         outMinMax<<"maxMagnitude="<<graphicsPtr->getMaxMagnitude()<<endl;
         outMinMax.close();

         
      }

   }
      //adding entry to file list
    out<<screenshotName.toStdString()<<endl;
    

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////




















