/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 *   THIS PLUGIN (C) BY JOHANNES KNABE, UNIVERSITY OF HERTFORDSHIRE,UK   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "CellShapeControlEnergy.h"
#include "CellShapeControlPlugin.h"

#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <PublicUtilities/NumericalUtils.h>
#include <CompuCell3D/Simulator.h>
#include <string>


#include <BasicUtils/BasicClassAccessor.h>
#include <BasicUtils/BasicClassGroup.h>

#include <CompuCell3D/steppables/GRNcells/GRNconcentrationsData.h>
#include <CompuCell3D/steppables/GRNcells/GRNcells.h>

using namespace std;

double lengthLambda=100.0;

double CellShapeControlEnergy::localEnergy(const Point3D &pt) {
  return 0;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CellShapeControlEnergy::initialize(Simulator * _simulator){
   simulator=_simulator;
   fieldDim=simulator->getPotts()->getCellFieldG()->getDim();

}

#define vecMinus(u,v) Point3D( ((u).x-(v).x),((u).y-(v).y),((u).z-(v).z) )
#define dot(u,v)      ((u).x * (v).x + (u).y * (v).y + (u).z * (v).z)
#define norm(v)       sqrt((float)dot(v,v))  // norm = length of vector
#define d(u,v)        norm(u-v)       // distance = norm of difference
#define cross(u,v)    Point3D( ( (u).y*(v).z-(u).z*(v).y ) , ( (u).z*(v).x-(u).x*(v).z ) , ( (u).x*(v).y-(u).y*(v).x )  )
#define PI 3.14159265

bool CellShapeControlEnergy::findExtremePoints(double angle, Point3D extra, vector<Point3D>::iterator begin, vector<Point3D>::iterator end, Point3D &Vmin, Point3D &Vmin2nd, Point3D &Vmax, Point3D &Vmax2nd){
	   Point3D * pixel;
	   Point3D orient;//the direction we want to control
	   orient.x=10.0*cos(angle);
	   orient.y=10.0*sin(angle);
	   Vmax=extra;
	   Vmax2nd=extra;
	   Vmin=extra;
	   Vmin2nd=extra;
	   //cerr<<"OLD LENGTH "<<angle<<" "<<sin(angle)<<" "<<cos(angle)<<" "<<orient<<" "<<endl;
	   //cerr <<"OLD PIXELS: ";
	   for( vector<Point3D>::iterator cInvItr=begin; cInvItr!=end; ++cInvItr){
				  pixel=&*cInvItr;
				  //cerr <<*pixel<<" ";
				  if(dot(orient,vecMinus(*pixel,Vmax))>0){
					  Vmax2nd=Vmax;
					  Vmax=*pixel;
					  continue;
				  }
				  if(dot(orient,vecMinus(*pixel,Vmin))<0){
					  Vmin2nd=Vmin;
					  Vmin=*pixel;
				  }
	   }
	   //cerr<<endl;
	   if((Vmax==Vmax2nd)||(Vmin==Vmin2nd)) //our starting point "extra" was an extremum
		   return false;                    //so we will have to scan again!
	   return true;

}

double CellShapeControlEnergy::changeEnergy(const Point3D &pt,
				  const CellG *newCell,
				  const CellG *oldCell) {
   float energy=0.0;
   GRNcells * ttt=(GRNcells *) simulator->steppableManager.get("GRNcells");
   if(ttt)
			GRNconcentrationsDataAccessorPtr = ttt->getGRNconcentrationsDataAccessorPtr();

   if(oldCell){
	   double targetRatio=GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->targetLengthRatio;
	   double angle=GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->angle;
	   //targetRatio=0.2;
	   //angle=PI*4.75;
	   Point3D starter=GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.at(0);
	   Point3D Vmax=starter;
	   Point3D Vmax2nd=starter;
	   Point3D Vmin=starter;
	   Point3D Vmin2nd=starter;
	   bool alright=findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
	   //cerr<<"OLD LENGTH "<<angle<<" "<<sin(angle)<<" "<<cos(angle)<<" "<<orient<<" "<<endl;
	   //cerr <<"OLD PIXELS: ";
       double currAngleLength=d(Vmin,Vmax);

	   if((!alright)&&(GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.size()>1)){ 
		   //take cellPixels.at(1) as starter instead of cellPixels.at(0)
		   starter=GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.at(1);
		   findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
	   }

	   if((Vmin.x==pt.x)&&(Vmin.y==pt.y)&&(Vmin.z==pt.z))
		  Vmin=Vmin2nd; //consider this one removed, so take 2nd closest
	   if((Vmax.x==pt.x)&&(Vmax.y==pt.y)&&(Vmax.z==pt.z))
		  Vmax=Vmax2nd; //consider this one removed, so take 2nd closest

       double oldAngleLength=d(Vmin,Vmax);

//now perpedicular
	   angle+=PI/2.0;
	   starter=GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.at(0);
	   Vmax=starter;
	   Vmax2nd=starter;
	   Vmin=starter;
	   Vmin2nd=starter;
	   alright=findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
	   //cerr<<"OLD LENGTH "<<angle<<" "<<sin(angle)<<" "<<cos(angle)<<" "<<orient<<" "<<endl;
	   //cerr <<"OLD PIXELS: ";
       double currPerpLength=d(Vmin,Vmax);
	   double currRatio;
	   if(currAngleLength==0)
		   currRatio=1;
	   else
		   currRatio=currPerpLength/currAngleLength;

	   if((!alright)&&(GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.size()>1)){ 
		   //take cellPixels.at(1) as starter instead of cellPixels.at(0)
		   starter=GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.at(1);
		   findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(oldCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
	   }

	   if((Vmin.x==pt.x)&&(Vmin.y==pt.y)&&(Vmin.z==pt.z))
		  Vmin=Vmin2nd; //consider this one removed, so take 2nd closest
	   if((Vmax.x==pt.x)&&(Vmax.y==pt.y)&&(Vmax.z==pt.z))
		  Vmax=Vmax2nd; //consider this one removed, so take 2nd closest

       double oldPerpLength=d(Vmin,Vmax);
	   double oldRatio;
	   if(oldAngleLength==0)
		   oldRatio=1;
	   else
		   oldRatio=oldPerpLength/oldAngleLength;

//and finally energy:
	   energy+=lengthLambda*(fabs(targetRatio-oldRatio)-fabs(targetRatio-currRatio));
	   //cerr<<"TARGET: "<<targetRatio<<" OLD RATIO: "<<oldRatio<<" CURR RATIO: "<<currRatio<<" = ENERGY: "<<energy<<endl;
	   //cerr <<"OLD LENGTH: "<<oldLength<<" CURR LENGTH: "<<currLength<<" "<<Vmin<<" "<<Vmax<<" "<<energy<<" "<<angle<<endl;
   }

   if(newCell){
	   double targetRatio=GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->targetLengthRatio;
	   double angle=GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->angle;
	   //targetRatio=0.2;
	   //angle=PI*4.75;
	   Point3D starter=pt; //adding pt to set of points
	   Point3D Vmax=starter;
	   Point3D Vmax2nd=starter;
	   Point3D Vmin=starter;
	   Point3D Vmin2nd=starter;

	   bool alright=findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
       double newAngleLength=d(Vmin,Vmax);

	   int i=0;
	   while(!alright){ //pt was an extremum, so we run again without it
		   if(i>=GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.size())
			   break;
		   starter=GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.at(i++);
		   alright=findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
	   }
	   if((Vmin.x==pt.x)&&(Vmin.y==pt.y)&&(Vmin.z==pt.z))
		  Vmin=Vmin2nd; //this is to be added, ignore for currLength
	   if((Vmax.x==pt.x)&&(Vmax.y==pt.y)&&(Vmax.z==pt.z))
		  Vmax=Vmax2nd; //this is to be added, ignore for currLength

	   double currAngleLength=d(Vmin,Vmax);

//now perpendicular
	   angle+=PI/2.0;
	   starter=pt; //adding pt to set of points
	   Vmax=starter;
	   Vmax2nd=starter;
	   Vmin=starter;
	   Vmin2nd=starter;

	   alright=findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
       double newPerpLength=d(Vmin,Vmax);
	   double newRatio;
	   if(newAngleLength==0)
		   newRatio=1;
	   else
		   newRatio=newPerpLength/newAngleLength;

	   i=0;
	   while(!alright){ //pt was an extremum, so we run again without it
		   if(i>=GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.size())
			   break;
		   starter=GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.at(i++);
		   alright=findExtremePoints(angle, starter, GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessorPtr->get(newCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
	   }
	   if((Vmin.x==pt.x)&&(Vmin.y==pt.y)&&(Vmin.z==pt.z))
		  Vmin=Vmin2nd; //this is to be added, ignore for currLength
	   if((Vmax.x==pt.x)&&(Vmax.y==pt.y)&&(Vmax.z==pt.z))
		  Vmax=Vmax2nd; //this is to be added, ignore for currLength

	   double currPerpLength=d(Vmin,Vmax);
	   double currRatio;
	   if(currAngleLength==0)
		   currRatio=1;
	   else
		   currRatio=currPerpLength/currAngleLength;

//finally energy:
	   energy+=lengthLambda*(fabs(targetRatio-newRatio)-fabs(targetRatio-currRatio));
	   //energy+=lengthLambda*(fabs(targetLength-newAngleLength)-fabs(targetLength-currAngleLength));
	   //cerr <<"NEW LENGTH: "<<newLength<<" CURR LENGTH: "<<currLength<<" "<<Vmin<<" "<<Vmax<<" "<<energy<<" "<<angle<<endl;
   }

//    cerr<<"energy="<<energy<<endl;
   return energy;
}


void CellShapeControlEnergy::readXML(XMLPullParser &in) {
  in.skip(TEXT);

  while (in.check(START_ELEMENT)) {
   if (in.getName() == "LambdaCellShapeControl") {
      lengthLambda = BasicString::parseDouble(in.matchSimple());

    }else if (in.getName() == "LambdaFlex") {//not used
      lambdaFlexFlag=true;
      in.matchSimple();
    }else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
			   "'!", in.getLocation());
    }

    in.skip(TEXT);
  }  



}

void CellShapeControlEnergy::writeXML(XMLSerializer &out) {
}

std::string CellShapeControlEnergy::toString(){
   return string("CellShapeControl");
}



