/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 *   THIS PLUGIN (C) BY JOHANNES KNABE, UNIVERSITY OF HERTFORDSHIRE,UK   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "CellShapeControlPlugin.h"

#include "CellShapeControlEnergy.h"


#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/plugins/PolarizationVector/PolarizationVector.h>
#include <CompuCell3D/plugins/PolarizationVector/PolarizationVectorPlugin.h>

using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <iostream>
using namespace std;

CellShapeControlPlugin::CellShapeControlPlugin() : cellShapeControlEnergy(0), cellFieldG(0) {}

CellShapeControlPlugin::~CellShapeControlPlugin() {
  if (cellShapeControlEnergy) delete cellShapeControlEnergy;
}

void CellShapeControlPlugin::init(Simulator *simulator) {
   cerr<<"INITIALIZE CELL SHAPE CONTROL PLUGIN"<<endl;   
   Potts3D *potts = simulator->getPotts();

   cellFieldG = potts->getCellFieldG();
   cellShapeControlEnergy = new CellShapeControlEnergy(potts);

   potts->registerEnergyFunctionWithName(cellShapeControlEnergy,"CellShapeControlEnergy");
   potts->getCellFactoryGroupPtr()->registerClass(&lambdaCellShapeControlAccessor);

}

void CellShapeControlPlugin::extraInit(Simulator *simulator) {
  cerr<<"EXTRA INITIALIZE CELL SHAPE CONTROL PLUGIN"<<endl;   
  Potts3D *potts = simulator->getPotts();
  cellFieldG = potts->getCellFieldG();
  cellShapeControlEnergy->initialize( simulator);
  cellShapeControlEnergy->setLambdaCellShapeControlAccessorPtr(this->getLambdaCellShapeControlAccessorPtr());
}


void CellShapeControlPlugin::readXML(XMLPullParser &in) {
  cellShapeControlEnergy->readXML(in);
}

void CellShapeControlPlugin::writeXML(XMLSerializer &out) {
  cellShapeControlEnergy->writeXML(out);
}
