/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "SurfaceEnergy.h"

#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <string>
using namespace std;

double SurfaceEnergy::diffEnergy(int surface, int diff) {
  return lambdaSurface *
    (diff * diff + 2 * diff * (surface - (int)targetSurface));
}


double SurfaceEnergy::localEnergy(const Point3D &pt) {
  return 0;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

double SurfaceEnergy::changeEnergy(const Point3D &pt,
				  const CellG *newCell,
				  const CellG *oldCell) {

  // E = lambda * (surface - targetSurface) ^ 2
  CellG *nCell;
  
  if (oldCell == newCell) return 0;

  unsigned int token = 0;
  double distance;
  int oldDiff = 0;
  int newDiff = 0;
  Point3D n;
  double energy = 0;


  bool print_flag=false;
  if(pt==Point3D(35,16,0))
    print_flag=true;

//    if(print_flag){
//       cerr<<"newCell="<<newCell<<" oldCell="<<oldCell<<endl;
//    }
  // Count surface difference
  while (true) {
    n = cellFieldG->getNeighbor(pt, token, distance, false);
    if (distance > 1) break;
    nCell = cellFieldG->get(n);
//     if (print_flag)
//     cerr<<"C++ chPt="<<pt<<" n="<<n<<endl;
    
//     if(print_flag){
//          cerr<<"nCell="<<nCell<<endl;
//       if(nCell){
//          cerr<<"nCell->id="<<nCell->id<<endl;
//       }else{
//          cerr<<"nCell.id=0"<<endl;
//       }
//     }

    if (newCell == nCell) newDiff--;
    else newDiff++;

    if (oldCell == nCell) oldDiff++;
    else oldDiff--;
  }

//    if (print_flag)
//     cerr<<" C++ oldDiff="<<oldDiff<<" newDiff="<<newDiff<<endl;

  if (newCell){
    energy += diffEnergy(newCell->surface, newDiff);
//     if (print_flag)
//     cerr<<"newCell->surface=" <<newCell->surface<<endl;
   }
  if (oldCell){
    energy += diffEnergy(oldCell->surface, oldDiff);
//     if (print_flag)
//     cerr<<"oldCell->surface=" <<oldCell->surface<<endl;
  }
   
//   cerr<<"Surface energy="<<energy<<endl;
//     cerr<<"pt="<<pt<<" lambdaSurface="<<lambdaSurface<<" Surface energy="<<energy<<endl;
//     cerr<<"C++ energy="<<energy<<endl;
//       if (print_flag)
//       cout<<"pt="<<pt<<" e="<<energy<<endl;
//    if(oldDiff==-4) exit(0);
  return energy;
}


void SurfaceEnergy::readXML(XMLPullParser &in) {
  in.skip(TEXT);

  while (in.check(START_ELEMENT)) {
    if (in.getName() == "TargetSurface") {
      targetSurface = BasicString::parseUInteger(in.matchSimple());

    } else if (in.getName() == "LambdaSurface") {
      lambdaSurface = BasicString::parseDouble(in.matchSimple());

    } else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
			   "'!", in.getLocation());
    }

    in.skip(TEXT);
  }  
}

void SurfaceEnergy::writeXML(XMLSerializer &out) {
}

std::string SurfaceEnergy::toString(){
   return string("Surface");
}