/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 *   THIS PLUGIN (C) BY JOHANNES KNABE, UNIVERSITY OF HERTFORDSHIRE,UK   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Field3D/Dim3D.h>
#include <CompuCell3D/plugins/CellType/CellTypePlugin.h>
#include <CompuCell3D/plugins/ContactMultiCad/ContactMultiCadEnergy.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <CompuCell3D/plugins/ContactMultiCad/ContactMultiCadData.h>

using namespace CompuCell3D;
#define PI 3.14159265

//define EXP_STL
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <CompuCell3D/steppables/GRNcells/GRNcells.h>
//include "GRNcells.h"

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <CompuCell3D/Potts3D/CellInventory.h>
#include "./GRNconcentrationsData.h"

#include <CompuCell3D/Potts3D/Cell.h>
#include <PublicUtilities/StringUtils.h>

#include <string>
#include <sstream>
#include <fstream>
#include <map>
#include <cmath>
#include <iostream>
#include <algorithm>

using namespace std;

GRNcells::GRNcells() :
  potts(0)
{}

#define vecMinus(u,v) Point3D( ((u).x-(v).x),((u).y-(v).y),((u).z-(v).z) )
#define dot(u,v)      ((u).x * (v).x + (u).y * (v).y + (u).z * (v).z)
#define norm(v)       sqrt((float)dot(v,v))  // norm = length of vector
#define d(u,v)        norm(u-v)       // distance = norm of difference
#define cross(u,v)    Point3D( ( (u).y*(v).z-(u).z*(v).y ) , ( (u).z*(v).x-(u).x*(v).z ) , ( (u).x*(v).y-(u).y*(v).x )  )
#define PI 3.14159265

bool GRNcells::findExtremePoints(double angle, Point3D extra, vector<Point3D>::iterator begin, vector<Point3D>::iterator end, Point3D &Vmin, Point3D &Vmin2nd, Point3D &Vmax, Point3D &Vmax2nd){
	   Point3D * pixel;
	   Point3D orient;//the direction we want to control
	   orient.x=10.0*cos(angle);
	   orient.y=10.0*sin(angle);
	   Vmax=extra;
	   Vmax2nd=extra;
	   Vmin=extra;
	   Vmin2nd=extra;
	   //cerr<<"OLD LENGTH "<<angle<<" "<<sin(angle)<<" "<<cos(angle)<<" "<<orient<<" "<<endl;
	   //cerr <<"OLD PIXELS: ";
	   for( vector<Point3D>::iterator cInvItr=begin; cInvItr!=end; ++cInvItr){
				  pixel=&*cInvItr;
				  //cerr <<*pixel<<" ";
				  if(dot(orient,vecMinus(*pixel,Vmax))>0){
					  Vmax2nd=Vmax;
					  Vmax=*pixel;
					  continue;
				  }
				  if(dot(orient,vecMinus(*pixel,Vmin))<0){
					  Vmin2nd=Vmin;
					  Vmin=*pixel;
				  }
	   }
	   //cerr<<endl;
	   if((Vmax==Vmax2nd)||(Vmin==Vmin2nd)) //our starting point "extra" was an extremum
		   return false;                    //so we will have to scan again!
	   return true;

}

//task 1:keep list of pixels belonging to a cell up to date
//task 2:keep cells' length in angle direction and in direction perpendicular to angle up to date
void GRNcells::field3DChange(const Point3D &pt, CellG *newCell,
                   CellG * oldCell) {
					 
	  if (oldCell) {
		  Point3D * pixel;
		  for( vector<Point3D>::iterator cInvItr=GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.begin();
			  cInvItr!=GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.end();++cInvItr){
				  pixel=&*cInvItr;
				  if((pixel->x==pt.x)&&(pixel->y==pt.y)&&(pixel->z==pt.z)){
					  GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.erase(cInvItr);
					  break;
				  }

		  }
		  //cerr<<"SIZE: "<<GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.size()<<" volume:"<<oldCell->volume<<endl;
		  if((GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.size()>0)){
		   double angle=GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->angle;
		   //angle=PI/4.0;
		   Point3D starter=GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.at(0);
		   Point3D Vmax=starter;
		   Point3D Vmax2nd=starter;
		   Point3D Vmin=starter;
		   Point3D Vmin2nd=starter;
		   findExtremePoints(angle, starter, GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
		   GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->angleLength=d(Vmin,Vmax);
		   angle+=PI/2.0;
		   Vmax=starter;
		   Vmax2nd=starter;
		   Vmin=starter;
		   Vmin2nd=starter;
		   findExtremePoints(angle, starter, GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
		   GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->perpendicLength=d(Vmin,Vmax);

		  }
		  else{
			  GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->angleLength=0;
			  GRNconcentrationsDataAccessor.get(oldCell->extraAttribPtr)->perpendicLength=0;
		  }
	  }
      if (newCell) {
		  bool dontAdd=false;
		  Point3D * pixel;
		  for( vector<Point3D>::iterator cInvItr=GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.begin();
			  cInvItr!=GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.end();++cInvItr){
				  pixel=&*cInvItr;
				  if((pixel->x==pt.x)&&(pixel->y==pt.y)&&(pixel->z==pt.z)){
					  dontAdd=true; //DO NOT ADD TWICE!
				  }
		  }
		  if(!dontAdd) //do add:
			  GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.push_back(pt);

		   double angle=GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->angle;
		   //angle=PI/4.0;
		   Point3D starter=GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.at(0);
		   Point3D Vmax=starter;
		   Point3D Vmax2nd=starter;
		   Point3D Vmin=starter;
		   Point3D Vmin2nd=starter;
		   findExtremePoints(angle, starter, GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
		   GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->angleLength=d(Vmin,Vmax);
		   angle+=PI/2.0;
		   Vmax=starter;
		   Vmax2nd=starter;
		   Vmin=starter;
		   Vmin2nd=starter;
		   findExtremePoints(angle, starter, GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.begin(), GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->cellPixels.end(), Vmin, Vmin2nd, Vmax, Vmax2nd);
		   GRNconcentrationsDataAccessor.get(newCell->extraAttribPtr)->perpendicLength=d(Vmin,Vmax);
	  }
}

void GRNcells::init(Simulator *simulator) {
this->simulator=simulator;
potts = simulator->getPotts();
potts->getCellFactoryGroupPtr()->registerClass(&GRNconcentrationsDataAccessor);
potts->registerCellGChangeWatcher(this);
//setGRNconcentrationsDataAccessorPtr(&GRNconcentrationsDataAccessor);

}

void GRNcells::start() {}

/*
Given a cell searches for one point on the CA grid that is part of it.
A good starting point should be passed to decrease search time, for example
the center of the cell's mass! 
*/
bool GRNcells::findCellPoint(CellG* cell,Point3D &cm, double depth){
	if(depth>4.5)
		return false;
	void* cF=potts->getCellFieldG();
    WatchableField3D<CellG *> *fieldG=(WatchableField3D<CellG *> *)cF;
	CellG * nCell; Neighbor neighbor;
	nCell = fieldG->get(cm);
	if((nCell)&&(nCell->id==cell->id))
		return true;
	BoundaryStrategy *boundaryStrategy=BoundaryStrategy::getInstance();
	unsigned int maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(depth);
	//cerr<"CA neighbors: ";
	for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
		neighbor=boundaryStrategy->getNeighborDirect(cm,nIdx);
		if(!neighbor.distance){
			//if distance is 0 then the neighbor returned is invalid
				continue;
		}
		nCell = fieldG->get(neighbor.pt);
		if((nCell)&&(nCell->id==cell->id)){
			cm.x=neighbor.pt.x;
			cm.y=neighbor.pt.y;
			cm.z=neighbor.pt.z;
			return true;
		}
		//cerr<<neighbor.pt.x<<"/"<<neighbor.pt.y<<", ";
	}//end for
	//cerr<<endl;
	//return false;
	return findCellPoint(cell, cm, depth+1.0);//recursively extend
}

void GRNcells::getCellPixels(CellG * cell,Point3D &cm,vector<Point3D>& cP)
{			CellG * nCell; Neighbor neighbor;
			void* cF=potts->getCellFieldG();
			WatchableField3D<CellG *> *fieldG=(WatchableField3D<CellG *> *)cF;
			BoundaryStrategy *boundaryStrategy=BoundaryStrategy::getInstance();
			unsigned int maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(1.05);
			for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
				neighbor=boundaryStrategy->getNeighborDirect(cm,nIdx);
				if(!neighbor.distance){
					//if distance is 0 then the neighbor returned is invalid
					continue;
				}
				nCell = fieldG->get(neighbor.pt);
				if((nCell)&&(nCell->id==cell->id)){
					//if not in list yet{
					bool inList=false;
					for(int i=0;i<cP.size();i++){
						if((cP.at(i).x==neighbor.pt.x)&&(cP.at(i).y==neighbor.pt.y)&&(cP.at(i).z==neighbor.pt.z)){
							inList=true;
							break;
						}
					}
					if(!inList){
						//cerr<<"Pushing neighbor: "<<neighbor.pt.x<<"/"<<neighbor.pt.y<<" "<<endl;
						Point3D cPpush(neighbor.pt.x,neighbor.pt.y,neighbor.pt.z);
						cP.push_back(cPpush);
						getCellPixels(cell,neighbor.pt,cP);
					}
				}
			}//end for
}

//absolute: set all pixel concentrations to avg value
void GRNcells::setAvgConcentration(vector<Point3D>& pixels, std::string field, float avgConc){
			std::map<std::string,CompuCell3D::Field3DImpl<float>*> & _fieldMap=simulator->getConcentrationFieldNameMap();
			std::map<std::string,Field3DImpl<float>*>::iterator mitr;
			for( mitr = _fieldMap.begin(); mitr != _fieldMap.end() ; ++mitr)
				if(mitr->first.compare(field)==0){
					for(int i=0;i<pixels.size();i++){
						mitr->second->set(pixels.at(i),avgConc);
					}//end for
					break;
				}//end if
}

//relative: every pixels value gets changed by changeConc. 
//N.B. if changeConc<0: negative values are not allowed.
void GRNcells::changeConcentration(vector<Point3D>& pixels, std::string field, float changeConc){
			std::map<std::string,CompuCell3D::Field3DImpl<float>*> & _fieldMap=simulator->getConcentrationFieldNameMap();
			std::map<std::string,Field3DImpl<float>*>::iterator mitr;
			float change;
			for( mitr = _fieldMap.begin(); mitr != _fieldMap.end() ; ++mitr)
				if(mitr->first.compare(field)==0){
					for(int i=0;i<pixels.size();i++){
						change=mitr->second->get(pixels.at(i))+changeConc;
						if(change<0.0) change=0.0;
						mitr->second->set(pixels.at(i),change);
					}//end for
					break;
				}//end if
}

//returns pixels' average concentration for chemical field, or a negative value if something goes wrong
float GRNcells::getAvgConcentration(vector<Point3D>& pixels, std::string field){
			//cerr<<"Cell "<<cell->id<<" has "<<cP.size()<<" pixels according to us... Volume: "<<cell->volume<<endl;
			float sum=0.0;
			std::map<std::string,CompuCell3D::Field3DImpl<float>*> & _fieldMap=simulator->getConcentrationFieldNameMap();
			std::map<std::string,Field3DImpl<float>*>::iterator mitr;
			for( mitr = _fieldMap.begin(); mitr != _fieldMap.end() ; ++mitr)
				if(mitr->first.compare(field)==0){
					for(int i=0;i<pixels.size();i++){
						sum+=mitr->second->get(pixels.at(i));
					}//end for
					break;
				}//end if
			return sum/pixels.size();
}

int cc=0;
int killCellTime=-1;
int killCellID=-1;
bool logCellData=false;
bool disableSecretion=false;

void GRNcells::step(const unsigned int currentStep) {
CellInventory *cellInventoryPtr=& potts->getCellInventory();
CellG * cell;
ContactMultiCadPlugin * cadherinPlugin=(ContactMultiCadPlugin *) simulator->pluginManager.get("ContactMultiCad");
BasicClassAccessor<ContactMultiCadData> * contactMultiCadDataAccessor;
if(cadherinPlugin)
	contactMultiCadDataAccessor = cadherinPlugin->getContactMultiCadDataAccessorPtr();
else{
	cerr<<"Cadherin plugin not running!"<<endl;
	exit(0);
}

CellInventory::cellInventoryIterator cInvItr;
for( cInvItr=cellInventoryPtr->cellInventoryBegin();
	 cInvItr!=cellInventoryPtr->cellInventoryEnd();
	 ++cInvItr){
		 cell=*cInvItr;
//  		     cell->targetVolume=10;
//				 cell->lambdaVolume=2;

		 //put something in initial cell(s)! N.B.: the "currentStep<2" means we assume concentrations get copied over in another place when mitosis takes place!
		 if((currentStep<2)&&(GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->proteinSiteActivations.size()==0)){
			 //initial cell values
			 cell->type=1;

			 //volume values:
  		     cell->targetVolume=5;
			 cell->lambdaVolume=20;

			 //surface values:
			 cell->targetSurface=1;
			 cell->lambdaSurface=1;

			 //stretchness values:
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->lambdaStretchness=40.0;
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->scaleSurface=1.0;

			 //length values:
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->lambdaLength=2.0;
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->targetLength=3;

			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->baseAngle=.0;
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->angle=.0;
			 //put something in initial GRN(s)!
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->concentrations.assign(GRN->concentrations.size(),0.0);
			 //std::vector<std::vector<std::vector<double> > > * t= GRN->4//& GRNconcentrationsDataAccessor->get(parentCell->extraAttribPtr)->proteinSiteActivations;
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->proteinSiteActivations.assign(GRN->genes.size(),std::vector<std::vector<double> >(0));
			 for(int i=0;i<GRN->genes.size();i++){
				 if(!GRN->genes.at(i))
					 continue;
				 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->proteinSiteActivations.at(i).assign(GRN->genes.at(i)->proteinSiteActivations->size(),std::vector<double>(0));
				 for(int j=0;j<GRN->genes.at(i)->proteinSiteActivations->size();j++){
					 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->proteinSiteActivations.at(i).at(j).assign(GRN->genes.at(i)->proteinSiteActivations->at(j).size(),0.0);
				 }
			 }
		 }

		 //we copy over the concentrations
		 GRN->concentrations.assign(GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->concentrations.begin(),GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->concentrations.end());
		 //but we only change pointers for the proteinSiteActivations
		 for(int i=0;i<GRN->genes.size();i++){
			 if(GRN->genes.at(i))
				GRN->genes.at(i)->proteinSiteActivations=GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->getProteinSiteActivationsForGene(i);
		 }
		 //N.B.: Sometimes scaling by KnabeGRN::inoutScaling is necessary for a value in [0,1]
		 //input diffusing chemicals
		 vector<Point3D> cP=GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->cellPixels;
/*		 vector<Point3D> cP(0);
		 Point3D cm(parentCell->xCM/parentCell->volume,parentCell->yCM/parentCell->volume,parentCell->zCM/parentCell->volume);
		 if(findCellPoint(cell,cm,1.75)){
		 	cP.push_back(cm);
		 	getCellPixels(cell,cm,cP);
		 }else{cout<<"NO starting point found! "<<endl; }*/
		 string field1="SP1";
		 string field2="SP2";
		 //concentration of KnabeGRN::angleReset is here used as factor for sensing morphogen concentration
		 //division by (1.0*KnabeGRN::inoutScaling) is just done to scale concentration to [0,1] (per gene, for 2 genes would be in [0,2])
		 //double field1in=(10.0*(GRN->concentrations.at(KnabeGRN::angleReset)/(1.0*KnabeGRN::inoutScaling)))*getAvgConcentration(cP,field1);
		 //double field2in=(10.0*(GRN->concentrations.at(KnabeGRN::angleReset)/(1.0*KnabeGRN::inoutScaling)))*getAvgConcentration(cP,field2);
		 double field1in=5.0*getAvgConcentration(cP,field1);
		 double field2in=5.0*getAvgConcentration(cP,field2);


		 //cerr<<"concs: "<<field1<<" "<<field1in<<" / "<<field2<<" "<<field2in<<endl;
		 GRN->concentrations[KnabeGRN::diffusion1ProteinInput]=field1in*KnabeGRN::inoutScaling;
		 GRN->concentrations[KnabeGRN::diffusion2ProteinInput]=field2in*KnabeGRN::inoutScaling;

double doublingVolume=24;//should be read from mitosis plugin in the future of course!!!
//not "deathProtein", here we set the actual volume ratio as iput!
GRN->concentrations.at(KnabeGRN::deathProtein)=((cell->volume*1.0)/(cell->targetVolume*1.0))*(1.0*KnabeGRN::inoutScaling);
		 GRN->process(0.0);//no input here right now...

		 //log concentration data if wanted ggg
		 if(logCellData==true){
			 ostringstream fullNameStr;
			 fullNameStr<<"cell"<<cell->id<<".cdf";
			 std::ofstream cellData(fullNameStr.str().c_str(),ios::app);
			 cellData<<currentStep;
			 for(int i=0;i<GRN->concentrations.size();i++)
				 cellData<<"\t"<<GRN->concentrations.at(i);
			 cellData<<endl;
		 }

		 //DO OUTPUT here
		 //cellType (2 prots)
		 if((GRN->concentrations.at(KnabeGRN::type2Protein)>KnabeGRN::threshold)&&(GRN->concentrations.at(KnabeGRN::type2Protein)>GRN->concentrations.at(KnabeGRN::type3Protein)))
			cell->type=2;
		 else
			if((GRN->concentrations.at(KnabeGRN::type3Protein)>KnabeGRN::threshold)&&(GRN->concentrations.at(KnabeGRN::type3Protein)>GRN->concentrations.at(KnabeGRN::type2Protein)))
				cell->type=3;
			else
				cell->type=1;

		 //secretion of diffusing chemicals (2 prots)
		 double field1out=.0;
		 double field2out=.0;
		 if(!disableSecretion){
			 field1out=GRN->concentrations.at(KnabeGRN::diffusion1Protein)/(1.0*KnabeGRN::inoutScaling);
			 field2out=GRN->concentrations.at(KnabeGRN::diffusion2Protein)/(1.0*KnabeGRN::inoutScaling);
		 }
		 //field1: setAvg for base3shpAngSGF, change for base3shpAngTGF
		 changeConcentration(cP,field1,field1out);//was: changeConcentration(cP,field1,-field1out); !
		 //field2: change both 3er times
		 changeConcentration(cP,field2,field2out);//was: changeConcentration !

		 //cadherin expression (3 prots)
		 contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(0,GRN->concentrations.at(KnabeGRN::cadherin1Protein)/(1.0*KnabeGRN::inoutScaling));
		 contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(1,GRN->concentrations.at(KnabeGRN::cadherin2Protein)/(1.0*KnabeGRN::inoutScaling));
		 contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(2,GRN->concentrations.at(KnabeGRN::cadherin3Protein)/(1.0*KnabeGRN::inoutScaling));

		 //cell death (targetVolume=0) (1 prot)
//		 if(GRN->concentrations.at(KnabeGRN::deathProtein)>KnabeGRN::threshold)
//			cell->targetVolume=0;
cell->targetVolume=floor((GRN->concentrations.at(KnabeGRN::growthProtein)/(1.0*KnabeGRN::inoutScaling))*doublingVolume);

//possible manipulation
if((currentStep>=killCellTime)&&(cell->id==killCellID))
  cell->targetVolume=0;

		 //targetVolume/growth (1 prot)
//		 if(GRN->concentrations.at(KnabeGRN::growthProtein)>KnabeGRN::threshold)
//			cell->targetVolume++;

		 //control ratio >=0.1 to around 1.5
		 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->targetLengthRatio=0.1+(GRN->concentrations.at(KnabeGRN::stretchnessControl)/(1.0*KnabeGRN::inoutScaling));

		 //control angle: baseAngle +- PI/2.0
		 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->angle=GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->baseAngle-(PI/2.0)+(PI*(GRN->concentrations.at(KnabeGRN::angleUp)/(1.0*KnabeGRN::inoutScaling)));

		 //set stretchness (surface*scaleSurface/sqrt(volume), scaleSurface in [0,2])
		 //maybe this should not be set directly but rather +/- as intermediate values are hard to maintain
//		 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->scaleSurface=2.0*(GRN->concentrations.at(KnabeGRN::stretchnessControl)/(1.0*KnabeGRN::inoutScaling));


//		 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->targetLength=1+(7.0*(GRN->concentrations.at(KnabeGRN::stretchnessControl)/(1.0*KnabeGRN::inoutScaling)));

/*		 //reset angle
		 if(GRN->concentrations.at(KnabeGRN::angleReset)>KnabeGRN::threshold)
			 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->angle=0;

		 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->angle+=GRN->concentrations.at(KnabeGRN::angleUp)/(1.0*KnabeGRN::inoutScaling)*(PI/2);
*/
		 /*for(int i=0;i<GRN->concentrations.size();i++)
			 cerr<<" "<<GRN->concentrations.at(i);
		 cerr<<endl;*/

		 //cadherin in/output:
		 //ContactMultiCadPlugin * ttt=(ContactMultiCadPlugin *) simulator->pluginManager.get("ContactMultiCad");
		 //BasicClassAccessor<ContactMultiCadData> * contactMultiCadDataAccessor = ttt->getContactMultiCadDataAccessorPtr();
		 //contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(0,1.0);

		 //GRN protein concentrations in/ouput
		 //GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->assignConc(0,0.77);

		 //diffusion in/output
		 //Point3D cm(cell->xCM/cell->volume,cell->yCM/cell->volume,cell->zCM/cell->volume);
		 //if(findCellPoint(cell,cm)){
		 //	cout<<"Yeah, starting point found!"<<endl;
		 //	vector<Point3D> cP(0);
		 //	cP.push_back(cm);
		 //	getCellPixels(cell,cm,cP);
		 //}
		 //float getAvgConcentration(std::vector<Point3D>& pixels, std::string field);
		 //changeConcentration(vector<Point3D>& pixels, std::string field, float changeConc)

		 //and copy back concentrations
		 GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->concentrations.assign(GRN->concentrations.begin(),GRN->concentrations.end());

	}//end for cellIterator

}

void GRNcells::finish() {}

void GRNcells::readXML(XMLPullParser &in) {
 in.skip(TEXT);
 while (in.check(START_ELEMENT)) {

    if (in.getName() == "killCellID") {
      killCellID = BasicString::parseUInteger(in.matchSimple());
	  cout<<"killCellID="<<killCellID<<endl;
	}else
    if (in.getName() == "logCellData") {
      logCellData=true;
	  in.matchSimple();
	  cout<<"logCellData=true"<<endl;
	}else
    if (in.getName() == "disableSecretion") {
      disableSecretion=true;
	  in.matchSimple();
	  cout<<"disableSecretion=true"<<endl;
	}else
    if (in.getName() == "killCellTime") {
      killCellTime = BasicString::parseUInteger(in.matchSimple());
	  cout<<"killCellTime="<<killCellTime<<endl;
	}else

        if (in.getName() == "Genome") {
      
		   std::string geno=in.matchSimple();
     	   GRN=new KnabeGRN(geno);
/*
//a simple testrun:
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   GRN->process(0.5);
		   for(int i=0;i<GRN->genes.size();i++){
			   //cerr<<"DEB: "<<GRN->genes[i]->proteinSiteActivations->size()<<endl;
		   //if(GRN->genes[i]->proteinSiteActivations->size()>0)
		//	   cerr<<"DEB2: "<<GRN->genes[i]->proteinSiteActivations->at(0).size()<<endl;
			   if((GRN->genes[i]!=NULL)&&(GRN->genes[i]->proteinSiteActivations->size()>0)&&(GRN->genes[i]->proteinSiteActivations->at(0).size()>0)){
				   cout<<"g"<<i<<": "<<endl;
				   cout<<GRN->genes[i]->proteinSiteActivations->at(0)[0]<<endl;
			   }
		   }
		 cout<<"cisFor0: "<<GRN->genes[0]->getNoOfCisSitesForProt(0)<<endl;
		 cout<<"cisFor1: "<<GRN->genes[0]->getNoOfCisSitesForProt(1)<<endl;
		 cout<<"cisFor2: "<<GRN->genes[0]->getNoOfCisSitesForProt(2)<<endl;
		 cout<<"cisFor3: "<<GRN->genes[0]->getNoOfCisSitesForProt(3)<<endl;
		 cout<<"cisFor4: "<<GRN->genes[0]->getNoOfCisSitesForProt(4)<<endl;
		 cout<<"cisFor5: "<<GRN->genes[0]->getNoOfCisSitesForProt(5)<<endl;
		 cout<<"cisFor6: "<<GRN->genes[0]->getNoOfCisSitesForProt(6)<<endl;
		 cout<<"cisFor7: "<<GRN->genes[0]->getNoOfCisSitesForProt(6)<<endl;
         //checkFreq = BasicString::parseUInteger();
         //checkSanity=true;
 exit(0);
 */
      }
      else {
         throw BasicException(string("Unexpected element '") + in.getName() +
                              "'!", in.getLocation());
      }
   
      in.skip(TEXT);
   }
}

void GRNcells::writeXML(XMLSerializer &out) {}



















//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF OLD STUFF
//mainly testing etc., should no longer be needed but you never know...








/*		 if(cell->id!=3){
			if((currentStep>30)&&(cellInventoryPtr->getCellInventorySize()<30))
			   cell->targetVolume=25;//FOR MITSOSIS
			 else
			   cell->targetVolume=15;
			 cell->lambdaVolume=20;
			 cell->targetSurface=1;
			 cell->lambdaSurface=1;
		 }
*/
/*
		 ContactMultiCadPlugin * ttt=(ContactMultiCadPlugin *) simulator->pluginManager.get("ContactMultiCad");
		 if(ttt){
			BasicClassAccessor<ContactMultiCadData> * contactMultiCadDataAccessor = ttt->getContactMultiCadDataAccessorPtr();
			if(cell->id==1){
			  contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(0,1.0);
			  GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->assignConc(0,0.77);
			  cerr<<"Concentration value is: "<<GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->getConc(0)<<endl;
  			  cell->type=2;
			}/*
			else{
				if(GRNconcentrationsDataAccessor.get(cell->extraAttribPtr)->getConc(0)>0.1){
					cerr<<"copying works!"<<endl;
					exit(0);
				}
			}*//*
			if(cell->id==2){
			  contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(1,1.0);
			  contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(2,1.0);
  			  cell->type=2;
			}
			if(cell->id==3){
				if(currentStep==300){
				  cell->targetVolume=0;
				  cell->targetSurface=0;
				  cell->type=1;
				}
			  contactMultiCadDataAccessor->get(cell->extraAttribPtr)->assignValue(2,1.0);
  			  //cell->type=2;
			  cerr<<"cell no. 3's volume: "<<cell->volume<<" target: "<<cell->targetVolume<<endl;
			}
		 }


		 //cout<<"ID: "<<cell->id<<" Target-Vol: "<<cell->targetVolume<<" Real-Vol: "<<cell->volume<<" Target-Sur: "<<cell->targetSurface<<" Real-Sur: "<<cell->surface<<endl;
		 //cout<<"ID: "<<cell->id<<" Mass center: "<<(cell->xCM/cell->volume)<<" / "<<(cell->yCM/cell->volume)<<endl;
		 if(cell->id==1){
			Point3D cm(cell->xCM/cell->volume,cell->yCM/cell->volume,cell->zCM/cell->volume);
			if(findCellPoint(cell,cm)){
				cout<<"Yeah, starting point found!"<<endl;
				vector<Point3D> pixels(0);
				pixels.push_back(cm);
				getCellPixels(cell,cm,pixels);
				if(currentStep==50)
					changeConcentration(pixels,"SP1",0.5);
				cout<<"Cell 1 has avg. concentrations - SP1: "<<getAvgConcentration(pixels,"SP1")<<" SP2: "<<getAvgConcentration(pixels,"SP2")<<endl;;
			}
		 }
*/

/*
   CellInventory *cellInventoryPtr=& potts->getCellInventory();
   CellInventory::cellInventoryIterator cInvItr;
   CellG * cell;
   ///loop over all the cells in the inventory   
   for( cInvItr = cellInventoryPtr->cellInventoryBegin() ; cInvItr !=cellInventoryPtr->cellInventoryEnd() ; ++cInvItr ){
      cell=*cInvItr;
         if(cell->id==id)
            break;
   }
	if(cell->id==id){

//use center of mass as starting point for search 
//cout<<"ID: "<<cell->id<<" Mass center: "<<(cell->xCM/cell->volume)<<" / "<<(cell->yCM/cell->volume)<<endl;
		Point3D cm(cell->xCM/cell->volume,cell->yCM/cell->volume,cell->zCM/cell->volume);
		if(findCellPoint(cell,cm)){
			cout<<"Yeah, starting point found!"<<endl;
			vector<Point3D> cP(0);
			cP.push_back(cm);
			getCellPixels(cell,cm,cP);

		}

	}
	cerr<<"ERROR: CELL "<<id<<" NOT FOUND!"<<endl;
	exit(1);
	return -1.0;

*//*
if(currentStep==10){//DIFFUSION EXAMPLE

   std::map<std::string,CompuCell3D::Field3DImpl<float>*> & _fieldMap=simulator->getConcentrationFieldNameMap();
   std::map<std::string,Field3DImpl<float>*>::iterator mitr;
   for( mitr = _fieldMap.begin(); mitr != _fieldMap.end() ; ++mitr){
      cerr<<"GOT FIELD: "<<mitr->first<<endl;

/*	  if(mitr->first.compare("SP1")==0){
		CompuCell3D::Dim3D fieldDim=mitr->second->getDim();
		cerr<<"Dimension:"<<fieldDim<<" xxx "<<mitr->first.c_str()<<endl;
		Point3D x(0,0,0);
		mitr->second->set(x,1.0);
	  }*//*
	  if(mitr->first.compare("SP2")==0){
		CompuCell3D::Dim3D fieldDim=mitr->second->getDim();
		cerr<<"Dimension:"<<fieldDim<<" xxx "<<mitr->first.c_str()<<endl;
		Point3D x(10,10,0);
		mitr->second->set(x,1.0);
	  }
   }
}
*/
//cout<<"Number of cells: "<<count<<endl;
		// Sleep(500);