#ifndef GRNCONCENTRATIONSDATA_H
#define GRNCONCENTRATIONSDATA_H

#include <vector>
#include <CompuCell3D/Field3D/Point3D.h>
namespace CompuCell3D {

class CellG;


class GRNconcentrationsData{

   public:
      typedef std::vector<float> ContainerType_t;
	  float lambdaStretchness;
	  float angle; //GRN controlled orientation for length ratio and for mitosis (+90 degrees for the latter)
	  float baseAngle; //underlying orientation, not modified by the GRN
	  float angleLength; //length in angle direction; not to be set from outside but internal storage, calculated by GRNcells
	  float perpendicLength; //length in angle+90degrees direction; not to be set from outside but internal storage, calculated by GRNcells
	  float targetLengthRatio; //target ratio of angleLength to perpendicLength (perpendicLength/angleLength)
	  float scaleSurface; //for stretchness, not used any longer
	  unsigned int targetLength; //not used any more
	  float lambdaLength;
	  std::vector<Point3D> cellPixels;

      GRNconcentrationsData():
		concentrations(std::vector<float>(8,0.0)),
		cellPixels(0),
		baseAngle(0),
		angle(0),
	  	angleLength(0),
	  	perpendicLength(0),
		targetLengthRatio(1),
		targetLength(0),
		lambdaLength(0),
		lambdaStretchness(1),
		scaleSurface(1)/*,
		proteinSiteActivations(std::vector<std::vector<std::vector<double> > >(0,0))*/
	  {}

      std::vector<float> concentrations; //one per GRN
	  std::vector<std::vector<std::vector<double> > > proteinSiteActivations; //several per Gene!

	  std::vector<std::vector<double> > * getProteinSiteActivationsForGene(int geneNo){
		  return & proteinSiteActivations.at(geneNo);
	  }

	  void addProteinSiteActivationsForGene(std::vector<std::vector<double> > PSA){
		  proteinSiteActivations.push_back(PSA);
	  }

      void assignConc(unsigned int _pos, float _value){
         if (_pos> concentrations.size()-1 ){
            unsigned int concentrationsSize=concentrations.size();
            for(unsigned int i=0 ; i<_pos-(concentrationsSize-1) ;++i){
               concentrations.push_back(0.);
            }
            concentrations[_pos]=_value;
         }else{
            concentrations[_pos]=_value;
         }
      }

     float getConc(unsigned int _pos){
         if (_pos>(concentrations.size()-1)){
            return 0.;
         } else{
            return concentrations[_pos];
         }  
     }

};


};
#endif
