/*
GRN model
Copyright (C) 2005-2008  Johannes Knabe
j.f.knabe@herts.ac.uk

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#pragma once

#include <iostream>
#include <sstream>
#include <cmath>
#include <string>
#include <stdexcept>
#include <vector>
#include "KnabeGene.h"
using namespace std;

vector<std::string> split(std::string& what, std::string delim);
unsigned short int fromBinary(vector<unsigned short int> bin);
vector<unsigned short int> toBinary(unsigned short int bin);
std::string intArrToString(vector<unsigned short int> in);
vector<unsigned short int> stringToIntArr(std::string in);

class KnabeGRN

{public:
 static const int protBitsNum=4;
 static const int maxProtNum=16;  //=2^protBitsNum  (i.e. 8 for three bits, but remember when to subtract one counting from zero)
 static const int globalLength=89;//=25+protBits*maxProtNum;//length of the bits in every GRN genome coding for global variables

 static const int type2Protein=0;
 static const int type3Protein=1;
 static const int diffusion1Protein=2; //the level or these protein regulates 
 static const int diffusion2Protein=3; //diffusion output, diffusion input might be different (see below)
 static const int stretchnessControl=4; //controls the relation of surface to volume
 static const int cadherin1Protein=5;
 static const int cadherin2Protein=6;
 static const int cadherin3Protein=7;
 static const int growthProtein=8;//aim to increase volume
 static const int deathProtein=9; //reset target volume to 0
 static const int angleUp=10;   //angle for mitosis
 static const int angleReset=11;//reset angle to 0
 static const int diffusion1ProteinInput=12;//was =2; , now different from input, i.e. some transduction
 static const int diffusion2ProteinInput=13;//was =3;   is involved to reduce interference
 

//inoutScaling: Input and Output are between 0 and 1, so we do some
//scaling when inputting/outputting protein concentrations
//150 was chosen because it is the ceiling of on genes output, i.e.
//one gene can produce maximally 150 instances of a protein per cycle
 static const int inoutScaling=150.0;
 static const int threshold=75;//used for cases where we need a boolean decision

 int howLong; //lifetime counter
 int currentGeneNo;    //number of genes
 double fitness;
 double bindingProportion;
 double saturationValue;
 vector<unsigned short int> cisSitesReceptiveFor; //number of cis-sites that match a protein (sum over all genes and cis-sites)
 vector<unsigned short int> genome;
 vector<KnabeGene*> genes;
 vector<unsigned short int> geneOutputProtein;
 vector<unsigned short int> geneOutputActivity;
 vector<double> concentrations;
 vector<double> proteinDecay;

public:
	KnabeGRN(std::string geno);
public:
	~KnabeGRN(void);
public:
	double process(double in, double in2, double in3);
public:
	double process(double in);

	void applySaturationCheck();
};
