/*
Artificial Genetic Regulatory Network
Copyright (C) 2008 Johannes Knabe
j.f.knabe@herts.ac.uk

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
//#include <CompuCell3D/Simulator.h>
//using namespace CompuCell3D;
#include "KnabeGRN.h"
#include "KnabeGene.h"
#include <iostream>
#include <sstream>
#include <cmath>
#include <string>
#include <stdexcept>
#include <windows.h>//for Sleep only!

using namespace std;

 int inputProtein=1; //this protein's level will be increased depending on "sensory" input
 int inputProtein2=3;
 int inputProtein3=4;
 int outputProtein=6; //this protein's level will determine the system#s output
//inoutScaling: Input and Output are between 0 and 1, so we do some
//scaling when inputting/outputting protein concentrations
//150 was chosen because it is the ceiling of on genes output, i.e.
//one gene can produce maximally 150 instances of a protein per cycle
 double inoutScaling=150.0;

 int lifetime=400;

 int protNum=8; //number of proteins

 double decayRates[]={0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.0,1.0,1.0,1.0,1.0,1.0}; //\in [0..1]
 double saturationValues[]={10,25,50,100,150,200,300,500};
 double bindingProportions[]={0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.0,1.0,1.0,1.0,1.0,1.0}; //\in [0..1]

/*
vector<std::string> split(std::string what, std::string delim){
	unsigned short int index=what.find_first_of(delim);
	vector<std::string> ret(1,"");
	ret.pop_back();
	ret.push_back(what.substr(0,index));
	while(index<string::npos){
		what=what.substr(index+1);
		int index2=what.find_first_of(delim);
		ret.push_back(what.substr(index,index2));
	cout<<">: "<<what.substr(index,index2).c_str()<<endl;
		index=index2;
	}
	return ret;
}
*/

vector<string> split(string& what,
                     string delim)
{
	vector<string> tokens;
    // Skip delimiters at beginning.
    string::size_type lastPos = what.find_first_not_of(delim, 0);
    // Find first "non-delimiter".
    string::size_type pos     = what.find_first_of(delim, lastPos);
    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(what.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = what.find_first_not_of(delim, pos);
        // Find next "non-delimiter"
        pos = what.find_first_of(delim, lastPos);
    }
	return tokens;
}

unsigned short int fromBinary(vector<unsigned short int> bin){
unsigned short int res=0;
for(int i=0;i<bin.size();i++){
  res=res<<1;
  res+=bin[i];
  }
return res;
}

/*currently returns a 3 "bit" array always*/
vector<unsigned short int> toBinary(unsigned short int bin){
vector<unsigned short int> res(3);
for(int i=2;i>=0;i--){
  res[i]=bin % 2;
  bin/=2;
}  
return res;
}

//int array to string conversion ({1,0,1}->"101")
std::string intArrToString(vector<unsigned short int> in){
std::ostringstream ret;
for(int i=0;i<in.size();i++)
  ret<<in[i];
return ret.str();
}

std::string intToString(unsigned short int val)
{
  std::ostringstream stream;
  stream << val;
  return stream.str();
}

//string to int array conversion ("101"->{1,0,1})
vector<unsigned short int> stringToIntArr(std::string in){
vector<unsigned short int> ret(in.length());
for(int i=0;i<ret.size();i++)
  ret[i]=atoi(in.substr(i,1).c_str());
return ret;
}


double lookupDecayRates(vector<unsigned short int> in){
int what=fromBinary(in);
return decayRates[what];
}

double lookupBindingProp(vector<unsigned short int> in){
int what=fromBinary(in);
return bindingProportions[what];
}

double lookupSaturationValue(vector<unsigned short int> in){
int what=fromBinary(in);
return saturationValues[what];
}


KnabeGRN::KnabeGRN(std::string geno)
{
	cout<<"KnabeGRN instance created, genome is: "<<geno<<endl;
	/*cout<<"converted:"<<intArrToString(stringToIntArr(geno))<<endl;
	cout<<"binary 7:"<<fromBinary(toBinary(7))<<endl;
	Sleep(1000);*/
howLong=0;
fitness=9999999999.0;
genome=stringToIntArr(geno);
concentrations.assign(protNum,0.0); //put protNum times 0 into vector
proteinDecay.assign(protNum,0.0);   //put protNum times 0 into vector

vector<std::string> splitGeno=split(intArrToString(genome),"3");//3 is the delimiter of genes
currentGeneNo=splitGeno.size()-1;//minus one as the last compartment is no gene but global vars

geneOutputProtein.assign(currentGeneNo,0); //put currentGeneNo times 0 in vector
geneOutputActivity.assign(currentGeneNo,0);//put currentGeneNo times 0 in vector

int offset=0;

//process "genes"
int count=0;
for(int i=0;i<splitGeno.size()-1;i++){
	if((splitGeno.at(i).length()>3)&&((splitGeno.at(i).find_last_of("2")<splitGeno.at(i).length()-4)||(splitGeno.at(i).find_last_of("2")==string::npos))){//needs to code for a protein at least to qualify as a gene (length at least four bits after the last "2" if there is any "2")
		KnabeGene *t=new KnabeGene(splitGeno.at(i),this);
		genes.push_back(t);//,this);
	}else
		genes.push_back(NULL);
}
cisSitesReceptiveFor.assign(protNum,0);
for(int j=0;j<protNum;j++){   //for every protein
  for(int i=0;i<currentGeneNo;i++){ //for every gene
   if(genes[i]!=NULL)
    //sum up the receptive cis-sites
    cisSitesReceptiveFor[j]+=genes[i]->getNoOfCisSitesForProt(j);
  }
}
offset=genome.size()-39;//this should of course depend on the index of the last 3 instead of fixed value!
for(int i=0;i<protNum;i++){
	vector<unsigned short int> dec;
	for(int j=0;j<4;j++)
		dec.push_back(genome.at(offset+j));
	proteinDecay[i]=lookupDecayRates(dec);
	offset+=4;
}

vector<unsigned short int> bind;
for(int j=0;j<4;j++)
	bind.push_back(genome.at(offset+j));
bindingProportion=lookupBindingProp(bind);
offset+=4;

vector<unsigned short int> sat;
for(int j=0;j<3;j++)
	sat.push_back(genome.at(offset+j));
saturationValue=lookupSaturationValue(sat);

	cout<<"check: "<<proteinDecay[3]<<" "<<bindingProportion<<" "<<saturationValue<<endl;
}






void KnabeGRN::applySaturationCheck(){
//make sure that every protein is not more present than it's
//saturationValue allows! (important: do we call this before decay
//or after decay? Arbitraryly decided for before)
for(int i=0;i<concentrations.size();i++){
	if(concentrations[i]>saturationValue)
    concentrations[i]=saturationValue;
  }
}


double KnabeGRN::process(double in, double in2, double in3){
double ret=process(in);
concentrations[inputProtein2]+=in2*inoutScaling;
concentrations[inputProtein3]+=in3*inoutScaling;
return ret;
}

double KnabeGRN::process(double in){
double res=.0;
/*was used for debug
for(int i=0;i<concentrations.length;i++)
  System.out.print(concentrations[i]+",");
System.out.println("");*/
//1. bind unbound proteins to matching cis sites
//concentrations/cisSitesReceptiveFor := per site fraction
for(int i=0;i<concentrations.size();i++){
  for(int j=0;j<genes.size();j++){
   if(genes[j]!=NULL){
    if(cisSitesReceptiveFor[i]!=0)
     //distribute every cis-site its share
     genes[j]->bindingProtein(i,((double)concentrations[i]*bindingProportion)/((double)cisSitesReceptiveFor[i]));
    else //avoid NaN
     genes[j]->bindingProtein(i,((double)concentrations[i]*bindingProportion));
   }
  }
  //of course this is lost now:
  concentrations[i]-=((double)concentrations[i]*bindingProportion);
}
//2. calculate activation of genes and give result to genes output function
for(int i=0;i<concentrations.size();i++){
  for(int j=0;j<genes.size();j++){
   if(genes[j]!=NULL)
    concentrations[i]+=genes[j]->getOutputForProtein(i);
    }
  }
applySaturationCheck();
//3. decay according to protein specific decay rate
for(int i=0;i<concentrations.size();i++)
  concentrations[i]=concentrations[i]*proteinDecay[i];
//4. output and input
res=concentrations[outputProtein]/inoutScaling;
concentrations[inputProtein]+=in*inoutScaling;
applySaturationCheck();
/*was used for debug
for(int i=0;i<concentrations.length;i++)
  System.out.print(concentrations[i]+",");
System.out.println("");*/
if(isnan(res)) //was used to punish NaN systems, which now hopefully 
  res=99.0;           //nevers happens anyway
return res;
}








KnabeGRN::~KnabeGRN(void)
{
}
