/*
Artificial Genetic Regulatory Network
Copyright (C) 2008 Johannes Knabe
j.f.knabe@herts.ac.uk

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include "KnabeGene.h"

#include <iostream>
#include <sstream>
#include <cmath>
#include <string>
#include <stdexcept>
#include <vector>
#include <windows.h>//for Sleep only!
#include "./KnabeGRN.h"

using namespace std;

 static int stdSlope=5;
 static int stdRange=150;
 static double actiInputCeiling=1000.0;
 void *holder;//of type KnabeGRN

KnabeGene::KnabeGene(std::string bases, void *holder)
{
	this->holder=holder;
	//space for testing:
	KnabeGRN *ttt=(KnabeGRN *)holder;
	cout<<bases.c_str()<<" "<<(ttt->fitness-9999999990.0)<<endl;
/*	vector<unsigned short int> test2=stringToIntArr("14567");
	vector<unsigned short int> testi(3);
	testi.assign(test2.begin()+1,test2.begin()+4);
	cout<<intArrToString(testi)<<endl;
	Sleep(500);*/

	numberOfCisSites=0;
	vector<std::string> splitGene=split(bases,"2");//2 is the delimiter for modules

	for(int i=0;i<splitGene.size()-1;i++){
		if(splitGene[i].length()>3)//needs to code for one protein binding site at least to qualify as a cis site
			numberOfCisSites++;
	}

//proteinSiteActivations=new vector<vector<double> >(0);
vector<unsigned short int> currProt(3,0); //{0,0,0}
for(int i=0;i<numberOfCisSites;i++){
	geneCisReceptorSites.push_back(vector<unsigned short int> (0,0));
	PSAinit.push_back(vector<double> (0,0.0));
}
cerr<<"DEBUG: "<<PSAinit.size()<<endl;
int count=0; int countProts; int bindings;
for(int i=0;i<splitGene.size()-1;i++){
	if(splitGene[i].length()>3){
		bindings=((splitGene[i].length()-1) / 3); //one for every three bit protein binding value
	    geneCisReceptorSites[count].assign(1+bindings,0); //value one for inhibitory or activatory and then one for every three bit protein binding value
		PSAinit[count].assign(bindings,0.0);
		cerr<<"DEBUG2: "<<PSAinit[count].size()<<endl;
		vector<unsigned short int> cis=stringToIntArr(splitGene[i]);
		geneCisReceptorSites[count][0]=cis[cis.size()-1]; //last value of cis site says whether it's activ/inhib
		cis.pop_back();//remove last value
		countProts=0;
		while(cis.size()>2){
			currProt.assign(cis.begin(),cis.begin()+3);//take first three bits
			geneCisReceptorSites[count][1+countProts]=fromBinary(currProt);
			countProts++;//decode, copy and increase counter
			cis.erase(cis.begin(),cis.begin()+3);//remove first three bits
		}//end of while
    count++;
	}//end of if(splitGene[i]...
}//end of for
vector<unsigned short int> glob=stringToIntArr(splitGene[splitGene.size()-1]);
geneOutputActivity=glob[glob.size()-1];  //last bit for activity type
vector<unsigned short int> outp(3);outp.assign(3,0);
for(int i=0;i<3;i++)
	outp[i]=glob.at(glob.size()-4+i);
geneOutputProtein =fromBinary(outp);

proteinSiteActivations=&PSAinit;
}//end of constructor

KnabeGene::~KnabeGene(void)
{
}

double tanhScaled(double x, double range, double floor, double rshift, double slope){
//# ------------------- Move 'S' curve vertically  ------------------- #
//# floorScale is what we add at the end to shift the whole 's' curve up.
double floorScale;
if(floor<-999990.0) //# undef, so we won't tweak output up or down at all
  floorScale = .0;
else{ //# check what floor would be, and adjust to match what we want it to be
  double naturalFloor = -(range/2);
  floorScale = floor - naturalFloor;
}
//# ------------------- Move 'S' curve horizontally------------------- #
//# By reducing value of x, we effectively shift the whole 'S' to the right.
//# Need to do this, if, for example, don't want to get any output before
//# input goes over 0.0.
x = x - rshift;

return ( (((exp(x/slope) - exp(-x/slope)) / 2) / 
          ((exp(x/slope) + exp(-x/slope)) / 2)
         ) 
         * (range/2)
       ) + floorScale;
}

/*
Currently there is only one output Protein per gene, so for all except this one immediately zero will be returned. For the output protein
activations on all cis-sites are summed (+ if activatory, - if inhibitory) and put through the specific tanh derived output function.
*/
double KnabeGene::getOutputForProtein(int which){
if(which!=geneOutputProtein) //We generate only one output protein!
  return 0.0;
double acti=.0;
double temp=.0;
double ret=.0;
KnabeGRN *ttt=(KnabeGRN *)holder;
for(int i=0;i<proteinSiteActivations->size();i++){ //over all cis-sites (one in Quick implementation)
  temp=999999.0;                                    
  for(int j=0;j<proteinSiteActivations->at(i).size();j++){//over all binding sites (AND combo)
    if(proteinSiteActivations->at(i)[j]<temp)             //not relevant for Quick
      temp=proteinSiteActivations->at(i)[j];              //implementation, as we take
    proteinSiteActivations->at(i)[j]*=ttt->proteinDecay[geneCisReceptorSites[i][j+1]];
                                    //decay           //smallest value and there is
    }                                                 //only one. Later AND combination.
  //sum up (+ or -) the activations (NOT and OR combos):
  if(geneCisReceptorSites[i][0]==0) //inhibitory
    acti-=temp;
  else                              //ativatory
    acti+=temp;
  }

double shift=0.0;
if(geneOutputActivity==1) //default 'off' gene
  shift=15.0;
else //default 'on' gene
  shift=-5.0;
//def MathLib.tanhScaled(x, range, floor=nil, rshift=0, slope=2)
//System.out.print("acti of "+acti);
if(acti>actiInputCeiling)
  acti=actiInputCeiling;
ret=tanhScaled(acti,stdRange,.0,shift,stdSlope);
//System.out.println(" results in: "+ret);
if(ret<.0) //concentrations can of course never be negative!
  ret=.0;  //(output is between 0 and floor+range, which is currently 0+150=150)
if(isnan(ret)){
  ret=.0;
  }
return ret;
}

/*
returns the decoded data of one cis site, specified by the param
numbero; which can be between [0,numberOfCisSites[ (like an Array)
the return value will be and int array, with:
1st value: 0 inhibitory or 1 activatory site
2nd value: Protein number which binds to this site
more values: n/a for Quick implementation but in the future this will
             be more of 2nd value, i.e. Protein number which are AND connected
*/
vector<unsigned short int>* KnabeGene::getCisSite(int numbero){
return &geneCisReceptorSites[numbero];
}

/*
This is called one every gene to tell it, what fraction of the global 
concentration of protein "which" binds to every matching cis-site.
(NB: The the number of matching cis-sites on this gene might be 0 to n;
 but every matching site gets "strength" to bind)
[ONLY INPUT TO THE GENE EVERY CYCLE]
*/
void KnabeGene::bindingProtein(int which, double strength){
for(int i=0;i<geneCisReceptorSites.size();i++){
  for(int j=1;j<geneCisReceptorSites[i].size();j++){ //start from 1 as array value one is only the activ/inhib info
    if(geneCisReceptorSites[i][j]==which) //if binding matches
      proteinSiteActivations->at(i)[j-1]+=strength;
    }
  }
}

/*
returns the number of Cis-sites this gene has for a particular protein
(i.e. the protein number "which" matches and can be docked)
in the Quick implementation this is always 0 or 1
*/
int KnabeGene::getNoOfCisSitesForProt(int which){
int ret=0;
for(int i=0;i<geneCisReceptorSites.size();i++){
  vector<unsigned short int> curr=geneCisReceptorSites[i];
  for(int j=1;j<curr.size();j++){//Position 0 is activ./inhib. and we don't care here
    if(curr[j]==which)
      ret++;
  }
}
return ret;
}


