/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "LengthConstraintEnergy.h"
#include <CompuCell3D/Automaton/Automaton.h>

#include <CompuCell3D/Potts3D/Potts3D.h>

using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <string>
using namespace std;

double LengthConstraintEnergy::localEnergy(const Point3D &pt) {
  return 0;
}


#define dot(u,v)   ((u).x * (v).x + (u).y * (v).y + (u).z * (v).z)
#define norm(v)    sqrt((float)dot(v,v))  // norm = length of vector
#define d(u,v)     norm(u-v)       // distance = norm of difference
#define cross(u,v) Point3D( ( (u).y*(v).z-(u).z*(v).y ) , ( (u).z*(v).x-(u).x*(v).z ) , ( (u).x*(v).y-(u).y*(v).x )  )
#define PI 3.14159265
#define isnan(x) ((x) != (x))

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
double LengthConstraintEnergy::changeEnergy(const Point3D &pt,
				  const CellG *newCell,
				  const CellG *oldCell) {

  /// E = lambda * (length - targetLength) ^ 2 
  
  double energy = 0;

  if (oldCell == newCell) return 0;
    
   //as in the original version 
  if (newCell){
    double xPtSum = newCell->xCM;
    double zPtSum = newCell->zCM;
    double xcm = (newCell->xCM / (float) newCell->volume);
    double zcm = (newCell->zCM / (float) newCell->volume);
    double zPtSumSQ = newCell->iXX - newCell->volume*zcm*zcm + 2*zcm*zPtSum;
    double xPtSumSQ = newCell->iZZ - newCell->volume*xcm*xcm + 2*xcm*xPtSum;
    double xzSum = (newCell->iXZ - xcm*zPtSum - zcm*xPtSum + newCell->volume*xcm*zcm) / -1.0; 
    double newXCM = (newCell->xCM + pt.x)/((float)newCell->volume + 1);
    double newZCM = (newCell->zCM + pt.z)/((float)newCell->volume + 1);
    xPtSum += pt.x;
    zPtSum += pt.z;
    xPtSumSQ += pt.x*pt.x;
    zPtSumSQ += pt.z*pt.z;
    xzSum += pt.x*pt.z;
    double newIxx = zPtSumSQ - 2*newZCM*zPtSum + (newCell->volume+1)*newZCM*newZCM;
    double newIzz = xPtSumSQ - 2*newXCM*xPtSum + (newCell->volume+1)*newXCM*newXCM;
    double newIxz = -xzSum + newXCM*zPtSum + newZCM*xPtSum - (newCell->volume+1)*newXCM*newZCM;
/*JK
Point3D targetOrient(80,0,80);

double newLambdaB=0.5*(newIxx+newIzz)+0.5*sqrt((newIxx-newIzz)*(newIxx-newIzz)+4*newIxz*newIxz);
Point3D newOrient(newIxz, 0, (newLambdaB-newIxx));

double currLambdaB=0.5*(newCell->iXX+newCell->iZZ)+0.5*sqrt((newCell->iXX-newCell->iZZ)*(newCell->iXX-newCell->iZZ)+4*newCell->iXZ*newCell->iXZ);
Point3D currOrient(newCell->iXZ, 0, (currLambdaB-newCell->iXX));

energy += 20.0* (d(newOrient,targetOrient)-d(currOrient,targetOrient));
cerr<<"newCell: "<<newOrient<<"  "<<targetOrient<<"  "<<d(newOrient,targetOrient)<<"  "<<currOrient<<"  "<<d(currOrient,targetOrient)<<endl;
*/
double targetLength=31.0;
double targetAngle =0;//PI/2.0
Point3D targetOrient(cos(targetAngle),0,sin(targetAngle));//length=1 !

double newLambdaB=0.5*(newIxx+newIzz)+0.5*sqrt((newIxx-newIzz)*(newIxx-newIzz)+4*newIxz*newIxz);
Point3D newOrient(newIxz, 0, (newLambdaB-newIxx));
double newLength=2*sqrt(newLambdaB);
double newAngleDist=acos(dot(newOrient,targetOrient)/(norm(newOrient)*1.0));

double currLambdaB=0.5*(newCell->iXX+newCell->iZZ)+0.5*sqrt((newCell->iXX-newCell->iZZ)*(newCell->iXX-newCell->iZZ)+4*newCell->iXZ*newCell->iXZ);
Point3D currOrient(newCell->iXZ, 0, (currLambdaB-newCell->iXX));
double currLength=2*sqrt(currLambdaB);
double currAngleDist=acos(dot(currOrient,targetOrient)/(norm(currOrient)*1.0));

if(!isnan(newAngleDist))
  energy += 5.0* newAngleDist;
if(!isnan(currAngleDist))
  energy += 5.0* currAngleDist;

double angleDiff=newAngleDist-currAngleDist;
if(!isnan(angleDiff))
  energy += 10.0* angleDiff;
else
  energy += 10.0;
energy += 5.00* (fabs(targetLength-newLength)-fabs(targetLength-currLength));
cerr << "newCell: "<<angleDiff<<" "<<newLength<<"  "<<norm(newOrient)<<" "<<newOrient<<"  "<<norm(currOrient)<<" "<<currOrient<<"  "<<targetOrient<<" "<<acos(dot(newOrient,targetOrient)/(norm(newOrient)*1.0))<<endl;

/*    double newIxx = zPtSumSQ - 2*newZCM*zPtSum + newCell->volume*newZCM*newZCM + (pt.z - newZCM)*(pt.z - newZCM);
    double newIzz = xPtSumSQ - 2*newXCM*xPtSum + newCell->volume*newXCM*newXCM + (pt.x - newXCM)*(pt.x - newXCM);
    double newIxz = -xzSum + newXCM*zPtSum + newZCM*xPtSum - newCell->volume*newXCM*newZCM - (pt.x - newXCM)*(pt.z - newZCM);*/
    //double currLength = 2*sqrt((float)((0.5*(newCell->iXX + newCell->iZZ)) + .5*sqrt((float)((newCell->iXX - newCell->iZZ)*(newCell->iXX - newCell->iZZ) + 4*(newCell->iXZ)*(newCell->iXZ)))));
    //JK double currLength = 4.0*sqrt(((float)((0.5*(newCell->iXX + newCell->iZZ)) + .5*sqrt((float)((newCell->iXX - newCell->iZZ)*(newCell->iXX - newCell->iZZ) + 4*(newCell->iXZ)*(newCell->iXZ)))))/(float)(newCell->volume));
    //cout << "CURRLENGTH: " << currLength << endl;
    //cout << newCell->iXX << " " << newCell->iZZ << " " << newCell->iXZ << endl;
    /*int aa;
    cin >> aa;*/
    //JK double currEnergy = lengthEnergyParamVector[newCell->type].lambdaLength * (currLength - lengthEnergyParamVector[newCell->type].targetLength)*(currLength - lengthEnergyParamVector[newCell->type].targetLength);
    //double newLength = 2*sqrt((float)((0.5*(newIxx + newIzz)) + .5*sqrt((float)((newIxx - newIzz)*(newIxx - newIzz) + 4*newIxz*newIxz))));
    //JK double newLength = 4.0*sqrt(((float)((0.5*(newIxx + newIzz)) + .5*sqrt((float)((newIxx - newIzz)*(newIxx - newIzz) + 4*newIxz*newIxz))))/(float)(newCell->volume));
    //JK double newEnergy = lengthEnergyParamVector[newCell->type].lambdaLength * (newLength - lengthEnergyParamVector[newCell->type].targetLength)*(newLength - lengthEnergyParamVector[newCell->type].targetLength);
    //JK energy += newEnergy - currEnergy;
  }
  if (oldCell) {
    double xPtSum = oldCell->xCM;
    double zPtSum = oldCell->zCM;
    double xcm = (oldCell->xCM / (float) oldCell->volume);
    double zcm = (oldCell->zCM / (float) oldCell->volume);
    double zPtSumSQ = oldCell->iXX - oldCell->volume*zcm*zcm + 2*zcm*zPtSum;
    double xPtSumSQ = oldCell->iZZ - oldCell->volume*xcm*xcm + 2*xcm*xPtSum;
    double xzSum = (oldCell->iXZ - xcm*zPtSum - zcm*xPtSum + oldCell->volume*xcm*zcm) / -1.0; 
    double newXCM = (oldCell->xCM - pt.x)/((float)oldCell->volume - 1);
    double newZCM = (oldCell->zCM - pt.z)/((float)oldCell->volume - 1);
    xPtSum -= pt.x;
    zPtSum -= pt.z;
    xPtSumSQ -= pt.x*pt.x;
    zPtSumSQ -= pt.z*pt.z;
    xzSum -= pt.x*pt.z;
    double newIxx = zPtSumSQ - 2*newZCM*zPtSum + (oldCell->volume-1)*newZCM*newZCM;
    double newIzz = xPtSumSQ - 2*newXCM*xPtSum + (oldCell->volume-1)*newXCM*newXCM;
    double newIxz = -xzSum + newXCM*zPtSum + newZCM*xPtSum - (oldCell->volume-1)*newXCM*newZCM;
/*JK
Point3D targetOrient(80,0,80);

double newLambdaB=0.5*(newIxx+newIzz)+0.5*sqrt((newIxx-newIzz)*(newIxx-newIzz)+4*newIxz*newIxz);
Point3D newOrient(newIxz, 0, (newLambdaB-newIxx));

double currLambdaB=0.5*(oldCell->iXX+oldCell->iZZ)+0.5*sqrt((oldCell->iXX-oldCell->iZZ)*(oldCell->iXX-oldCell->iZZ)+4*oldCell->iXZ*oldCell->iXZ);
Point3D currOrient(oldCell->iXZ, 0, (currLambdaB-oldCell->iXX));

energy += 20.0* (d(newOrient,targetOrient)-d(currOrient,targetOrient));
*/
double targetLength=31.0;
double targetAngle =0;//PI/2.0
Point3D targetOrient(cos(targetAngle),0,sin(targetAngle));//length=1 !

double newLambdaB=0.5*(newIxx+newIzz)+0.5*sqrt((newIxx-newIzz)*(newIxx-newIzz)+4*newIxz*newIxz);
Point3D newOrient(newIxz, 0, (newLambdaB-newIxx));
double newLength=2*sqrt(newLambdaB);
double newAngleDist=acos(dot(newOrient,targetOrient)/(norm(newOrient)*1.0));

double currLambdaB=0.5*(oldCell->iXX+oldCell->iZZ)+0.5*sqrt((oldCell->iXX-oldCell->iZZ)*(oldCell->iXX-oldCell->iZZ)+4*oldCell->iXZ*oldCell->iXZ);
Point3D currOrient(oldCell->iXZ, 0, (currLambdaB-oldCell->iXX));
double currLength=2*sqrt(currLambdaB);
double currAngleDist=acos(dot(currOrient,targetOrient)/(norm(currOrient)*1.0));

if(!isnan(newAngleDist))
  energy += 5.0* newAngleDist;
if(!isnan(currAngleDist))
  energy += 5.0* currAngleDist;

double angleDiff=newAngleDist-currAngleDist;
if(!isnan(angleDiff))
  energy += 10.0* angleDiff;
else
  energy += 10.0;
energy += 5.00* (fabs(targetLength-newLength)-fabs(targetLength-currLength));

    /*double newIxx = zPtSumSQ - 2*newZCM*zPtSum + oldCell->volume*newZCM*newZCM - (pt.z - newZCM)*(pt.z - newZCM);
    double newIzz = xPtSumSQ - 2*newXCM*xPtSum + oldCell->volume*newXCM*newXCM - (pt.x - newXCM)*(pt.x - newXCM);
    double newIxz = -xzSum + newXCM*zPtSum + newZCM*xPtSum - oldCell->volume*newXCM*newZCM + (pt.x - newXCM)*(pt.z - newZCM);*/

    //double currLength = 2*sqrt((float)((0.5*(oldCell->iXX + oldCell->iZZ)) + .5*sqrt((float)((oldCell->iXX - oldCell->iZZ)*(oldCell->iXX - oldCell->iZZ) + 4*(oldCell->iXZ)*(oldCell->iXZ)))));
    //JK double currLength = 4.0*sqrt(((float)((0.5*(oldCell->iXX + oldCell->iZZ)) + .5*sqrt((float)((oldCell->iXX - oldCell->iZZ)*(oldCell->iXX - oldCell->iZZ) + 4*(oldCell->iXZ)*(oldCell->iXZ)))))/(float)(oldCell->volume));
    /*cout << "CURRLENGTH: " << currLength << endl;
    cout << oldCell->iXX << " " << oldCell->iZZ << " " << oldCell->iXZ << endl;
    cout << "TERM1: " << (0.5*(oldCell->iXX + oldCell->iZZ)) << endl;
    cout << "TERM2: " << .5*sqrt((float)((oldCell->iXX - oldCell->iZZ)*(oldCell->iXX - oldCell->iZZ) + 4*(oldCell->iXZ)*(oldCell->iXZ))) << endl;
    cout << "EXPRESSION: " << ((float)((0.5*(oldCell->iXX + oldCell->iZZ)) + .5*sqrt((float)((oldCell->iXX - oldCell->iZZ)*(oldCell->iXX - oldCell->iZZ) + 4*(oldCell->iXZ)*(oldCell->iXZ)))))/(float)(oldCell->volume) << endl;
    int aa;
    cin >> aa;*/
    //JK double currEnergy = lengthEnergyParamVector[oldCell->type].lambdaLength * (currLength - lengthEnergyParamVector[oldCell->type].targetLength)*(currLength - lengthEnergyParamVector[oldCell->type].targetLength);
    //double newLength = 2*sqrt((float)((0.5*(newIxx + newIzz)) + .5*sqrt((float)((newIxx - newIzz)*(newIxx - newIzz) + 4*newIxz*newIxz))));
    //JK double newLength = 4.0*sqrt(((float)((0.5*(newIxx + newIzz)) + .5*sqrt((float)((newIxx - newIzz)*(newIxx - newIzz) + 4*newIxz*newIxz))))/(float)(oldCell->volume));
    //JK double newEnergy = lengthEnergyParamVector[oldCell->type].lambdaLength * (newLength - lengthEnergyParamVector[oldCell->type].targetLength) * (newLength - lengthEnergyParamVector[oldCell->type].targetLength);
    //JK energy += newEnergy - currEnergy;
  }
  return energy;
}


void LengthConstraintEnergy::readXML(XMLPullParser &in) {
  in.skip(TEXT);
  cout << "HHHH" << endl;
  string type;
  float lambda;
  unsigned int targetLen;
  while (in.check(START_ELEMENT)) {
    cout << "HELLO THERE" << endl;
    if (in.getName() == "LengthEnergyParameters") {
      type = in.getAttribute("CellType").value;
      typeNameVec.push_back(type);
      targetLen=BasicString::parseUInteger(in.getAttribute("TargetLength").value);
      lambda= BasicString::parseDouble(in.getAttribute("LambdaLength").value);
      lengthEnergyParamVector.push_back(LengthEnergyParam(targetLen,lambda));
      in.matchSimple();
    }
    else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
			   "'!", in.getLocation());
    }

    in.skip(TEXT);
  }
}

void LengthConstraintEnergy::writeXML(XMLSerializer &out) {
}


std::string LengthConstraintEnergy::toString(){
   return string("LengthConstraint");
}

void LengthConstraintEnergy::initTypeId(Potts3D * potts,Simulator *simulator){
   unsigned char maxType(0);
   simu=simulator;
   Automaton * automaton=potts->getAutomaton();
   vector<unsigned char> typeIdVec(typeNameVec.size(),0);
   
   vector<LengthEnergyParam> lepVec=lengthEnergyParamVector;//temporaty storage
   //translate type name to type id
   for(unsigned int i =0 ; i < typeNameVec.size() ;++i){
      typeIdVec[i]=automaton->getTypeId(typeNameVec[i]);
      
      if(typeIdVec[i]>maxType)
         maxType=typeIdVec[i];
  }

   //assigning vector lambda targetVol pairs in such a wav that it will be possible to use e.g.vec[cellType].lambda statements
   // note that some of the vector elements migh be left default initialized
   lengthEnergyParamVector.assign(maxType+1,LengthEnergyParam());
  
   for(unsigned int i =0 ; i < typeIdVec.size() ;++i){
      lengthEnergyParamVector[typeIdVec[i]]=lepVec[i];
   }

}
