/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "MitosisPlugin.h"


#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Field3D/Dim3D.h>
#include <CompuCell3D/plugins/CellType/CellTypePlugin.h>
#include <CompuCell3D/plugins/ContactMultiCad/ContactMultiCadEnergy.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <BasicUtils/BasicClassAccessor.h>
#include <CompuCell3D/plugins/ContactMultiCad/ContactMultiCadData.h>
#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/steppables/GRNcells/GRNconcentrationsData.h>
#include <CompuCell3D/steppables/GRNcells/GRNcells.h>

using namespace CompuCell3D;
//define EXP_STL

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <PublicUtilities/StringUtils.h>
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <string>
#include <sstream>
#include <fstream>
#include <map>
#include <cmath>
#include <iostream>
#include <algorithm>
using namespace std;

MitosisPlugin::MitosisPlugin() {potts=0; split=false; GRNconcentrationsDataAccessorPtr=0;}

MitosisPlugin::~MitosisPlugin() {}

Simulator *simu;

void MitosisPlugin::init(Simulator *simulator) {

  potts = simulator->getPotts();
  simu=simulator;


   simulator->getPotts()->registerCellGChangeWatcher(this);
   simulator->getPotts()->registerStepper(this);
   turnOn();
}


void MitosisPlugin::field3DChange(const Point3D &pt, CellG *newCell,
				  CellG *oldCell) {

  // Note: We cannot mitosis the cell here because  updating the field
  // from this function would cause further field3DChange calls which
  // would be out of order for any change listeners who  happened to
  // be after this one in the list.


  if (newCell){
//       cerr<<"this is the mitosis newCell "<<newCell<<endl;
//       cerr<<" DoublingVolume="<<doublingVolume<<endl;
      if(newCell->volume>= doublingVolume){
         split = true;
         splitPt = pt;

      }
  }


}

void MitosisPlugin::turnOn() {on = true;}
void MitosisPlugin::turnOff() {on = false;}

void MitosisPlugin::step() {
	if(!GRNconcentrationsDataAccessorPtr){
		GRNcells * ttt=(GRNcells *) simu->steppableManager.get("GRNcells");
		if(ttt)
			GRNconcentrationsDataAccessorPtr = ttt->getGRNconcentrationsDataAccessorPtr();
	}
	bool didMitosis=doMitosis();
	if(didMitosis){
      updateAttributes();
   }
   
}

//works only as long as there is only one split per MC step
CellG * MitosisPlugin::getChildCell(){return childCell;}
CellG * MitosisPlugin::getParentCell(){return parentCell;}


void MitosisPlugin::updateAttributes(){

   ///copying parameters
   childCell->type = parentCell->type;
   childCell->targetVolume = parentCell->targetVolume/2;//after mitosis reduce target volume
   parentCell->targetVolume= parentCell->targetVolume/2;//in both cells!!!
   childCell->targetSurface= parentCell->targetSurface;
   childCell->angle= parentCell->angle;//don't think this is used anywhere
   childCell->averageConcentration=parentCell->averageConcentration;
   childCell->clusterId=parentCell->clusterId;
   childCell->flag=parentCell->flag;
   childCell->lambdaSurface=parentCell->lambdaSurface;
   childCell->lambdaVolume=parentCell->lambdaVolume;
   childCell->subtype=parentCell->subtype;

   //copy GRN data over
   if(GRNconcentrationsDataAccessorPtr){
	  //copy angle for mitosis + PI/2
      GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->baseAngle+=PI/2.0;
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->baseAngle=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->baseAngle;
      GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->angle+=PI/2.0;
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->angle=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->angle;
	  //copy stretchness values
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->lambdaStretchness=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->lambdaStretchness;
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->scaleSurface=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->scaleSurface;
	  //copy length values
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->lambdaLength=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->lambdaLength;
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->targetLength=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->targetLength;
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->targetLengthRatio=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->targetLengthRatio;
	  //copy concentrations (one vector per GRN)
	  vector<float> * tt=& GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->concentrations;
	  for(int i=0;i<tt->size();i++)
		  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->assignConc(i,tt->at(i));
	  //create proteinSiteActivations (several vectors per GENE)
	  //!N.B.: These are not copied but set to 0.0!
	  GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->proteinSiteActivations.assign(GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->proteinSiteActivations.size(),std::vector<std::vector<double> >(0));
	  for(int i=0;i<GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->proteinSiteActivations.size();i++){
		 if(GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->proteinSiteActivations.size()==0)
			 continue;
		 GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->proteinSiteActivations.at(i).assign(GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->proteinSiteActivations.at(i).size(),std::vector<double>(0));
		 for(int j=0;j<GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->proteinSiteActivations.at(i).size();j++){
			 GRNconcentrationsDataAccessorPtr->get(childCell->extraAttribPtr)->proteinSiteActivations.at(i).at(j).assign(GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->proteinSiteActivations.at(i).at(j).size(),0.0);
		 }
	  }

   }else{
	   cerr<<endl<<endl<<"GRNcells steppable not running!"<<endl<<endl;
   }

}

//Would changing "possibleNew" to a different cell type leave "sameType" separated?
//N.B.: NOT SURE THIS IS WORKING RIGHT NOW
bool MitosisPlugin::lonely(Point3D &possibleNew, Point3D &sameType){
	return false;
/*
	void* cF=potts->getCellFieldG();
    WatchableField3D<CellG *> *fieldG=(WatchableField3D<CellG *> *)cF;
	CellG * nCell; CellG * sameTypeCell; Neighbor neighbor;
	sameTypeCell = fieldG->get(sameType);
	BoundaryStrategy *boundaryStrategy=BoundaryStrategy::getInstance();
	unsigned int maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(1.05);
	for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
		neighbor=boundaryStrategy->getNeighborDirect(sameType,nIdx);
		if(!neighbor.distance){
			//if distance is 0 then the neighbor returned is invalid
				continue;
		}

		if(!((neighbor.pt.x==possibleNew.x)&&(neighbor.pt.y==possibleNew.y)&&(neighbor.pt.z==possibleNew.z))){
			nCell = fieldG->get(neighbor.pt);
			if((sameTypeCell)&&(nCell)&&(sameTypeCell->id==nCell->id)){//if only one neighbor is of the same type we're fine
				return false;
			}
		}
	}//end for
	return true;*/
}

bool MitosisPlugin::doMitosis(){
  bool didMitosis=false;

  if (split && on) {
    split = false;
    
	cellField = potts->getCellFieldG();
	void* cF=potts->getCellFieldG();
    WatchableField3D<CellG *> *_cellField=(WatchableField3D<CellG *> *)cF;
    //reseting poiters to parent and child cell - neessary otherwise may get some strange side effects when mitisis is aborted
    childCell=0;
    parentCell=0;

    CellG *cell = cellField->get(splitPt);
    parentCell=cell;

    ASSERT_OR_THROW("Cell should not be NULL at mitosis point!", cell);

	float angle=0;
	if(GRNconcentrationsDataAccessorPtr){
	  angle=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->angle;
	}else cerr<<"COULD NOT GET ANGLE FOR MITOSIS!!!"<<endl;

	Point3D V0=Point3D(potts->getMinCoordinates());
	angle+=PI/2.0; //mitosis is 90 degree perpendicular to cell orientation
	V0.x+=10.0*cos(angle);
	V0.y+=10.0*sin(angle);

	V0.y-=100;
	V0.x-=100;
	Point3D V1=Point3D(potts->getMinCoordinates());
	V1.y-=100;
	V1.x-=100;

	Point3D V2=V1; V2.z+=10;//for 2D case only, very simple 3D plane

	Plane anglePlane=Plane(V0,V1,V2);

	/*Point3D cm(parentCell->xCM/parentCell->volume,parentCell->yCM/parentCell->volume,parentCell->zCM/parentCell->volume);
	vector<Point3D> cP(0);
	if(findCellPoint(parentCell,cm,1.75)){
		cP.push_back(cm);
		getCellPixels(parentCell,cm,cP);
	}else{cout<<"NO starting point found (Mitosis)!"<<endl; exit(0);}*/
	vector<Point3D> cP=GRNconcentrationsDataAccessorPtr->get(parentCell->extraAttribPtr)->cellPixels;
	vector<PointDist> ttt;
	Point3D B;
	for( int i=0; i<cP.size(); i++){
		float disti=anglePlane.pbase_Plane(cP.at(i),anglePlane,&B);
		ttt.push_back(PointDist(cP.at(i),disti));
		//cerr<<cP.at(i).x<<"/"<<cP.at(i).y<<"/"<<cP.at(i).z<<" : "<<disti<<endl;
	}

	sort( ttt.begin(), ttt.end(), PointDist::cmp );

    CellG *splitCell = 0;
	for( int i=0; i<ttt.size()/2; i++){

            if (splitCell){
			   int sanityCheck=splitCell->volume;
			   _cellField->set(ttt.at(i).P, splitCell);	
			   if(sanityCheck==splitCell->volume){
				   cerr<<"At this point VolumeTracker should have changed the volume (same for other fieldchangewatchers) - Insanity, have to exit"<<endl;
				   splitCell->volume++;//for some reason VolumeTracker doesn't always get it, so we do this update manually
				   parentCell->volume--;
				   exit(1);
			   }
            }
            else{
               splitCell = potts->createCellG(ttt.at(i).P);
               childCell=splitCell;
            }

	}

/* //old splitting routine with arbitrary split
    int volume = cell->volume;
	//cerr<<"here we go splitting! volume: "<<cell->volume<<endl;
    int newVol = 0;
    int targetVol = volume / 2;

	vector<Point3D> ary0Vec;
    vector<Point3D> ary1Vec;

    vector<Point3D> *tmp;;
    vector<Point3D> *ary0=&ary0Vec;
    vector<Point3D> *ary1=&ary1Vec;
    
    ary0->clear();
    ary1->clear();

    // Put the first point in the array
    ary0->push_back(splitPt);

    CellG *splitCell = 0;

    // Do a breadth first search for approximately half of the  cell's
    // volume.
    
	while (ary0->size() > 0 && newVol < targetVol) {
      Point3D n;
      CellG *nCell;
      // Loop over all the points from the last round.
      for (unsigned int i = 0; i < ary0->size(); i++) {
         unsigned int token = 0;
         double distance = 0;

         while (distance <= 3.0) {
         n = cellField->getNeighbor((*ary0)[i], token, distance);
      
          nCell=cellField->get(n);

         bool noFragments=true;

		 if (nCell == cell){   //if(cellField->get(n)==cell)
		CellG * neighborCell; 
		Neighbor neighbor;
		void* cF=potts->getCellFieldG();
		WatchableField3D<CellG *> *fieldG=(WatchableField3D<CellG *> *)cF;
		BoundaryStrategy *boundaryStrategy=BoundaryStrategy::getInstance();
		unsigned int maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(1.05);
		for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
			neighbor=boundaryStrategy->getNeighborDirect(n,nIdx);
			if((!neighbor.distance)||(!noFragments)){
				//if distance is 0 then the neighbor returned is invalid
				continue;
			}
			if(lonely(n,neighbor.pt)){
				noFragments=false;
				break;
			}

		}//end for
		 }

         // Add to split cell
if ((nCell == cell)&&(noFragments)) {//if neighbor is part of old cell...

            ary1->push_back(n);                   //add 3dPoint to new cell

            newVol++;
            if (splitCell){
			   int sanityCheck=splitCell->volume;
               _cellField->set(n, splitCell);	
			   if(sanityCheck==splitCell->volume){
				   cerr<<"At this point VolumeTracker should have changed the volume (same for other fieldchangewatchers) - Insanity, have to exit"<<endl;
				   splitCell->volume++;//for some reason VolumeTracker doesn't always get it, so we do this update manually
				   parentCell->volume--;
				   exit(1);
			   }
            }
            else{
               splitCell = potts->createCellG(n);
               childCell=splitCell;
            }
            if (newVol >= targetVol) break;
         }
		 }

         if (newVol >= targetVol) break;
      }

      // Swap arrays;
      tmp = ary0;
      ary0 = ary1;
      ary1 = tmp;
      ary1->clear();
    }*/
   if(!childCell){
      cerr<<"Fragmented Cell - mitosis aborted"<<endl;
      didMitosis=false;
      return didMitosis;
   }
   
   if(childCell && fabs((float)childCell->volume-parentCell->volume)>2.0){
      cerr<<"cell was fragmented before mitosis, volumes of parent and child cells might significantly differ"<<endl;
      cerr<<"C++ childCell.volume="<<childCell->volume<<" parentCell.volume="<<parentCell->volume<<endl;
      didMitosis=true;
      return didMitosis;
   }

  }else{//this means that mitosis loop was not called
      didMitosis=false;
      return didMitosis;
  }

//Everything went fine, mitosis suceeded witho no side effects
didMitosis=true;
return didMitosis;


}
void MitosisPlugin::readXML(XMLPullParser &in) {
  in.skip(TEXT);

  while (in.check(START_ELEMENT)) {
    if (in.getName() == "DoublingVolume") {
      doublingVolume = BasicString::parseUInteger(in.matchSimple());
      ASSERT_OR_THROW("DoublingVolume must be > 1!", doublingVolume > 1);
      
    } else {
      THROW(string("Unexpected element '") + in.getName() + "'!");
    }
    
    in.skip(TEXT);
  }
}

void MitosisPlugin::writeXML(XMLSerializer &out) {
}



