/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "AdvectionDiffusionSolverFE.h"
#include "FlexibleDiffusionSolverFE.h"
#include "ReactionDiffusionSolverFE_SavHog.h"
#include "FastDiffusionSolver2DFE.h"
//include "ReactionDiffusionFile.h"

#include <CompuCell3D/Simulator.h>
using namespace CompuCell3D;

#include <BasicUtils/BasicPluginProxy.h>

BasicPluginProxy<Steppable, AdvectionDiffusionSolverFE> 
advectionDiffusionSolverProxy("AdvectionDiffusionSolverFE", "Solves advection diffusion equation on the cell field",
	    &Simulator::steppableManager);

BasicPluginProxy<Steppable, FlexibleDiffusionSolverFE> 
flexibleDiffusionSolverProxy("FlexibleDiffusionSolverFE", "Solves diffusion equation on the lattice. Uses Forward Euler method - finite difference",
	    &Simulator::steppableManager);

BasicPluginProxy<Steppable, ReactionDiffusionSolverFE_SavHog> 
reactionDiffusion_SavHogSolverProxy("ReactionDiffusionSolverFE_SavHog", "Solves reaction-diffusion equation on the lattice - used in dictyostelium simulation",
	    &Simulator::steppableManager);

BasicPluginProxy<Steppable, FastDiffusionSolver2DFE> 
fastDiffusionSolverProxy("FastDiffusionSolver2DFE", "Solves diffusion equation on the lattice. Provides limited flexibility but is faster than FlexibleDiffusionSolver however operates only in the xy plane and is 2D only",
	    &Simulator::steppableManager);

//BasicPluginProxy<Steppable, ReactionDiffusionFile>
//reactionDiffusionFileProxy("ReactionDiffusionFile", "Loads from files precalculated reaction-diffusion fields",
//            &Simulator::steppableManager);
	    