/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/



#include "PlasticityEnergy.h"
#include <CompuCell3D/plugins/Plasticity/PlasticityTrackerPlugin.h>

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <iostream>

#define EXP_STL
#include "PlasticityPlugin.h"
using namespace std;

PlasticityPlugin::PlasticityPlugin() : plasticityEnergy(0), cellFieldG(0) {}

PlasticityPlugin::~PlasticityPlugin() {
  if (plasticityEnergy) delete plasticityEnergy;
}

void PlasticityPlugin::init(Simulator *simulator) {
   Potts3D *potts = simulator->getPotts();
   cellFieldG = potts->getCellFieldG();

   plasticityEnergy = new PlasticityEnergy(cellFieldG);
   
   potts->registerEnergyFunctionWithName(plasticityEnergy,"PlasticityEnergy");



}

void PlasticityPlugin::extraInit(Simulator *simulator) {
  Potts3D *potts = simulator->getPotts();
  cellFieldG = potts->getCellFieldG();
  plasticityEnergy->initialize( simulator);
  


   PlasticityTrackerPlugin * trackerPlugin = (PlasticityTrackerPlugin *) Simulator::pluginManager.get("PlasticityTracker"); //this will load PlasticityTracker plugin if it si not already loaded  

   plasticityEnergy->setPlasticityTrackerAccessorPtr(trackerPlugin->getPlasticityTrackerAccessorPtr() );

//   surfaceEnergy->setMaxNeighborIndex( trackerPlugin->getMaxNeighborIndex() );

}


void PlasticityPlugin::readXML(XMLPullParser &in) {
  plasticityEnergy->readXML(in);
}

void PlasticityPlugin::writeXML(XMLSerializer &out) {
  plasticityEnergy->writeXML(out);
}
