/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR1 PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef STRETCHNESSENERGY_H
#define STRETCHNESSENERGY_H

#include <CompuCell3D/Potts3D/EnergyFunction.h>

#include <XMLCereal/XMLSerializable.h>


#include <map>
#include <vector>
#include <set>

template <class T>
class BasicClassAccessor;

#include <BasicUtils/BasicArray.h>
#include <CompuCell3D/steppables/GRNcells/GRNconcentrationsData.h>
#include <CompuCell3D/steppables/GRNcells/GRNcells.h>


namespace CompuCell3D {

  

  template <class T> class Field3D;
  class Point3D;
  class Simulator;

  /** 
   * Calculates surface energy based on a target surface and
   * lambda surface.
   */
  class BoundaryStrategy;

  class StretchnessEnergy: public EnergyFunction, public virtual XMLSerializable {

    Field3D<CellG *> *cellFieldG;

	BasicClassAccessor<GRNconcentrationsData> * GRNconcentrationsDataAccessorPtr;

    unsigned int targetStretchness;
    double lambdaStretchness;
    double scaleSurface;
    BoundaryStrategy *boundaryStrategy;
    unsigned int maxNeighborIndex;

	Simulator *simu;

  public:
    StretchnessEnergy(Field3D<CellG *> *_cellFieldG, Simulator *simulator) :
          cellFieldG(_cellFieldG), targetStretchness(0),lambdaStretchness(0),scaleSurface(1.0),boundaryStrategy(0),maxNeighborIndex(0)
    {simu=simulator;GRNconcentrationsDataAccessorPtr=0;}
    

      
    double diffEnergy(int surface, int diff);

    void setMaxNeighborIndex(unsigned int _maxNeighborIndex){maxNeighborIndex=_maxNeighborIndex;}
//     void setBoundaryStrategy(BoundaryStrategy *_boundaryStrategy){boundaryStrategy=_boundaryStrategy;}

    virtual double localEnergy(const Point3D &pt);
    virtual double changeEnergy(const Point3D &pt, const CellG *newCell,
				const CellG *oldCell);

				
    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
    virtual std::string toString();
  };
};
#endif
