/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "SurfaceLocalFlexPlugin.h"
#include "SurfaceLocalFlexEnergy.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>


#include <iostream>
using namespace std;

SurfaceLocalFlexPlugin::SurfaceLocalFlexPlugin() : potts(0),surfaceLocalFlexEnergyPtr(0) 
{

   

}

SurfaceLocalFlexPlugin::~SurfaceLocalFlexPlugin() 
{
   if (surfaceLocalFlexEnergyPtr) delete surfaceLocalFlexEnergyPtr; 
   surfaceLocalFlexEnergyPtr=0;
}

void SurfaceLocalFlexPlugin::init(Simulator *simulator) {
   
  potts = simulator->getPotts();  
  
  surfaceLocalFlexEnergyPtr=new SurfaceLocalFlexEnergy(potts);

  
  potts->registerEnergyFunctionWithName(surfaceLocalFlexEnergyPtr,"SurfaceLocalFlex");


}

void SurfaceLocalFlexPlugin::extraInit(Simulator *simulator) {
  Potts3D *potts = simulator->getPotts();
  
  

  


  SurfaceTrackerPlugin * trackerPlugin = (SurfaceTrackerPlugin *) Simulator::pluginManager.get("SurfaceTracker"); //this will load SurfaceTracker plugin if it si not already loaded  

  surfaceLocalFlexEnergyPtr->setMaxNeighborIndex( trackerPlugin->getMaxNeighborIndex() );

}


void SurfaceLocalFlexPlugin::readXML(XMLPullParser &in) {
  surfaceLocalFlexEnergyPtr->readXML(in);
}

void SurfaceLocalFlexPlugin::writeXML(XMLSerializer &out) {
  surfaceLocalFlexEnergyPtr->writeXML(out);
}

