/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "SurfaceTrackerPlugin.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <iostream>
using namespace std;

SurfaceTrackerPlugin::SurfaceTrackerPlugin() :  cellFieldG(0),boundaryStrategy(0),maxNeighborIndex(0),maxNeighborDepth(1.1),deeperNeighborsRequested(false) {}

SurfaceTrackerPlugin::~SurfaceTrackerPlugin() {
}

void SurfaceTrackerPlugin::init(Simulator *simulator) {
  Potts3D *potts = simulator->getPotts();
  cellFieldG = potts->getCellFieldG();
  
  potts->registerCellGChangeWatcher(this);
  
  boundaryStrategy=BoundaryStrategy::getInstance();
  maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(maxNeighborDepth);//depth=1.1 - means 1st nearest neighbor

}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SurfaceTrackerPlugin::field3DChange(const Point3D &pt, CellG *newCell,
				  CellG *oldCell) {

  unsigned int token = 0;
  double distance;
  int oldDiff = 0;
  int newDiff = 0;
  CellG *nCell=0;
  Neighbor neighbor; 


  // Count surface difference

//    cerr<<"maxNeighborIndex="<<maxNeighborIndex<<endl;
   for(unsigned int nIdx=0 ; nIdx <= maxNeighborIndex ; ++nIdx ){
      neighbor=boundaryStrategy->getNeighborDirect(const_cast<Point3D&>(pt),nIdx);
      if(!neighbor.distance){
      //if distance is 0 then the neighbor returned is invalid
      continue;
      }
      nCell = cellFieldG->get(neighbor.pt);
      if (newCell == nCell) newDiff--;
      else newDiff++;
   
      if (oldCell == nCell) oldDiff++;
      else oldDiff--;

   }


  if (newCell) newCell->surface += newDiff;
  if (oldCell) oldCell->surface += oldDiff;


}


void SurfaceTrackerPlugin::readXML(XMLPullParser &in) {
   in.skip(TEXT);

  
  while (in.check(START_ELEMENT)) {
    if (in.getName() == "MaxNeighborDistance") {
     deeperNeighborsRequested=true;
     maxNeighborDepth = BasicString::parseDouble(in.matchSimple());

    }
    else {
      throw BasicException(string("Unexpected element '") + in.getName() +
                           "'!", in.getLocation());
    }

    in.skip(TEXT);
  }

  if(deeperNeighborsRequested){//reeveluate maxNEighborIndex only if requested by the user
      boundaryStrategy=BoundaryStrategy::getInstance();
      maxNeighborIndex=boundaryStrategy->getMaxNeighborIndexFromDepth(maxNeighborDepth );//depth=1.1 - means 1st nearest neighbor
  }

}

void SurfaceTrackerPlugin::writeXML(XMLSerializer &out) {
}
