/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "LengthConstraintPlugin.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>


#include <iostream>
using namespace std;

LengthConstraintPlugin::LengthConstraintPlugin() : potts(0) {}

LengthConstraintPlugin::~LengthConstraintPlugin() {}

void LengthConstraintPlugin::init(Simulator *simulator) {
  this->simulator=simulator; 
  potts = simulator->getPotts();  
  potts->registerEnergyFunction(&lengthConstraintEnergy);
  potts->registerCellGChangeWatcher(this);
  //potts->registerStepper(this);

}

void LengthConstraintPlugin::extraInit(Simulator *simulator){
   lengthConstraintEnergy.initTypeId(potts);
}


void LengthConstraintPlugin::field3DChange(const Point3D &pt, CellG *newCell,
				 CellG *oldCell) {
    double xcm, ycm, zcm;
    if (newCell)
    {
       // Assumption: COM and Volume has been updated.
        double xcmOld, zcmOld, xcm, zcm;
        if (newCell->volume > 1) {
          xcmOld = (newCell->xCM - pt.x)/ ((float)newCell->volume - 1);
          zcmOld = (newCell->zCM - pt.z)/ ((float)newCell->volume - 1);
        }
        else
        {
            xcmOld = 0;
            zcmOld = 0;
        }
        xcm = (float) newCell->xCM / (float) newCell->volume;
        zcm = (float) newCell->zCM / (float) newCell->volume;
        double xPtSum = newCell->xCM - pt.x;
        double zPtSum = newCell->zCM - pt.z;
        double zPtSumSQ = newCell->iXX - (newCell->volume - 1)*zcmOld*zcmOld + 2*zcmOld*zPtSum + pt.z*pt.z;
        double xPtSumSQ = newCell->iZZ - (newCell->volume - 1)*xcmOld*xcmOld + 2*xcmOld*xPtSum + pt.x*pt.x;
        double xzSum = ((newCell->iXZ - xcmOld*zPtSum - zcmOld*xPtSum + (newCell->volume - 1)*xcmOld*zcmOld) / -1.0) + pt.x*pt.z;
        xPtSum += pt.x;
        zPtSum += pt.z;
        newCell->iXX = zPtSumSQ - 2*zcm*zPtSum + (newCell->volume)*zcm*zcm;
        newCell->iZZ = xPtSumSQ - 2*xcm*xPtSum + (newCell->volume)*xcm*xcm;
        newCell->iXZ = -xzSum + xcm*zPtSum + zcm*xPtSum - (newCell->volume)*xcm*zcm;
    }  
    if (oldCell)
    {
       // Assumption: COM and Volume has been updated.
       double xcmOld = (oldCell->xCM + pt.x) / ((float)oldCell->volume + 1);
       double zcmOld = (oldCell->zCM + pt.z) / ((float)oldCell->volume + 1);
       xcm = (float) oldCell->xCM / (float) oldCell->volume;
       zcm = (float) oldCell->zCM / (float) oldCell->volume;
       double xPtSum = oldCell->xCM + pt.x;
       double zPtSum = oldCell->zCM + pt.z;
       double xPtSumSQ = oldCell->iZZ - (oldCell->volume + 1)*xcmOld*xcmOld + 2*xcmOld*xPtSum - pt.x*pt.x;
       double zPtSumSQ = oldCell->iXX - (oldCell->volume + 1)*zcmOld*zcmOld + 2*zcmOld*zPtSum - pt.z*pt.z;
       double xzSum = ((oldCell->iXZ - xcmOld*zPtSum - zcmOld*xPtSum + (oldCell->volume + 1)*xcmOld*zcmOld) / -1.0) - pt.x*pt.z;
       xPtSum -= pt.x;
       zPtSum -= pt.z;
       oldCell->iXX = zPtSumSQ - 2*zcm*zPtSum + (oldCell->volume)*zcm*zcm;
       oldCell->iZZ = xPtSumSQ - 2*xcm*xPtSum + (oldCell->volume)*xcm*xcm;
       oldCell->iXZ = -xzSum + xcm*zPtSum + zcm*xPtSum - (oldCell->volume)*xcm*zcm;
    }  
}

/*
void LengthConstraintPlugin::step() {
   
}*/

void LengthConstraintPlugin::readXML(XMLPullParser &in) {
  lengthConstraintEnergy.readXML(in);
}

void LengthConstraintPlugin::writeXML(XMLSerializer &out) {
  lengthConstraintEnergy.writeXML(out);
}
