/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "SurfacePlugin.h"

#include "SurfaceEnergy.h"
#include <CompuCell3D/plugins/SurfaceTracker/SurfaceTrackerPlugin.h>

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <iostream>
using namespace std;

SurfacePlugin::SurfacePlugin() : surfaceEnergy(0), cellFieldG(0) {}

SurfacePlugin::~SurfacePlugin() {
  if (surfaceEnergy) delete surfaceEnergy;
}

void SurfacePlugin::init(Simulator *simulator) {
  Potts3D *potts = simulator->getPotts();
  cellFieldG = potts->getCellFieldG();

  surfaceEnergy = new SurfaceEnergy(cellFieldG);
  potts->registerEnergyFunctionWithName(surfaceEnergy,"SurfaceEnergy");
  


}

void SurfacePlugin::extraInit(Simulator *simulator) {
  Potts3D *potts = simulator->getPotts();
  cellFieldG = potts->getCellFieldG();
  

  


  SurfaceTrackerPlugin * trackerPlugin = (SurfaceTrackerPlugin *) Simulator::pluginManager.get("SurfaceTracker"); //this will load SurfaceTracker plugin if it si not already loaded  

  surfaceEnergy->setMaxNeighborIndex( trackerPlugin->getMaxNeighborIndex() );

}


void SurfacePlugin::readXML(XMLPullParser &in) {
  surfaceEnergy->readXML(in);
}

void SurfacePlugin::writeXML(XMLSerializer &out) {
  surfaceEnergy->writeXML(out);
}
