/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "SurfaceFlexPlugin.h"
#include "SurfaceFlexEnergy.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/plugins/SurfaceTracker/SurfaceTrackerPlugin.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>


#include <iostream>
using namespace std;

SurfaceFlexPlugin::SurfaceFlexPlugin() : potts(0) {}

SurfaceFlexPlugin::~SurfaceFlexPlugin() {
  if (surfaceEnergy) delete surfaceEnergy;
}


void SurfaceFlexPlugin::init(Simulator *simulator) {


  potts = simulator->getPotts();
  cellFieldG = potts->getCellFieldG();


  surfaceEnergy = new SurfaceFlexEnergy(cellFieldG);
  potts->registerEnergyFunctionWithName(surfaceEnergy,"SurfaceEnergy");
  Simulator::pluginManager.get("SurfaceTracker"); //this will load SurfaceTracker plugin if it si not already loaded  


}



void SurfaceFlexPlugin::extraInit(Simulator *simulator){


   cerr<<"potts addr="<<potts<<endl;
   surfaceEnergy->initTypeId(potts);

   cellFieldG = potts->getCellFieldG();
  
   SurfaceTrackerPlugin * trackerPlugin = (SurfaceTrackerPlugin *) Simulator::pluginManager.get("SurfaceTracker"); //this will load SurfaceTracker plugin if it si not already loaded  

   surfaceEnergy->setMaxNeighborIndex( trackerPlugin->getMaxNeighborIndex() );

  



}







void SurfaceFlexPlugin::readXML(XMLPullParser &in) {
  surfaceEnergy->readXML(in);
}

void SurfaceFlexPlugin::writeXML(XMLSerializer &out) {
  surfaceEnergy->writeXML(out);
}
