/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR1 PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef SURFACELOCALFLEXENERGY_H
#define SURFACELOCALFLEXENERGY_H

#include <CompuCell3D/Potts3D/EnergyFunction.h>
#include <XMLCereal/XMLSerializable.h>
#include <CompuCell3D/Potts3D/Cell.h>


namespace CompuCell3D {

  
  	
  /** 
   * Calculates surface energy based on a target volume and
   * lambda volume that are properties of a single cell.
   */
  class SurfaceLocalFlexEnergy: public EnergyFunction, public virtual XMLSerializable {

   Field3D<CellG *> *cellFieldG;
   double scaleSurface;

    BoundaryStrategy *boundaryStrategy;
    unsigned int maxNeighborIndex;

  public:
    SurfaceLocalFlexEnergy():cellFieldG(0),scaleSurface(1.0),boundaryStrategy(0),maxNeighborIndex(0) {}
    void setMaxNeighborIndex(unsigned int _maxNeighborIndex){maxNeighborIndex=_maxNeighborIndex;} 
    SurfaceLocalFlexEnergy(Potts3D *potts);
    double diffLocalFlexEnergy(const CellG *cell, int diff);
    virtual double localEnergy(const Point3D &pt);
       
    virtual double changeEnergy(const Point3D &pt, const CellG *newCell,
				const CellG *oldCell);

				
    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
  };
};
#endif



