#include "DiffusionSolverFE.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <CompuCell3D/Field3D/Field3DImpl.h>
#include <CompuCell3D/Boundary/BoundaryStrategy.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/Field3DIO.h>
#include <BasicUtils/BasicClassGroup.h>




#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <BasicUtils/BasicRandomNumberGenerator.h>
#include <PublicUtilities/StringUtils.h>
#include <string>
#include <cmath>
#include <iostream>
#include <fstream>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// std::ostream & operator<<(std::ostream & out,CompuCell3D::DiffusionData & diffData){
//    
//    
// }


using namespace CompuCell3D;
using namespace std;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
DiffusionSolverFE::DiffusionSolverFE()
 :deltaX(1.0),deltaT(1.0)
{
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
DiffusionSolverFE::~DiffusionSolverFE()
{
   for (int i = 0  ; i < concentrationFieldVector.size() ; ++i){
      if(concentrationFieldVector[i]) {
         delete concentrationFieldVector[i];
         concentrationFieldVector[i]=0;
       }
   }
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::init(Simulator *simulator) {
  simPtr=simulator;
  potts = simulator->getPotts();
  automaton=potts->getAutomaton();
   
  ///getting cell inventory
  cellInventoryPtr=& potts->getCellInventory(); 
  
  ///getting field ptr from Potts3D
  cellFieldG=potts->getCellFieldG();
  fieldDim=cellFieldG->getDim();
  
  

  
  ///setting member function pointers
  diffusePtr=&DiffusionSolverFE::diffuse;
  secretePtr=&DiffusionSolverFE::secrete;
  
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::extraInit(Simulator *simulator){


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::start() {
//     if(diffConst> (1.0/6.0-0.05) ){ //hard coded condtion for stability of the solutions - assume dt=1 dx=dy=dz=1
// 
//       cerr<<"CANNOT SOLVE DIFFUSION EQUATION: STABILITY PROBLEM - DIFFUSION CONSTANT TOO LARGE. EXITING..."<<endl;
//       exit(0);
// 
//    }

   
//cerr<<"GOT HERE BEFORE INITIALIZE FIELD"<<endl;
   dt_dx2=deltaT/(deltaX*deltaX);
   initializeConcentration();
   
//   cerr<<"GOT HERE AFTER INITIALIZE FIELD"<<endl;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DiffusionSolverFE::initializeConcentration(){

   for(unsigned int i = 0 ; i <diffDataVec.size() ; ++i){
      if(diffDataVec[i].concentrationFileName.empty()) continue;

      readConcentrationField(diffDataVec[i].concentrationFileName,concentrationFieldVector[i]);
   }


}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::step(const unsigned int _currentStep) {

    currentStep=_currentStep;

   (this->*secretePtr)();

   (this->*diffusePtr)();
   

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::secreteSingleField(unsigned int idx){


SecretionData & secrData=secrDataVec[idx];


std::map<unsigned char,float>::iterator mitr;
std::map<unsigned char,float>::iterator end_mitr=secrData.typeIdSecrConstMap.end();

CellG *currentCellPtr;

ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
Array3D_t & concentrationArray = concentrationFieldPtr->getContainer();

float currentConcentration;
float secrConst;
float secrConstMedium=0.0;

bool secreteInMedium=false;
//the assumption is that medium has type ID 0
mitr=secrData.typeIdSecrConstMap.find(automaton->getTypeId("Medium"));

if( mitr != end_mitr){
   secreteInMedium=true;
   secrConstMedium=mitr->second;
   
}

Point3D pt;
   for (int z = 1; z < workFieldDim.z -1; z++)
      for (int y = 1; y < workFieldDim.y -1; y++)
         for (int x = 1; x < workFieldDim.x -1; x++){
            currentCellPtr=cellFieldG->get(Point3D(x-1,y-1,z-1));
            currentConcentration = concentrationArray[x][y][z];
            
            if(secreteInMedium && ! currentCellPtr){
               concentrationArray[x][y][z]=currentConcentration+secrConstMedium;
            }

            if(currentCellPtr){
               mitr=secrData.typeIdSecrConstMap.find(currentCellPtr->type);
               if(mitr!=end_mitr){
                  secrConst=mitr->second;
                  concentrationArray[x][y][z]=currentConcentration+secrConst;
               }
            }
         }


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::secrete() {

   for(unsigned int i = 0 ; i < secrDataVec.size() ; ++i ){
      secreteSingleField(i);
/*      for(unsigned int j = 0 ; j <secrDataVec[i].secretionFcnPtrVec.size() ; ++j){
         (this->*secrDataVec[i].secretionFcnPtrVec[j])(i);
      }*/
               
   }

      

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// void DiffusionSolverFE::diffuseSingleField(unsigned int idx){
// 
// /// 'n' denotes neighbor
// 
// ///this is the diffusion equation 
// ///C_{xx}+C_{yy}+C_{zz}=(1/a)*C_{t}
// ///a - diffusivity - diffConst
// 
// ///Finite difference method:
// ///T_{0,\delta \tau}=F*\sum_{i=1}^N T_{i}+(1-N*F)*T_{0}
// ///N - number of neighbors
// ///will have to double check this formula
// 
// 
// 
// Point3D pt;
// 
// float concentrationSum=0.0;
// float updatedConcentration=0.0;
// float currentConcentration=0.0;
// short neighborCounter=0;
// 
// DiffusionData & diffData = diffDataVec[idx];
// float diffConst=diffData.diffConst;
// float decayConst=diffData.decayConst;
// float deltaT=diffData.deltaT;
// float deltaX=diffData.deltaX;
// float dt_dx2=deltaT/(deltaX*deltaX);
//    
//    Neighbor n;
//    BoundaryStrategy * boundaryStrategy=BoundaryStrategy::getInstance();
//    ConcentrationField_t * concentrationField=concentrationFieldVector[idx];
//    ConcentrationField_t * scratchField=concentrationFieldVector[concentrationFieldVector.size()-1];
// 
//    Array3D_t & concentrationArray = concentrationField->getArray3DPtr()->getContainer();
//    Array3D_t & scratchArray = scratchField->getArray3DPtr()->getContainer();
// 
//    
//    for (pt.z = 0; pt.z < fieldDim.z; pt.z++)
//      for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
//        for (pt.x = 0; pt.x < fieldDim.x; pt.x++){
// 
//          currentConcentration = concentrationArray[pt.x][pt.y][pt.z];
// 
//          updatedConcentration=0.0;
//          concentrationSum=0.0;
//          neighborCounter=0;
// 
//          //loop over nearest neighbors
//          //Note the <= is crucial as we want to get element with maxNeighborIndex
//          for (int i = 0  ; i<=maxNeighborIndex ; ++i ){ 
//       
//             n=boundaryStrategy->getNeighborDirect(pt,i);
//             if(!n.distance) //means neighbor is invalid
//                continue;
// 
//             concentrationSum += concentrationArray[n.pt.x][n.pt.y][n.pt.z];
// 
// //             concentrationSum += concentrationArray[pt.x-1][pt.y][pt.z];
// //             concentrationSum += concentrationArray[pt.x+1][pt.y][pt.z];
// //             concentrationSum += concentrationArray[pt.x][pt.y+1][pt.z];
// //             concentrationSum += concentrationArray[pt.x][pt.y+1][pt.z];
// 
//             ++neighborCounter;
//             
//          }
// 
//          updatedConcentration =  dt_dx2*diffConst*(concentrationSum - neighborCounter*currentConcentration)+currentConcentration;
//                                     
//          //processing decay 
//          updatedConcentration-=deltaT*(decayConst*currentConcentration); 
// 
//          
//          scratchArray[pt.x][pt.y][pt.z]=updatedConcentration;//updating scratch
// 
// 
//       }
//       //copy updated values from scratch to concentration field
//       scrarch2Concentration(scratchField,concentrationField);
//    
// 
// }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DiffusionSolverFE::diffuseSingleField(unsigned int idx){

/// 'n' denotes neighbor

///this is the diffusion equation 
///C_{xx}+C_{yy}+C_{zz}=(1/a)*C_{t}
///a - diffusivity - diffConst

///Finite difference method:
///T_{0,\delta \tau}=F*\sum_{i=1}^N T_{i}+(1-N*F)*T_{0}
///N - number of neighbors
///will have to double check this formula



Point3D pt;
Point3D n;

float concentrationSum=0.0;
float updatedConcentration=0.0;
float currentConcentration=0.0;
short neighborCounter=0;

DiffusionData & diffData = diffDataVec[idx];
float diffConst=diffData.diffConst;
float decayConst=diffData.decayConst;
float deltaT=diffData.deltaT;
float deltaX=diffData.deltaX;
float dt_dx2=deltaT/(deltaX*deltaX);
CellG *currentCellPtr;

std::set<unsigned char>::iterator end_sitr_decay=diffDataVec[idx].avoidDecayInIdSet.end();   
bool avoidDecayInMedium=false;

if(diffDataVec[idx].avoidDecayInIdSet.find(automaton->getTypeId("Medium")) != end_sitr_decay){
   avoidDecayInMedium=true;
}

//    BoundaryStrategy * boundaryStrategy=BoundaryStrategy::getInstance();
   ConcentrationField_t * concentrationFieldPtr=concentrationFieldVector[idx];
//    initWorkField(concentrationFieldPtr,&workField); //copy concentration array to workArray

   Array3D_t & workArray = concentrationFieldPtr->getContainer();
   Array3D_t & scratchArray = scratchField.getContainer();
   
//     workArray[20][20][1]=1000.0;
//    cerr<<"workArray[20][20][1]="<<workArray[20][20][1]<<endl;
//    cerr<<"concentrationFieldPtr->get(Point3D(19,19,0))="<<concentrationFieldPtr->get(Point3D(19,19,0))<<endl;
   
   for (int z = 1; z < workFieldDim.z-1; z++)
     for (int y = 1; y < workFieldDim.y-1; y++)
       for (int x = 1; x < workFieldDim.x-1; x++){
         currentCellPtr=cellFieldG->get(Point3D(x-1,y-1,z-1));
         currentConcentration = workArray[x][y][z];

         updatedConcentration=0.0;
         concentrationSum=0.0;
         neighborCounter=0;

         //loop over nearest neighbors
         //Note the <= is crucial as we want to get element with maxNeighborIndex
         
         for (int i = 0  ; i<offsetVec.size() ; ++i ){ 
            Point3D & offset = offsetVec[i];

//             n=pt+offsetVec[i];
//             concentrationSum += workArray[n.x][n.y][n.z];
            concentrationSum += workArray[x+offset.x][y+offset.y][z+offset.z];
            

            ++neighborCounter;
            
         }

//             concentrationSum += workArray[x-1][y][z];
//             concentrationSum += workArray[x+1][y][z];
//             concentrationSum += workArray[x][y+1][z];
//             concentrationSum += workArray[x][y+1][z];
//             neighborCounter=4;

         updatedConcentration =  dt_dx2*diffConst*(concentrationSum - neighborCounter*currentConcentration)+currentConcentration;
                                    
         //processing decay 
         if(currentCellPtr){
            if(diffData.avoidDecayInIdSet.find(currentCellPtr->type)!=end_sitr_decay){
               ;//decay in this type is forbidden
            }else{
               updatedConcentration-=deltaT*(decayConst*currentConcentration);//decay in this type is allowed

            }
         }else{
            if(avoidDecayInMedium){
               ;//decay in Medium is forbidden
            }else{
               updatedConcentration-=deltaT*(decayConst*currentConcentration); //decay in Medium is allowed
//                cerr<<"currentCellPtr="<<currentCellPtr<<endl;
            }
         }
//           updatedConcentration-=deltaT*(decayConst*currentConcentration); 

         
         scratchArray[x][y][z]=updatedConcentration;//updating scratch

      }

//       scratchArray[31][31][1]=3000.0;
//       
//       cerr<<"setting scratchArray to "<<scratchArray[31][31][1]<<endl;
      //copy updated values from scratch to concentration field
       scrarch2Concentration2(&scratchField,concentrationFieldPtr);
   

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::diffuse() {

   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i ){
      diffuseSingleField(i);
   }

      

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::initWorkField(ConcentrationField_t *_concentrationField,ConcentrationField_t *_workField){

   Array3D_t & concentrationArray=_concentrationField->getContainer();
   Array3D_t & workArray=_workField->getContainer();
   //zeroing workField
   for(int x=0 ; x < workFieldDim.x ; ++x)
      for(int y=0 ; y < workFieldDim.y ; ++y)
         for(int z=0 ; z < workFieldDim.z ; ++z){
            workArray[x][y][z]=0.0;
         }

   for(int x=0 ; x < fieldDim.x ; ++x)
      for(int y=0 ; y < fieldDim.y ; ++y)
         for(int z=0 ; z < fieldDim.z ; ++z){
            workArray[x+1][y+1][z+1]=concentrationArray[x][y][z];

         }


}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::scrarch2Concentration( ConcentrationField_t *_scratchField, 
                                                ConcentrationField_t *_concentrationField)
{

Array3D_t & concentrationArray = _concentrationField->getContainer();
Array3D_t & scratchArray = _scratchField->getContainer();


precision_t tempValue;


   //goes everywhere except for lattice border
   for (int z = 1; z < workFieldDim.z-1; z++)
    for (int y = 1; y < workFieldDim.y-1; y++)
      for (int x = 1; x < workFieldDim.x-1; x++){
         
         tempValue=scratchArray[x][y][z];

         concentrationArray[x-1][y-1][z-1]=tempValue;

      }
   

// Array3D_t & concentrationArray = concentrationField->getArray3DPtr()->getContainer();
// Array3D_t & scratchArray = scratchField->getArray3DPtr()->getContainer();
// 
// 
// precision_t tempValue;
// 
//    for (int z = 0; z < fieldDim.z; z++)
//     for (int y = 0; y < fieldDim.y; y++)
//       for (int x = 0; x < fieldDim.x; x++){
//          
//          tempValue=scratchArray[x][y][z];
//          concentrationArray[x][y][z]=tempValue;
//          scratchArray[x][y][z]=0.0;
//          
//          
//       }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::scrarch2Concentration2( ConcentrationField_t *_scratchField, 
                                                ConcentrationField_t *_concentrationField)
{

// Array3D_t & concentrationArray = _concentrationField->getContainer();
// Array3D_t & scratchArray = _scratchField->getContainer();

// cerr<<"BEFORE SWITCH"<<endl;
// 
// cerr<<"concentrationArray[20][20][1]="<<concentrationArray[20][20][1]<<endl;
// cerr<<"scratchArray[20][20][1]="<<scratchArray[20][20][1]<<endl;


_scratchField->switchContainersQuick(*(_concentrationField));


// Array3D_t & concentrationArray2 = _concentrationField->getContainer();
// Array3D_t & scratchArray2 = _scratchField->getContainer();
// 
// 
// cerr<<"AFTER SWITCH"<<endl;
// 
// 
// cerr<<"concentrationArray2[20][20][1]="<<concentrationArray2[20][20][1]<<endl;
// cerr<<"scratchArray2[20][20][1]="<<scratchArray2[20][20][1]<<endl;

   //zeroing scratch field
//    for (int z = 0; z < workFieldDim.z; z++)
//     for (int y = 0; y < workFieldDim.y; y++)
//       for (int x = 0; x < workFieldDim.x; x++){
// 
//          scratchArray[x][y][z]=0.0;
// 
//       }


}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::outputField( std::ostream & _out, Field3DImpl<float> *_concentrationField){
// Point3D pt;
// float tempValue;
// 
//    for (pt.z = 0; pt.z < fieldDim.z; pt.z++)
//     for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
//       for (pt.x = 0; pt.x < fieldDim.x; pt.x++){
//          tempValue=_concentrationField->get(pt);
//          _out<<pt<<" "<<tempValue<<endl;
//       }
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::readConcentrationField(std::string fileName,Field3DImpl<float> *concentrationField){

   ifstream in(fileName.c_str());

   ASSERT_OR_THROW(string("Could not open chemical concentration file '") +
                      fileName + "'!", in.is_open());

   Point3D pt;
   float c;
   while(!in.eof()){
      in>>pt.x>>pt.y>>pt.z>>c;
      if(!in.fail())
         concentrationField->set(pt,c);
   }

   

//    for (pt.z = 0; pt.z < fieldDim.z; pt.z++)
//     for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
//       for (pt.x = 0; pt.x < fieldDim.x; pt.x++){
//           if(pt.x==27&&pt.y==27) 
//          cerr<<"pt="<<pt<<" concentrationField="<<concentrationField->get(pt)<<endl;
//       }   



}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DiffusionSolverFE::readXML(XMLPullParser &in) {

  in.skip(TEXT);
  string concentrationFieldName;
  unsigned int concentrationFieldCounter=0;

  float diffConst;
  float decayConst;
  float secrConst;
  float deltaX;
  float deltaT;

   while(in.check(START_ELEMENT)) {
      if (in.getName() == "DiffusionField") {
         cerr<<"Got Diffusion field tag"<<endl;
         diffDataVec.push_back(DiffusionData());
         secrDataVec.push_back(SecretionData());
         DiffusionData & diffData=diffDataVec[diffDataVec.size()-1];
         SecretionData & secrData=secrDataVec[secrDataVec.size()-1];
         diffData.setAutomaton(potts->getAutomaton());
         secrData.setAutomaton(potts->getAutomaton());
         
         in.match(START_ELEMENT);
         in.skip(TEXT);
       
         while(in.check(START_ELEMENT)){
            cerr<<"AFTER INNER LOOP MATCH START_ELEMENT"<<endl;
            if(in.getName() == "DiffusionData"){
               cerr<<"Will read diffusionData"<<endl;
               
               in.match(START_ELEMENT);
               diffData.readXML(in);
               in.match(END_ELEMENT);
               //in.skip(TEXT);
               cerr<<"FINISHED reading diffusionData"<<endl;
            }
            else if (in.getName() == "SecretionData"){
               cerr<<"Will read secrData"<<endl;
               in.match(START_ELEMENT);
               secrData.readXML(in);
               in.match(END_ELEMENT);
               cerr<<"FINISHED read secrData"<<endl;
            }
            else{
               throw BasicException(string("Unexpected element '") + in.getName() +
                                 "'!", in.getLocation());
            }
           in.skip(TEXT);
         }
         in.skip(TEXT);
         in.match(END_ELEMENT);
         cerr<<"AFTER INNER LOOP "<<endl;
         
/*         in.match(START_ELEMENT);
         diffData.readXML(in);
         in.match(END_ELEMENT);*/
      }
      in.skip(TEXT);
      cerr<<"AFTER OUTER LOOP "<<endl;
      //exit(0);
   }
   


   cerr<<"FlexibleDiffusionSolverFE: extra Init in read XML"<<endl;

   ///allocate fields including scrartch field
   concentrationFieldVector.assign(diffDataVec.size()+1,0);
   workFieldDim=Dim3D(fieldDim.x+2,fieldDim.y+2,fieldDim.z+2); 
   for (int i = 0  ; i < concentrationFieldVector.size() ; ++i){
      concentrationFieldVector[i]=new ConcentrationField_t();
      concentrationFieldVector[i]->setDim(workFieldDim);

   }
//    allocateDiffusableFieldVector(diffDataVec.size()+1,fieldDim); //+1 is for additional scratch field
   
   ///first "translate" type names to type id's
//    Automaton *automaton=potts->getAutomaton();
//    for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
//       for(unsigned int j = 0 ; j < diffDataVec[i].avoidTypeVec.size() ; ++j){
//          diffDataVec[i].avoidTypeIdSet.insert(automaton->getTypeId(diffDataVec[i].avoidTypeVec[j]));
//       }
//       for(unsigned int j = 0 ; j < diffDataVec[i].secreteTypeVec.size() ; ++j){
//          diffDataVec[i].secreteTypeIdSet.insert(automaton->getTypeId(diffDataVec[i].secreteTypeVec[j]));
//       }
//    }

   ///assign vector of field names
   concentrationFieldNameVector.assign(diffDataVec.size(),string(""));

   cerr<<"diffDataVec.size()="<<diffDataVec.size()<<endl;
   
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
      concentrationFieldNameVector[i] = diffDataVec[i].fieldName;
      cerr<<" concentrationFieldNameVector[i]="<<concentrationFieldNameVector[i]<<endl;
   }
   
   //register fields once they have been allocated
   for(unsigned int i = 0 ; i < diffDataVec.size() ; ++i){
      simPtr->registerConcentrationField(concentrationFieldNameVector[i] , concentrationFieldVector[i]);
      cerr<<"registring field: "<<concentrationFieldNameVector[i]<<" field address="<<concentrationFieldVector[i]<<endl;
   }

   
   //obtaining maxNeighborIndex  - used in iterating over neighboring pixels
   maxNeighborIndex=BoundaryStrategy::getInstance()->getMaxNeighborIndexFromDepth(1.1);
   const std::vector<Point3D> & offsetVecRef=BoundaryStrategy::getInstance()->getOffsetVec();

   for(int i = 0 ; i <= maxNeighborIndex ; ++i){
      offsetVec.push_back(offsetVecRef[i]);
//       cerr<<"offsetVec["<<i<<"]="<<offsetVec[i]<<endl;
   }
   
//    cerr<<"THIS IS HERE"<<endl;
//    Array3DBorders<precision_t> array;
//    float a=0.0;
//    Dim3D fieldDimTry=Dim3D(2,2,2);
//    cerr<<" THIS IS DIM PASSED TO ARRAY "<<fieldDimTry<<endl;
//    array.allocateArray(fieldDimTry,a);
//    cerr<<" THIS IS INTERNAL DIM="<<array.getInternalDim()<<endl;
//    exit(0);

   //Now will create field with one pixel wide layer around.

   workField.setDim(workFieldDim);
   scratchField.setDim(workFieldDim);
   cerr<<"ALLOCATED ALL FIELDS"<<endl;

//   cerr<<"GOT HERE"<<endl;
   
   //exit(0);
    

}

