#include "ReactionDiffusionSolverFE_SavHog.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <CompuCell3D/Field3D/Field3DImpl.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <BasicUtils/BasicClassGroup.h>



#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <BasicUtils/BasicRandomNumberGenerator.h>
#include <string>
#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>

using namespace CompuCell3D;
using namespace std;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ReactionDiffusionSolverFE_SavHog::ReactionDiffusionSolverFE_SavHog()
 : DiffusableVector<float>(),deltaX(1.0),deltaT(1.0)
{
   imposeDiffusionBox=false;
   dumpFrequency=0;
//    C1=20;
//    C2=3;
//    C3=15;
//    c1=0.0065;
//    c2=0.841;
//    a=0.15;
//    k=3.5;
//    b=0.35;
//    eps1=0.5;
//    eps2=0.0589;
//    eps3=0.5;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ReactionDiffusionSolverFE_SavHog::~ReactionDiffusionSolverFE_SavHog()
{

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReactionDiffusionSolverFE_SavHog::init(Simulator *simulator) {
  simPtr=simulator;
  potts = simulator->getPotts();
  automaton=potts->getAutomaton();
   
  ///getting cell inventory
  cellInventoryPtr=& potts->getCellInventory(); 
  
  ///getting field ptr from Potts3D
  cellFieldG=potts->getCellFieldG();
  fieldDim=cellFieldG->getDim();
  
  ///setting member function pointers
  diffusePtr=&ReactionDiffusionSolverFE_SavHog::diffuse;



  
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReactionDiffusionSolverFE_SavHog::start() {

   dt_dx2=deltaT/(deltaX*deltaX);
   initializeConcentration();
   
   
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ReactionDiffusionSolverFE_SavHog::initializeConcentration(){

   
   BasicRandomNumberGenerator *rand = BasicRandomNumberGenerator::getInstance();
   

   CellInventory::cellInventoryIterator cInvItr;

   CellG * cell;
   map<CellG *,float> concentrationMap;
   map<CellG *,float> refractorinessMap;

   float concentrationMin=0.5;
   float concentrationMax=1.0;
   
   float refractorinessMin=1.0;
   float refractorinessMax=2.0;
   
   float randomConcentration;
   float randomRefractoriness;
   
   
   float x,y,z;
   
   for(cInvItr=cellInventoryPtr->cellInventoryBegin() ; cInvItr !=cellInventoryPtr->cellInventoryEnd() ;++cInvItr ){

      cell=*cInvItr;
      if(cell->type!=automaton->getTypeId("Ground") && cell->type!=automaton->getTypeId("Wall")){
      // do not put anything in the groundCell or wall
         randomConcentration=fabs(concentrationMax-concentrationMin)*rand->getRatio()+concentrationMin;
         randomRefractoriness=fabs(refractorinessMax-refractorinessMin)*rand->getRatio()+refractorinessMin;
         
         concentrationMap.insert(make_pair(cell,randomConcentration));
         refractorinessMap.insert(make_pair(cell,randomRefractoriness));
      }
      
   }
   Point3D pt;
   CellG * currentCellPtr;
   map<CellG *,float>::iterator concentrationMapItr;
   map<CellG *,float>::iterator refractorinessMapItr;

   Array3D_t & concentrationArray0 = concentrationFieldVector[0]->getContainer();
   Array3D_t & concentrationArray1 = concentrationFieldVector[1]->getContainer();

   
   //once we have decided about the initial values we need to initialize lattice
   for (int z = 1; z < workFieldDim.z-1; z++)
      for (int y = 1; y < workFieldDim.y-1; y++)
         for (int x = 1; x < workFieldDim.x-1; x++){

            pt=Point3D(x-1,y-1,z-1);
            currentCellPtr=cellFieldG->get(pt);
            if(!currentCellPtr) continue;///medium does not participate in FN equations
            
            //only autocycling cells will have concentration and refractoriness initialized
            if(currentCellPtr->type!=automaton->getTypeId("Autocycling"))continue;
            
            concentrationMapItr=concentrationMap.find(currentCellPtr);
            if(concentrationMapItr!=concentrationMap.end()){///setting all pixels belonging to given cell to same concentr.
               concentrationArray0[x][y][z]=concentrationMapItr->second;
            }

            refractorinessMapItr=refractorinessMap.find(currentCellPtr);
            if(refractorinessMapItr!=refractorinessMap.end()){///setting all pixels belonging to given cell to same refract.
               concentrationArray1[x][y][z]=refractorinessMapItr->second;
            }


   }



}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReactionDiffusionSolverFE_SavHog::step(const unsigned int _currentStep) {
   currentStep=_currentStep;
   
   (this->*diffusePtr)();

   if(dumpFrequency && !(_currentStep % dumpFrequency )){
      dumpResults(_currentStep);
   }

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReactionDiffusionSolverFE_SavHog::diffuse() {
/// 'n' denotes neighbor

///this is the diffusion equation 
///C_{xx}+C_{yy}+C_{zz}=(1/a)*C_{t}
///a - diffusivity - diffConst

///Finite difference method:
///T_{0,\delta \tau}=F*\sum_{i=1}^N T_{i}+(1-N*F)*T_{0}
///N - number of neighbors
///will have to double check this formula


Point3D pt, n;
unsigned int token = 0;
double distance;
CellG * currentCellPtr=0,*nCell=0;

short currentCellType=0;
float concentrationSum=0.0;
float updatedConcentration=0.0;

float currentRefract=0.0;//refractoriness
float updatedRefract=0.0;//refractoriness

float currentConcentration=0.0;
short neighborCounter=0;

unsigned char autocyclingType=automaton->getTypeId("Autocycling");
unsigned char presporeType=automaton->getTypeId("Prespore");
unsigned char prestalkType=automaton->getTypeId("Prestalk");
unsigned char wallType=automaton->getTypeId("Wall");



float F;/// F=a*\delta \tau / (\delta x)^2
        /// in the simplest case (\delta*\tau=1 \delta x=1) F=a;

   Array3D_t & concentrationArray0 = concentrationFieldVector[0]->getContainer();
   Array3D_t & concentrationArray1 = concentrationFieldVector[1]->getContainer();
   Array3D_t & scratchArray = concentrationFieldVector[2]->getContainer();


   F=diffConst;
                    
   for (int z = 1; z < workFieldDim.z-1; z++)
    for (int y = 1; y < workFieldDim.y-1; y++)
      for (int x = 1; x < workFieldDim.x-1; x++){
         pt=Point3D(x-1,y-1,z-1);
         currentCellPtr=cellFieldG->get(pt);
         /// go to the next iteration in case you get Medium cell - medium does not participate in the diffusion
         ///(model dependent may be changed if necessary
         if(!currentCellPtr){
            
            if(imposeDiffusionBox)
               if(!insideDiffusionBox(pt)) continue;
               
          }     
         updatedConcentration=0.0;
         updatedRefract=0.0;
         concentrationSum=0.0;
         neighborCounter=0;

         ///loop over nearest neighbors
         for (int i = 0  ; i<offsetVec.size() ; ++i ){ 
            Point3D & offset = offsetVec[i];
            n=pt+offset;
            nCell = cellFieldG->get(n); //even if n is "out of the lattice" the returned pointer will be zero which is 
                                        //fine in this case

            if(imposeDiffusionBox)
               if(!insideDiffusionBox(n) && !nCell) continue;

            concentrationSum += concentrationArray0[x+offset.x][y+offset.y][z+offset.z];
            ++neighborCounter;
            
         }
    
         currentConcentration = concentrationArray0[x][y][z];
         currentRefract = concentrationArray1[x][y][z];

         if(
            currentCellPtr &&
            (
               currentCellPtr->type==autocyclingType||
               currentCellPtr->type==presporeType||
               currentCellPtr->type==prestalkType
            )
            
         ){
         
            updatedConcentration =  dt_dx2*F*(concentrationSum - neighborCounter*currentConcentration)
                                    -deltaT*(f(currentConcentration)+currentRefract)
                                    +currentConcentration;
         }
         else {
            if(currentCellPtr && currentCellPtr->type==wallType){
               updatedConcentration =currentConcentration;
            }else {

               updatedConcentration =  dt_dx2*F*(concentrationSum - neighborCounter*currentConcentration)
                                    +currentConcentration;
            }
         
         }
         

         scratchArray[x][y][z]=updatedConcentration;///updating scratch


         ///Refractoriness
         
         if(currentCellPtr && currentCellPtr->type==autocyclingType){
            updatedRefract = deltaT*eps(currentConcentration)*(k*currentConcentration-b-currentRefract)+currentRefract;
            concentrationArray1[x][y][z]=updatedRefract;///updating Refractoriness
         }
         else if(currentCellPtr &&
                  (currentCellPtr->type==prestalkType || currentCellPtr->type==presporeType ))
         {
            updatedRefract = deltaT*eps(currentConcentration)*(k*currentConcentration-currentRefract)+currentRefract;///remove -b
            concentrationArray1[x][y][z]=updatedRefract;///updating Refractoriness
         }
         else{
            
         }

      }
      ///copy updated values from scratch to concentration field   
      scrarch2Concentration(concentrationFieldVector[2],concentrationFieldVector[0]);
      

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReactionDiffusionSolverFE_SavHog::scrarch2Concentration( ConcentrationField_t *scratchField, ConcentrationField_t *concentrationField){

scratchField->switchContainersQuick(*(concentrationField));


}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReactionDiffusionSolverFE_SavHog::outputField( std::ostream & _out, ConcentrationField_t *_concentrationField){
Point3D pt;
float tempValue;

   for (pt.z = 0; pt.z < fieldDim.z; pt.z++)
    for (pt.y = 0; pt.y < fieldDim.y; pt.y++)
      for (pt.x = 0; pt.x < fieldDim.x; pt.x++){
         tempValue=_concentrationField->get(pt);
         _out<<pt<<" "<<tempValue<<endl;
      }
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ReactionDiffusionSolverFE_SavHog::dumpResults(unsigned int _step){
   ofstream out;
   ostringstream fileNameActivator;
   fileNameActivator<<"Activator"<<_step<<".txt";
   out.open(fileNameActivator.str().c_str());

   outputField(out,concentrationFieldVector[0]);
   out.close();

   ostringstream fileNameInhibitor;
   fileNameInhibitor<<"Inhibitor"<<_step<<".txt";
   out.open(fileNameInhibitor.str().c_str());

   outputField(out,concentrationFieldVector[1]);

   out.close();

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void ReactionDiffusionSolverFE_SavHog::readXML(XMLPullParser &in) {
  in.skip(TEXT);
  string concentrationFieldName;
  unsigned int concentrationFieldCounter=0;
  workFieldDim=Dim3D(fieldDim.x+2,fieldDim.y+2,fieldDim.z+2);
  bool numberOfFieldsDeclared=false;
    
  while (in.check(START_ELEMENT)) {
    if (in.getName() == "DiffusionConstant") {
      diffConst = BasicString::parseDouble(in.matchSimple());

    }
    else if (in.getName() == "DecayConstant") {
      decayConst = BasicString::parseDouble(in.matchSimple());

    }
    else if (in.getName() == "DeltaX") {
      deltaX = BasicString::parseDouble(in.matchSimple());
    }
    else if (in.getName() == "DeltaT") {
      deltaT = BasicString::parseDouble(in.matchSimple());
    }
    else if (in.getName() == "MaxDiffusionZ") {
       maxDiffusionZ= BasicString::parseUInteger(in.matchSimple());
    }

    else if (in.getName() == "DumpResults") {
       dumpFrequency = BasicString::parseUInteger(in.getAttribute("Frequency").value);
       in.matchSimple();
    }


    else if (in.getName() == "fFunctionParameters") {
       C1 = BasicString::parseDouble(in.getAttribute("C1").value);
       C2 = BasicString::parseDouble(in.getAttribute("C2").value);
       C3 = BasicString::parseDouble(in.getAttribute("C3").value);
       a = BasicString::parseDouble(in.getAttribute("a").value);
       in.matchSimple();
    }

    else if (in.getName() == "epsFunctionParameters") {
       eps1 = BasicString::parseDouble(in.getAttribute("eps1").value);
       eps2 = BasicString::parseDouble(in.getAttribute("eps2").value);
       eps3 = BasicString::parseDouble(in.getAttribute("eps3").value);
       in.matchSimple();
    }
    else if (in.getName() == "IntervalParameters") {
       c1 = BasicString::parseDouble(in.getAttribute("c1").value);
       c2 = BasicString::parseDouble(in.getAttribute("c2").value);
       in.matchSimple();
    }
    else if (in.getName() == "RefractorinessParameters") {
       k = BasicString::parseDouble(in.getAttribute("k").value);
       b = BasicString::parseDouble(in.getAttribute("b").value);
       in.matchSimple();
    }

    else if (in.getName() == "MinDiffusionBoxCorner") {
      minDiffusionBoxCorner.readXML(in);
      in.matchSimple();
    }
    
    else if (in.getName() == "MaxDiffusionBoxCorner") {
      maxDiffusionBoxCorner.readXML(in);
      in.matchSimple();
    }

                
    else if (in.getName() == "NumberOfFields") {
       numberOfFields = BasicString::parseUInteger(in.matchSimple());
      ///constructing concentration and scratch fields
      allocateDiffusableFieldVector(3, workFieldDim);/// c,r, scratch fields
      numberOfFieldsDeclared=true;
    }
    else if (in.getName() == "FieldName") {
      concentrationFieldName = in.matchSimple();
      ASSERT_OR_THROW("You are trying to define more field that you declared!",concentrationFieldCounter<=numberOfFields);
      ASSERT_OR_THROW("You need to declare how many fields you will be using.\n Use <NumberOfFields>N</NumberOfFields> syntax before listing any fields.\n N denotes number of fields",numberOfFieldsDeclared);

      concentrationFieldNameVector[concentrationFieldCounter] = concentrationFieldName;
      ///register concentration field to make it accesible from external viewers etc.

      
      simPtr->registerConcentrationField(concentrationFieldName , concentrationFieldVector[concentrationFieldCounter]);
      ++concentrationFieldCounter;
    }
    
     else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
            "'!", in.getLocation());
    }

    in.skip(TEXT);
  }

  if(minDiffusionBoxCorner==maxDiffusionBoxCorner)
      imposeDiffusionBox=false;
  else
      imposeDiffusionBox=true;

  

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
