/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "PIFInitializer.h"
#include <CompuCell3D/Automaton/Automaton.h>
#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Potts3D/TypeTransition.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <CompuCell3D/Field3D/Dim3D.h>

using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>

#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>

#include <string>
#include <map>
#include <sstream>

using namespace std;


PIFInitializer::PIFInitializer() :
  potts(0), pifname("") {}

PIFInitializer::PIFInitializer(string filename) :
  potts(0), pifname(filename) {}

void PIFInitializer::init(Simulator *simulator) {
   //cerr<<"inside init of PIFInitializer"<<endl;
  potts = simulator->getPotts();
}

void PIFInitializer::start() {

   //cerr<<"n\n\nLOADED SUCCESSFULLY PIFINITIALIZER  \n\n\n\n"<<endl;

  std::ifstream piffile(pifname.c_str(), ios::in);
  ASSERT_OR_THROW(string("Could not open\n"+pifname+"\nMake sure it exists and is in correct directory"),piffile.good());
  Field3D<CellG *> *cellFieldG = potts->getCellFieldG();
  ASSERT_OR_THROW("initField() Cell field cannot be null!", cellFieldG);

  Dim3D dim = cellFieldG->getDim();

  int spin;
  int clusterId;
  std::string celltype;
  std::string first;
  std::string second;
  std::string line;

  int xLow, xHigh, yLow, yHigh, zLow, zHigh;
  std::map<int, Point3D> spinMap; // Used to check if a cell of the same spin is
                             // listed twice.
  
  Point3D cellPt;
  CellG* cell;

  TypeTransition * typeTransitionPtr=potts->getTypeTransition();


        getline(piffile,line);
        istringstream pif(line);
        pif >> first >> second;
        cout << "First: " << first << " Second: " << second << "\n";
        if(second == "Clusters") {
                cerr << "Clusters Included" << "\n";
                while(getline(piffile,line)) {
                        istringstream pif(line);
                        pif >> clusterId>> spin >> celltype >> xLow;
//             cerr << "  Cluster Id:  " <<clusterId<< "  Spin: " << spin 
//                << "  Type: " <<  celltype <<"\n";

     ASSERT_OR_THROW("PIF reader: xLow out of bounds", xLow >= 0 && xLow < dim.x);
     pif >> xHigh;
     ASSERT_OR_THROW("PIF reader: xHigh out of bounds.", xHigh >= 0 && xHigh < dim.x);
     ASSERT_OR_THROW("PIF reader: xHigh is smaller than xLow.", xHigh >= xLow); 
     pif >> yLow;
     ASSERT_OR_THROW("PIF reader: yLow out of bounds.", yLow >= 0 && yLow < dim.y);
     pif >> yHigh;   
     ASSERT_OR_THROW("PIF reader: yHigh out of bounds.", yHigh >= 0 && yHigh < dim.y);
     ASSERT_OR_THROW("PIF reader: yHigh is smaller than yLow.", yHigh >= yLow);
     pif >> zLow;
     ASSERT_OR_THROW("PIF reader: zLow out of bounds.", zLow >= 0 && zLow < dim.z);
     pif >> zHigh;
     ASSERT_OR_THROW("PIF reader: zHigh out of bounds.", zHigh >= 0 && zHigh < dim.z);
     ASSERT_OR_THROW("PIF reader: zHigh is smaller than xLow.", zHigh >= zLow);
                     if (spinMap.count(spin) != 0) // Spin multiply listed
     {
        for (cellPt.z = zLow; cellPt.z <= zHigh; cellPt.z++)
           for (cellPt.y = yLow; cellPt.y <= yHigh; cellPt.y++)
               for (cellPt.x = xLow; cellPt.x <= xHigh; cellPt.x++)
                   cellFieldG->set(cellPt, cellFieldG->get(spinMap[spin]));
     }
     else // First time for this spin, we need to create a new cell
     {
        spinMap[spin] = Point3D(xLow, yLow, zLow);
        cell = potts->createCellG(Point3D(xLow, yLow, zLow));
        
        for (cellPt.z = zLow; cellPt.z <= zHigh; cellPt.z++)
            for (cellPt.y = yLow; cellPt.y <= yHigh; cellPt.y++)
                for (cellPt.x = xLow; cellPt.x <= xHigh; cellPt.x++)
                    cellFieldG->set(cellPt, cell);
        
//         cell->type=potts->getAutomaton()->getTypeId(celltype);             

         //cerr << "CELLTYPE: " << celltype << "\n";
         //cerr << "CLUSTERID: " << clusterId << "\n";
        typeTransitionPtr->setType( cell,potts->getAutomaton()->getTypeId(celltype));
        cell->clusterId=clusterId;
        //cerr << "1. Cell Type from cell->:  " <<(int)cell->type<< "\n";
        //cerr << "1. ClusterID from cell->:  " <<(int)cell->clusterId<< "\n";

        
     }


}
        }
                else {
                //cerr << "Only Cell Types" << "\n";
                        pif >> xLow;
                int tmp = atoi(first.c_str());
                spin = tmp;
                celltype = second;
                //cerr << "spin: " << spin << " celltype: : " << celltype << 
                //     " xLow: " << xLow << endl;
     ASSERT_OR_THROW("PIF reader: xLow out of bounds", xLow >= 0 && xLow < dim.x);
     pif >> xHigh;
     ASSERT_OR_THROW("PIF reader: xHigh out of bounds.", xHigh >= 0 && xHigh < dim.x);
     ASSERT_OR_THROW("PIF reader: xHigh is smaller than xLow.", xHigh >= xLow); 
     pif >> yLow;
     ASSERT_OR_THROW("PIF reader: yLow out of bounds.", yLow >= 0 && yLow < dim.y);
     pif >> yHigh;   
     ASSERT_OR_THROW("PIF reader: yHigh out of bounds.", yHigh >= 0 && yHigh < dim.y);
     ASSERT_OR_THROW("PIF reader: yHigh is smaller than yLow.", yHigh >= yLow);
     pif >> zLow;
     ASSERT_OR_THROW("PIF reader: zLow out of bounds.", zLow >= 0 && zLow < dim.z);
     pif >> zHigh;
     ASSERT_OR_THROW("PIF reader: zHigh out of bounds.", zHigh >= 0 && zHigh < dim.z);
     ASSERT_OR_THROW("PIF reader: zHigh is smaller than xLow.", zHigh >= zLow);

                     if (spinMap.count(spin) != 0) // Spin multiply listed
     {
        for (cellPt.z = zLow; cellPt.z <= zHigh; cellPt.z++)
           for (cellPt.y = yLow; cellPt.y <= yHigh; cellPt.y++)
               for (cellPt.x = xLow; cellPt.x <= xHigh; cellPt.x++)
                   cellFieldG->set(cellPt, cellFieldG->get(spinMap[spin]));
     }
     else // First time for this spin, we need to create a new cell
     {
        spinMap[spin] = Point3D(xLow, yLow, zLow);
        cell = potts->createCellG(Point3D(xLow, yLow, zLow));
        
        for (cellPt.z = zLow; cellPt.z <= zHigh; cellPt.z++)
            for (cellPt.y = yLow; cellPt.y <= yHigh; cellPt.y++)
                for (cellPt.x = xLow; cellPt.x <= xHigh; cellPt.x++)
                    cellFieldG->set(cellPt, cell);
        

        typeTransitionPtr->setType( cell,potts->getAutomaton()->getTypeId(celltype));
//cerr << "1. Cell Type from cell->:  " <<(int)cell->type<< "\n";
// cerr << "getline(pif,line): " << getline(pif,line) << endl;
//    cerr << "getline(piffline,line): " <<  getline(piffile,line) << endl;
     }
                while(getline(piffile,line)) {
                  //cerr << "PINGPINGPINGPINGP5Ng" << endl;
                        istringstream pif(line);
                        pif >> spin >> celltype >> xLow;
                //cerr << "spin: " << spin << " celltype: : " << celltype << 
                //     " xLow: " << xLow << endl;
     ASSERT_OR_THROW("PIF reader: xLow out of bounds", xLow >= 0 && xLow < dim.x);
     pif >> xHigh;
     ASSERT_OR_THROW("PIF reader: xHigh out of bounds.", xHigh >= 0 && xHigh < dim.x);
     ASSERT_OR_THROW("PIF reader: xHigh is smaller than xLow.", xHigh >= xLow); 
     pif >> yLow;
     ASSERT_OR_THROW("PIF reader: yLow out of bounds.", yLow >= 0 && yLow < dim.y);
     pif >> yHigh;   
     ASSERT_OR_THROW("PIF reader: yHigh out of bounds.", yHigh >= 0 && yHigh < dim.y);
     ASSERT_OR_THROW("PIF reader: yHigh is smaller than yLow.", yHigh >= yLow);
     pif >> zLow;
     ASSERT_OR_THROW("PIF reader: zLow out of bounds.", zLow >= 0 && zLow < dim.z);
     pif >> zHigh;
     ASSERT_OR_THROW("PIF reader: zHigh out of bounds.", zHigh >= 0 && zHigh < dim.z);
     ASSERT_OR_THROW("PIF reader: zHigh is smaller than xLow.", zHigh >= zLow);

                     if (spinMap.count(spin) != 0) // Spin multiply listed
     {
        for (cellPt.z = zLow; cellPt.z <= zHigh; cellPt.z++)
           for (cellPt.y = yLow; cellPt.y <= yHigh; cellPt.y++)
               for (cellPt.x = xLow; cellPt.x <= xHigh; cellPt.x++)
                   cellFieldG->set(cellPt, cellFieldG->get(spinMap[spin]));
        //cerr << "2. Cell Type from cell->:  " <<(int)cell->type<< "\n";

     }
     else // First time for this spin, we need to create a new cell
     {
        spinMap[spin] = Point3D(xLow, yLow, zLow);
        cell = potts->createCellG(Point3D(xLow, yLow, zLow));
        
        for (cellPt.z = zLow; cellPt.z <= zHigh; cellPt.z++)
            for (cellPt.y = yLow; cellPt.y <= yHigh; cellPt.y++)
                for (cellPt.x = xLow; cellPt.x <= xHigh; cellPt.x++)
                    cellFieldG->set(cellPt, cell);
        

        typeTransitionPtr->setType( cell,potts->getAutomaton()->getTypeId(celltype));
        //cerr << "2. Cell Type from cell->:  " <<(int)cell->type<< "\n";
   
     }


}
        }


}

void PIFInitializer::readXML(XMLPullParser &in) {
   cerr<<"inside readXML"<<endl;
  in.skip(TEXT);
   cerr<<"inside readXML after skip(TEXT)"<<endl;

  while (in.check(START_ELEMENT)) {
    if (in.getName() == "PIFName") {
      cerr<<"got PIFName"<<endl; 
      pifname = in.matchSimple();
      cerr<<"scanned Name:"<<pifname<<endl;

    } 
    else {
      throw BasicException(string("Unexpected element '") + in.getName() + 
			   "'!", in.getLocation());
    }
   cerr<<"inside readXML while loop before skip(TEXT)"<<endl;
   in.skip(TEXT);
   cerr<<"inside readXML while loop after skip(TEXT)"<<endl;
  }


cerr<<"exiting readXML"<<endl;
}

void PIFInitializer::writeXML(XMLSerializer &out) {
}
