#include "NumericalUtils.h"
#include <cmath>
#include <CompuCell3D/Potts3D/Cell.h>
#include <CompuCell3D/Field3D/Point3D.h>
#include <iostream>
using namespace std;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
float findMin( float _d , int _dim ){

   float minD = _d ;
   
   if( fabs(_d+_dim) < fabs(minD) ){
      minD=_d+_dim;
   }

   if( fabs(_d-_dim) < fabs(minD) ){
      minD=_d-_dim;
   }

   return minD;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
double dist(double x0,double y0,double z0){
   return sqrt((x0)*(x0)+(y0)*(y0)+(z0)*(z0));
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

double dist(double x0,double y0,double z0,double x1,double y1,double z1){
   return sqrt((x0-x1)*(x0-x1)+(y0-y1)*(y0-y1)+(z0-z1)*(z0-z1));
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


namespace CompuCell3D{


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


Coordinates3D<int> precalculateCentroid(const Point3D & pt, const CellG *_cell, int _volumeIncrement,const Point3D & _fieldDim){
   Point3D shiftVec;
   Point3D shiftedPt;
   int xCM,yCM,zCM;
//    if(pt.x==0 || pt.x==_fieldDim.x-1)
//       cerr<<"_cell->xCM="<<_cell->xCM<<" volumeIncr="<<_volumeIncrement<<" pt="<<pt<<endl;

   //shift is defined to be zero vector for non-periodic b.c. - everything reduces to naive calculations then   
   shiftVec.x= (short)((_cell->xCM/(float)(_cell->volume)-_fieldDim.x/2));
   shiftVec.y= (short)((_cell->yCM/(float)(_cell->volume)-_fieldDim.y/2));
   shiftVec.z= (short)((_cell->zCM/(float)(_cell->volume)-_fieldDim.z/2));

    //shift CM to approximately center of lattice, new centroids are:
    xCM = _cell->xCM - shiftVec.x*(_cell->volume);
    yCM = _cell->yCM - shiftVec.y*(_cell->volume);
    zCM = _cell->zCM - shiftVec.z*(_cell->volume);

    //Now shift pt
    shiftedPt=pt;
    shiftedPt-=shiftVec;
//     if(pt.x==0 || pt.x==_fieldDim.x-1)
//       cerr<<"EARLY shiftedPt.x="<<shiftedPt.x<<" xCM="<<xCM<<" xcm="<<xCM/(float)_cell->volume<<endl;

    //making sure that shifted point is in the lattice
    if(shiftedPt.x < 0){
      shiftedPt.x += _fieldDim.x;
//       if(pt.x==0 || pt.x==_fieldDim.x-1)
//          cerr<<"SHIFTTING shiftedPt.x="<<shiftedPt.x<<" _fieldDim.x="<<_fieldDim.x<<endl;
    }else if (shiftedPt.x > _fieldDim.x-1){
      shiftedPt.x -= _fieldDim.x;
    }  

    if(shiftedPt.y < 0){
      shiftedPt.y += _fieldDim.y;
    }else if (shiftedPt.y > _fieldDim.y-1){
      shiftedPt.y -= _fieldDim.y;
    }  

    if(shiftedPt.z < 0){
      shiftedPt.z += _fieldDim.z;
    }else if (shiftedPt.z > _fieldDim.z-1){
      shiftedPt.z -= _fieldDim.z;
    }
    //update shifted centroids
    xCM += shiftedPt.x*(_volumeIncrement>0?1:-1);
    yCM += shiftedPt.y*(_volumeIncrement>0?1:-1);
    zCM += shiftedPt.z*(_volumeIncrement>0?1:-1);
    
    //shift back centroids
    xCM += shiftVec.x * (_cell->volume+_volumeIncrement);
    yCM += shiftVec.y * (_cell->volume+_volumeIncrement);
    zCM += shiftVec.z * (_cell->volume+_volumeIncrement);
   
//     if(pt.x==0 || pt.x==_fieldDim.x-1)
//       cerr<<"shiftedPt.x="<<shiftedPt.x<<" xCM="<<xCM<<endl;


   return Coordinates3D<int>(xCM,yCM,zCM);
}


double distInvariantCM(double x0,double y0,double z0,double x1,double y1,double z1,const Point3D & _fieldDim){

   Point3D shiftVec;
   double x0_t,y0_t,z0_t;
   double x1_t,y1_t,z1_t;

   //shift is defined to be zero vector for non-periodic b.c. - everything reduces to naive calculations then   
   shiftVec.x= (short)(x0-_fieldDim.x/2);
   shiftVec.y= (short)(y0-_fieldDim.y/2);
   shiftVec.z= (short)(z0-_fieldDim.z/2);

   //moving x0,y0,z0 to approximetely center of the lattice
   x0_t=x0-shiftVec.x;
   y0_t=y0-shiftVec.y;
   z0_t=z0-shiftVec.z;

   //shifting accordingly other coordinates
   x1_t=x1-shiftVec.x;
   y1_t=y1-shiftVec.y;
   z1_t=z1-shiftVec.z;

    //making sure that x1_t,y1_t,z1_t is in the lattice

    if(x1_t < 0){
      x1_t += _fieldDim.x;
    }else if (x1_t > _fieldDim.x-1){
      x1_t -= _fieldDim.x;
    }  

    if(y1_t < 0){
      y1_t += _fieldDim.y;
    }else if (y1_t > _fieldDim.y-1){
      y1_t -= _fieldDim.y;
    }  

    if(z1_t < 0){
      z1_t += _fieldDim.z;
    }else if (z1_t > _fieldDim.z-1){
      z1_t -= _fieldDim.z;
    }  

   return dist(x0_t,y0_t,z0_t,x1_t,y1_t,z1_t);

}


//returns invariant distance vector - v=_pt1-_pt0 that it "immune" to periodic boundary conditions

Point3D distanceVectorInvariant(const Point3D & _pt1 ,const Point3D & _pt0,const Point3D & _fieldDim){

   Point3D shiftVec;
   double x0_t,y0_t,z0_t;
   double x1_t,y1_t,z1_t;

   //shift is defined to be zero vector for non-periodic b.c. - everything reduces to naive calculations then   
   shiftVec.x= (short)(_pt0.x-_fieldDim.x/2);
   shiftVec.y= (short)(_pt0.y-_fieldDim.y/2);
   shiftVec.z= (short)(_pt0.z-_fieldDim.z/2);

   //moving x0,y0,z0 to approximetely center of the lattice
   x0_t=_pt0.x-shiftVec.x;
   y0_t=_pt0.y-shiftVec.y;
   z0_t=_pt0.z-shiftVec.z;

   //shifting accordingly other coordinates
   x1_t=_pt1.x-shiftVec.x;
   y1_t=_pt1.y-shiftVec.y;
   z1_t=_pt1.z-shiftVec.z;

    //making sure that x1_t,y1_t,z1_t is in the lattice

    if(x1_t < 0){
      x1_t += _fieldDim.x;
    }else if (x1_t > _fieldDim.x-1){
      x1_t -= _fieldDim.x;
    }  

    if(y1_t < 0){
      y1_t += _fieldDim.y;
    }else if (y1_t > _fieldDim.y-1){
      y1_t -= _fieldDim.y;
    }  

    if(z1_t < 0){
      z1_t += _fieldDim.z;
    }else if (z1_t > _fieldDim.z-1){
      z1_t -= _fieldDim.z;
    }  


   return Point3D(static_cast<short>(x1_t-x0_t),static_cast<short>(y1_t-y0_t),static_cast<short>(z1_t-z0_t));
   

}


CenterOfMassPair_t precalculateAfterFlipCM(
                  const Point3D &pt, 
                  const CellG *newCell, 
                  const CellG *oldCell,
                  const Point3D & fieldDim, 
                  const Point3D & boundaryConditionIndicator
                  )

{

   CenterOfMassPair_t centerOfMassPair;
   centerOfMassPair.first = Coordinates3D<float>(0.,0.,0.);
   centerOfMassPair.second = Coordinates3D<float>(0.,0.,0.);

   Coordinates3D<float> &newCellCM=centerOfMassPair.first;
   Coordinates3D<float> &oldCellCM=centerOfMassPair.second;

   //if no boundary conditions are present
   if ( !boundaryConditionIndicator.x && !boundaryConditionIndicator.y && !boundaryConditionIndicator.z ){

      
      if (oldCell) {
         oldCellCM.XRef()=oldCell->xCM-pt.x;
         oldCellCM.YRef()=oldCell->xCM-pt.y;
         oldCellCM.ZRef()=oldCell->xCM-pt.z;

         if(oldCell->volume>1){
            oldCellCM.XRef()=(oldCell->xCM-pt.x)/((float)oldCell->volume-1);
            oldCellCM.YRef()=(oldCell->yCM-pt.y)/((float)oldCell->volume-1);
            oldCellCM.ZRef()=(oldCell->zCM-pt.z)/((float)oldCell->volume-1);
         }else{
         
            oldCellCM.XRef()=oldCell->xCM;
            oldCellCM.YRef()=oldCell->xCM;
            oldCellCM.ZRef()=oldCell->xCM;
         
            

         }

      }

      if (newCell) {
      
         newCellCM.XRef()=(newCell->xCM+pt.x)/((float)newCell->volume+1);
         newCellCM.YRef()=(newCell->xCM+pt.y)/((float)newCell->volume+1);
         newCellCM.ZRef()=(newCell->xCM+pt.z)/((float)newCell->volume+1);
      

      }

      return centerOfMassPair;
   }

   //if there are boundary conditions defined that we have to do some shifts to correctly calculate center of mass
   //This approach will work only for cells whose span is much smaller that lattice dimension in the "periodic "direction
   //e.g. cell that is very long and "wraps lattice" will have miscalculated CM using this algorithm. On the other hand, you do not real expect
   //cells to have dimensions comparable to lattice...
   

   
   Point3D shiftVec;
   Point3D shiftedPt;
   int xCM,yCM,zCM; //temp centroids

   int x,y,z;
   int xo,yo,zo;
//     cerr<<"CM PLUGIN"<<endl;
    
  if (oldCell) {

   xo=oldCell->xCM;
   yo=oldCell->yCM;
   zo=oldCell->zCM;

        

      x=oldCell->xCM-pt.x;
      y=oldCell->yCM-pt.y;
      z=oldCell->zCM-pt.z;
    //calculating shiftVec - to translate CM


    //shift is defined to be zero vector for non-periodic b.c. - everything reduces to naive calculations then   
    shiftVec.x= (short)((oldCell->xCM/(float)(oldCell->volume)-fieldDim.x/2)*boundaryConditionIndicator.x);
    shiftVec.y= (short)((oldCell->yCM/(float)(oldCell->volume)-fieldDim.y/2)*boundaryConditionIndicator.y);
    shiftVec.z= (short)((oldCell->zCM/(float)(oldCell->volume)-fieldDim.z/2)*boundaryConditionIndicator.z);

    //shift CM to approximately center of lattice, new centroids are:
    xCM = oldCell->xCM - shiftVec.x*(oldCell->volume);
    yCM = oldCell->yCM - shiftVec.y*(oldCell->volume);
    zCM = oldCell->zCM - shiftVec.z*(oldCell->volume);
    //Now shift pt
    shiftedPt=pt;
    shiftedPt-=shiftVec;
    
    //making sure that shifterd point is in the lattice
    if(shiftedPt.x < 0){
      shiftedPt.x += fieldDim.x;
    }else if (shiftedPt.x > fieldDim.x-1){
      shiftedPt.x -= fieldDim.x;
    }  

    if(shiftedPt.y < 0){
      shiftedPt.y += fieldDim.y;
    }else if (shiftedPt.y > fieldDim.y-1){
      shiftedPt.y -= fieldDim.y;
    }  

    if(shiftedPt.z < 0){
      shiftedPt.z += fieldDim.z;
    }else if (shiftedPt.z > fieldDim.z-1){
      shiftedPt.z -= fieldDim.z;
    }
    //update shifted centroids
    xCM -= shiftedPt.x;
    yCM -= shiftedPt.y;
    zCM -= shiftedPt.z;

    //shift back centroids
    xCM += shiftVec.x * (oldCell->volume-1);
    yCM += shiftVec.y * (oldCell->volume-1);
    zCM += shiftVec.z * (oldCell->volume-1);

    //Check if CM is in the lattice
    if( xCM/((float)oldCell->volume-1) < 0){
      xCM += fieldDim.x*(oldCell->volume-1);
    }else if ( xCM/((float)oldCell->volume -1)> fieldDim.x){ //will allow to have xCM/vol slightly bigger (by 1) value than max lattice point
                                                         //to avoid rollovers for unsigned int from oldCell->xCM
                                                         

       xCM -= fieldDim.x*(oldCell->volume-1);

     
    }

    if( yCM/((float)oldCell->volume-1) < 0){
      yCM += fieldDim.y*(oldCell->volume-1);
    }else if ( yCM/((float)oldCell->volume-1) > fieldDim.y){
      yCM -= fieldDim.y*(oldCell->volume-1);
    }

    if( zCM/((float)oldCell->volume-1) < 0){
      zCM += fieldDim.z*(oldCell->volume-1);
    }else if ( zCM/((float)oldCell->volume-1) > fieldDim.z){
      zCM -= fieldDim.z*(oldCell->volume-1);
    }

   
   if(oldCell->volume>1){
      oldCellCM.XRef()=xCM/((float)oldCell->volume-1);
      oldCellCM.YRef()=yCM/((float)oldCell->volume-1);
      oldCellCM.ZRef()=zCM/((float)oldCell->volume-1);
   }else{
   
      oldCellCM.XRef()=zCM;
      oldCellCM.YRef()=yCM;
      oldCellCM.ZRef()=zCM;

   }


  }

  if (newCell) {

    xo=newCell->xCM;
    yo=newCell->yCM;
    zo=newCell->zCM;


      x=newCell->xCM+pt.x;
      y=newCell->yCM+pt.y;
      z=newCell->zCM+pt.z;

  
    if(newCell->volume==1){
      shiftVec.x=0;
      shiftVec.y=0;
      shiftVec.z=0;
    }else{
      shiftVec.x= (short)((newCell->xCM/(float)(newCell->volume)-fieldDim.x/2)*boundaryConditionIndicator.x);
      shiftVec.y= (short)((newCell->yCM/(float)(newCell->volume)-fieldDim.y/2)*boundaryConditionIndicator.y);
      shiftVec.z= (short)((newCell->zCM/(float)(newCell->volume)-fieldDim.z/2)*boundaryConditionIndicator.z);
    
      
    }
    
    //if CM of the cell is too close to the "middle" of the lattice correct shift vector

    
    //shift CM to approximately center of lattice , new centroids are:
    xCM = newCell->xCM - shiftVec.x*(newCell->volume);
    yCM = newCell->yCM - shiftVec.y*(newCell->volume);
    zCM = newCell->zCM - shiftVec.z*(newCell->volume);
    //Now shift pt
    shiftedPt=pt;
    shiftedPt-=shiftVec;

    //making sure that shifted point is in the lattice
    if(shiftedPt.x < 0){
      shiftedPt.x += fieldDim.x;
    }else if (shiftedPt.x > fieldDim.x-1){
//       cerr<<"shifted pt="<<shiftedPt<<endl;
      shiftedPt.x -= fieldDim.x;
    }  

    if(shiftedPt.y < 0){
      shiftedPt.y += fieldDim.y;
    }else if (shiftedPt.y > fieldDim.y-1){
      shiftedPt.y -= fieldDim.y;
    }  

    if(shiftedPt.z < 0){
      shiftedPt.z += fieldDim.z;
    }else if (shiftedPt.z > fieldDim.z-1){
      shiftedPt.z -= fieldDim.z;
    }    

    //update shifted centroids
    xCM += shiftedPt.x;
    yCM += shiftedPt.y;
    zCM += shiftedPt.z;
    
    //shift back centroids
    xCM += shiftVec.x * (newCell->volume+1);
    yCM += shiftVec.y * (newCell->volume+1);
    zCM += shiftVec.z * (newCell->volume+1);
    
    //Check if CM is in the lattice
    if( xCM/((float)newCell->volume+1) < 0){
      xCM += fieldDim.x*(newCell->volume+1);
    }else if ( xCM/((float)newCell->volume+1) > fieldDim.x){ //will allow to have xCM/vol slightly bigger (by 1) value than max lattice point
                                                         //to avoid rollovers for unsigned int from oldCell->xCM
      xCM -= fieldDim.x*(newCell->volume+1);
    }

    if( yCM/((float)newCell->volume+1) < 0){
      yCM += fieldDim.y*(newCell->volume+1);
    }else if ( yCM/((float)newCell->volume+1) > fieldDim.y){
      yCM -= fieldDim.y*(newCell->volume+1);
    }

    if( zCM/((float)newCell->volume+1) < 0){
      zCM += fieldDim.z*(newCell->volume+1);
    }else if ( zCM/((float)newCell->volume+1) > fieldDim.z){
      zCM -= fieldDim.z*(newCell->volume+1);
    }

   
   newCellCM.XRef()=xCM/((float)newCell->volume+1);
   newCellCM.YRef()=yCM/((float)newCell->volume+1);
   newCellCM.ZRef()=zCM/((float)newCell->volume+1);

   return centerOfMassPair;
    

  }

}


};
